package com.caimei365.order.components;

import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.utils.MathUtil;
import com.caimei365.order.utils.ImageUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ArrayUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.IntStream;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/29
 */
@Slf4j
@Service
public class ProductService {
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private BaseMapper baseMapper;

    /**
     * 设置商品图片及税费
     * @param cartItemVo 购物车商品
     * @return taxFlag 是否需要设置税费
     */
    public boolean setCartItemImgAndTax(CartItemVo cartItemVo) {
        // 图片路径
        String image = ImageUtil.getImageUrl("product", cartItemVo.getImage(), domain);
        cartItemVo.setImage(image);
        // 是否添加税费，不含税商品 开票需添加税费
        boolean taxFlag = (Integer.valueOf(0).equals(cartItemVo.getIncludedTax()) && (Integer.valueOf(1).equals(cartItemVo.getInvoiceType()) || Integer.valueOf(2).equals(cartItemVo.getInvoiceType())));
        if (taxFlag) {
            BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(cartItemVo.getPrice(), cartItemVo.getTaxRate()), 100, 2);
            BigDecimal originalTax = MathUtil.div(MathUtil.mul(cartItemVo.getOriginalPrice(), cartItemVo.getTaxRate()), 100, 2);
            cartItemVo.setPrice(MathUtil.add(cartItemVo.getPrice(), cartItemTax).doubleValue());
            cartItemVo.setOriginalPrice(MathUtil.add(cartItemVo.getOriginalPrice(), originalTax).doubleValue());
        }
        return taxFlag;
    }

    /**
     * 设置购物车阶梯价
     * @param cartItemVo
     * @param taxFlag
     */
    public void setCartLadderPrices(CartItemVo cartItemVo, boolean taxFlag) {
        // 阶梯价列表
        List<LadderPriceVo> ladderPrices = baseMapper.getLadderPriceList(cartItemVo.getProductId());
        if (!CollectionUtils.isEmpty(ladderPrices)) {
            IntStream.range(0, ladderPrices.size()).forEach(i -> {
                boolean isThisLadder;
                // 添加税费
                if (taxFlag) {
                    BigDecimal addedValueTax = MathUtil.div(MathUtil.mul(ladderPrices.get(i).getBuyPrice(), cartItemVo.getTaxRate()), 100, 2);
                    ladderPrices.get(i).setBuyPrice(MathUtil.add(ladderPrices.get(i).getBuyPrice(), addedValueTax).doubleValue());
                }
                if (i == ladderPrices.size() - 1) {
                    ladderPrices.get(i).setMaxNum(0);
                    ladderPrices.get(i).setNumRange("≥" + ladderPrices.get(i).getBuyNum());
                    isThisLadder = (cartItemVo.getNumber() >= ladderPrices.get(i).getBuyNum());
                } else {
                    ladderPrices.get(i).setMaxNum(ladderPrices.get(i + 1).getBuyNum());
                    String buyNumRangeShow = ladderPrices.get(i).getBuyNum() + "~" + (ladderPrices.get(i + 1).getBuyNum() - 1);
                    ladderPrices.get(i).setNumRange(buyNumRangeShow);
                    isThisLadder = (cartItemVo.getNumber() >= ladderPrices.get(i).getBuyNum() && cartItemVo.getNumber() <= ladderPrices.get(i).getMaxNum());
                }
                if (isThisLadder) {
                    cartItemVo.setPrice(ladderPrices.get(i).getBuyPrice());
                    cartItemVo.setOriginalPrice(ladderPrices.get(i).getBuyPrice());
                }
            });
            cartItemVo.setMin(ladderPrices.get(0).getBuyNum());
            cartItemVo.setLadderPrices(ladderPrices);
        } else {
            cartItemVo.setLadderFlag(0);
        }
    }

    /**
     * 当前促销活动的价格计算列表
     *
     * @param promotions 促销活动
     * @param cartItemVo 当前购物车商品
     * @param taxFlag    计算税费标志
     * @return 价格计算列表
     */
    public List<PromotionPriceVo> getPromotionProducts(PromotionsVo promotions, CartItemVo cartItemVo, boolean taxFlag) {
        List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
        PromotionPriceVo promotionPrice = new PromotionPriceVo();
        promotionPrice.setProductId(cartItemVo.getProductId());
        promotionPrice.setNumber(cartItemVo.getNumber());
        // 设置价格
        if (promotions.getType() == 1 && promotions.getMode() == 1) {
            // 单品优惠价添加税费
            if (taxFlag) {
                BigDecimal addedValueTax = MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), cartItemVo.getTaxRate()), BigDecimal.valueOf(100), 2);
                promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(), addedValueTax).doubleValue());
            }
            promotionPrice.setPrice(promotions.getTouchPrice());
            promotionPrice.setOriginalPrice(cartItemVo.getOriginalPrice());
            // 设置商品活动价
            cartItemVo.setPrice(promotions.getTouchPrice());
        } else {
            promotionPrice.setPrice(cartItemVo.getPrice());
        }
        promotionPriceList.add(promotionPrice);
        return promotionPriceList;
    }

    /**
     * 把当前促销更新购物车总促销
     * @param totalPromotions 总促销列表
     * @param promotionsIds   总促销Id集合
     * @param promotions      当前促销
     * @param promotionPriceList  当前促销商品价格计算列表
     */
    public void updateTotalPromotions(List<PromotionsVo> totalPromotions, List<Integer> promotionsIds, PromotionsVo promotions, List<PromotionPriceVo> promotionPriceList) {
        if (promotionsIds.contains(promotions.getId())) {
            // 列表已有该促销活动
            totalPromotions.forEach(item -> {
                if (item.getId().equals(promotions.getId())) {
                    promotionPriceList.addAll(item.getProductList());
                    item.setProductList(promotionPriceList);
                }
            });
        } else {
            // 新的促销活动
            promotionsIds.add(promotions.getId());
            if (promotions.getMode() == 3) {
                // 获取赠品
                List<CartItemVo> giftList = baseMapper.getPromotionGifts(promotions.getId());
                promotions.setGiftList(giftList);
            }
            promotions.setProductList(promotionPriceList);
            // 添加到总促销
            totalPromotions.add(promotions);
        }
    }

    /**
     * 是否订单充值商品或定金商品
     * @param productId 商品id
     * @return Boolean
     */
    public boolean isRechargeProduct(Integer productId){
        int[] products = {6060, 6061, 6062, 6063, 6064, 6065, 6066, 6067, 6068, 6069};
        return ArrayUtils.contains(products, productId);
    }

    /**
     * 计算运费
     * @param userId     用户ID
     * @param townId     地区Id
     * @param productIdList 商品Id列表
     */
    public Map<String, Object> computePostage(Integer userId, Integer townId, List<String> productIdList) {
        // 返回数据初始化
        Map<String, Object> postageMap = new HashMap<>(2);
        // 运费标志：0包邮 -1到付 1遵循运费规则
        int postageFlag = 0;
        // 运费
        Double postage = 0.00d;
        // 可用采美豆
        Integer userBeans = baseMapper.getUserBeans(userId);
        postageMap.put("userBeans", userBeans);
        // 获取商品运费 （商品数据库 0包邮 1到付 2默认（遵循运费规则））
        List<ProductPostageVo> postageFlagList = baseMapper.getPostageFlagList(productIdList);
        if (null != postageFlagList && postageFlagList.size() > 0) {
            for (ProductPostageVo postageVo : postageFlagList){
                if (null != postageVo) {
                    // 是否是仪器 或 设置了运费到付
                    boolean flag = (null == postageVo.getCommodityType() || 2 == postageVo.getCommodityType() || (null != postageVo.getPostageFlag() && 1 == postageVo.getPostageFlag()));
                    if (flag) {
                        // 到付
                        postageMap.put("postageFlag", -1);
                        postageMap.put("postage", 0.00d);
                        return postageMap;
                    } else if (null != postageVo.getPostageFlag() && 2 == postageVo.getPostageFlag()) {
                        // 若有不包邮商品，则全部不包邮（商品数据库：2默认（遵循运费规则），这里返回前端(按照订单数据库)：1遵循运费规则）
                        postageFlag = 1;
                    }
                }
            }
        }
        // 是否首单（统计订单数）
        Integer count = baseMapper.countUserOrder(userId);
        // 包邮条件：首单 或 包邮（上一步未被设置成不包邮）
        boolean freeFlag = ((null != count && count == 0) || postageFlag == 0);
        if (freeFlag) {
            // 包邮
            postageMap.put("postageFlag", 0);
            postageMap.put("postage", 0.00d);
            return postageMap;
        } else {
            /*
             * 运费计算：广东省内运费15元,深圳市内运费10,其他地区到付
             */
            postageFlag = 1;
            // 获取根据地区Id获取省市地址信息
            AddressVo address = baseMapper.getProvinceIdAndCityId(townId);
            if (null != address && 202 == address.getCityId()) {
                // 深圳市内运费10
                postage = 10.00d;
            } else if (null != address && 19 == address.getProvinceId()){
                // 广东省内运费15元
                postage = 15.00d;
            } else {
                // 到付
                postageFlag = -1;
            }
            postageMap.put("postageFlag", postageFlag);
            postageMap.put("postage", postage);
            return postageMap;
        }
    }

    /**
     * 更新复购价格库
     * @param order         主订单
     * @param orderProduct  订单商品
     * @param secondFlag    是否二手商品
     * @param note          日志备注
     */
    public void savePurchasePrice(OrderPo order, OrderProductPo orderProduct, boolean secondFlag, String note) {
        // 非二手订单, 非活动商品，非阶梯价商品，非运费商品，非赠品 才可以写入复购价
        boolean purchaseFlag = ((!secondFlag) && Integer.valueOf(0).equals(orderProduct.getActProduct()) && orderProduct.getShopId() != 998 && orderProduct.getPrice() > 0);
        boolean historyFlag = false;
        // 不是二手订单才可以写入价格库
        if (purchaseFlag) {
            Date date = new Date();
            String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
            // 机构价大于成本价   ---成本价已经在整理订单商品时计算好（固定或比例）
            boolean costFlag = MathUtil.compare(orderProduct.getPrice(), orderProduct.getCostPrice()) > 0;
            //  查询当前商品复购价信息
            PurchasePricePo purchase = baseMapper.getPurchasePricePo(order.getUserId(), orderProduct.getProductId());
            // 已有复购价
            if (null != purchase) {
                // 复购价大于机构价（降价）
                boolean priceFlag1 = MathUtil.compare(purchase.getCurrentPrice(), orderProduct.getPrice()) > 0;
                // 成本价大于复购价（亏本）
                boolean priceFlag2 = MathUtil.compare(orderProduct.getCostPrice(), purchase.getCurrentPrice()) > 0;
                if (priceFlag2 || (priceFlag1 && costFlag)) {
                    // 更新价格库复购价
                    purchase.setCurrentPrice(orderProduct.getPrice());
                    purchase.setTaxRate(orderProduct.getTaxRate());
                    purchase.setUpdateTime(curDateStr);
                    // 更新复购价格库
                    baseMapper.updatePurchasePrice(purchase);
                    historyFlag = true;
                    log.info(note + ">>>>>>>>>>>更新复购价格库(update[repeat_purchase_price])ProductId：" + orderProduct.getProductId() + "，orderId：" + order.getOrderId());
                }
            } else {
                // 没有复购价
                if (costFlag) {
                    // 新增复购价到价格库
                    purchase = new PurchasePricePo();
                    purchase.setUserId(order.getUserId());
                    purchase.setClubId(order.getClubId());
                    purchase.setOrderId(order.getOrderId());
                    purchase.setProductId(orderProduct.getProductId());
                    purchase.setShopId(orderProduct.getShopId());
                    purchase.setShopName(orderProduct.getShopName());
                    purchase.setCurrentPrice(orderProduct.getPrice());
                    purchase.setTaxRate(orderProduct.getTaxRate());
                    purchase.setCreateTime(curDateStr);
                    purchase.setUpdateTime(curDateStr);
                    purchase.setDelFlag(0);
                    // 新增复购价格库
                    baseMapper.insertPurchasePrice(purchase);
                    historyFlag = true;
                    log.info(note + ">>>>>>>>>>>新增复购价格库(insert[repeat_purchase_price])ProductId：" + orderProduct.getProductId() + "，orderId：" + order.getOrderId());
                }
            }
            if (historyFlag) {
                // 历史复购价记录
                PurchaseHistoryPo purchaseHistory = new PurchaseHistoryPo();
                purchaseHistory.setUserId(order.getUserId());
                purchaseHistory.setClubId(order.getClubId());
                purchaseHistory.setOrderId(order.getOrderId());
                purchaseHistory.setProductId(orderProduct.getProductId());
                purchaseHistory.setCurrentCostPrice(orderProduct.getCostPrice());
                purchaseHistory.setPrice(orderProduct.getPrice());
                purchaseHistory.setCreateTime(curDateStr);
                purchaseHistory.setDelFlag(0);
                baseMapper.insertPurchaseHistory(purchaseHistory);
                log.info(note + ">>>>>>>>>>>新增历史复购价记录(insert[repeat_purchase_price_history])ProductId：" + orderProduct.getProductId() + "，orderId：" + order.getOrderId());
            }
        }
    }

    /**
     * 保存余额抵扣到收款记录
     * @param balancePayFee 余额抵扣
     * @param orderId       主订单Id
     * @param note          日志备注
     */
    public void saveDiscernReceipt(Double balancePayFee, Integer orderId, String note) {
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        // 识别款项表
        DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
        discernReceipt.setPayWay(3);
        discernReceipt.setPayType(16);
        discernReceipt.setReceiptType(1);
        discernReceipt.setReceiptStatus(3);
        discernReceipt.setReceiptAmount(balancePayFee);
        discernReceipt.setConfirmType(4);
        discernReceipt.setReceiptDate(curDateStr);
        discernReceipt.setConfirmDate(curDateStr);
        discernReceipt.setReviewDate(curDateStr);
        discernReceipt.setUpdateDate(curDateStr);
        discernReceipt.setDelFlag(0);
        // 保存 收款记录
        baseMapper.insertDiscernReceipt(discernReceipt);
        log.info(note + ">>>>>>>>>>>保存识别款项(insert[cm_discern_receipt])id：" + discernReceipt.getId() + "，orderId：" + orderId);
        // 收款项和订单关系表
        OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
        relation.setReceiptId(discernReceipt.getId());
        relation.setOrderId(orderId);
        relation.setAssociateAmount(balancePayFee);
        relation.setRelationType(2);
        relation.setDelFlag(0);
        // 保存 收款项和订单关系
        baseMapper.insertOrderReceiptRelation(relation);
        log.info(note + ">>>>>>>>>>>收款项和订单关系(insert[cm_receipt_order_relation])id：" + relation.getId() + "，orderId：" + orderId);
    }

    /**
     * 更新用户采美豆
     * @param beansHistory  采美豆使用记录
     * @param userBeans     用户原有采美豆数量
     * @param note          日志备注
     */
    public void updateUserBeans(UserBeansHistoryPo beansHistory, Integer userBeans, String note) {
        // 保存 采美豆使用记录
        baseMapper.insertBeansHistory(beansHistory);
        baseMapper.updateUserBeans(beansHistory.getUserId(), userBeans);
        log.info(note + ">>>>>更新用户采美豆(update[user(userBeans:"+ userBeans +")])，userId:" + beansHistory.getUserId());
    }

    /**
     * 设置超级会员优惠商品价格
     * @param cartItemVo    当前购物车商品
     * @param taxFlag       计算税费标志
     * @param svipUserFlag
     */
    public void setSvipProductPrice(CartItemVo cartItemVo, boolean taxFlag, boolean svipUserFlag) {
        Integer svipPriceType = cartItemVo.getSvipPriceType();
        Double discountPrice = 0d;
        if (1 == svipPriceType) {
            // 折扣价
            cartItemVo.setSvipPriceTag(MathUtil.div(cartItemVo.getSvipDiscount(), 10, 1) + "折");
            // 设置折扣价（原价格已计算税费，无需重复计算）
            discountPrice = MathUtil.div(MathUtil.mul(cartItemVo.getPrice(), cartItemVo.getSvipDiscount()), 100, 2).doubleValue();
        } else if (2 == svipPriceType) {
            // 直接优惠价
            discountPrice = cartItemVo.getSvipDiscountPrice();
            if (taxFlag) {
                //添加税费
                BigDecimal thisTaxFee = MathUtil.div(MathUtil.mul(discountPrice, cartItemVo.getTaxRate()), 100, 2);
                discountPrice = MathUtil.add(discountPrice, thisTaxFee).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            }
            cartItemVo.setSvipPriceTag("￥" + discountPrice);
        }
        if (svipUserFlag) {
            // 超级会员设置优惠价
            cartItemVo.setPrice(discountPrice);
            // 设置优惠金额=（商品原价-折扣价）*商品数量
            cartItemVo.setSvipTotalReducedPrice(MathUtil.mul(MathUtil.sub(cartItemVo.getOriginalPrice(), discountPrice), cartItemVo.getNumber()).doubleValue());
        }
    }
}
