package com.caimei365.order.controller;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.CartDto;
import com.caimei365.order.model.dto.InvoiceDto;
import com.caimei365.order.model.vo.InvoiceVo;
import com.caimei365.order.service.CartClubService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 机构购物车API
 *
 * @author : Charles
 * @date : 2021/6/25
 */
@Api(tags="机构购物车API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/club")
public class CartClubApi {

    private final CartClubService cartClubService;

    /**
     * 购物车列表详细数据
     */
    @ApiOperation("购物车列表详细数据（旧：/shoppingCart/list）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "source", value = "来源 : 1 网站 ; 2 小程序")
    })
    @GetMapping("/cart/list")
    public ResponseJson<Map<String,Object>> getShoppingCartList(Integer userId, Integer source) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return cartClubService.getShoppingCartList(userId, source);
    }

    /**
     * 网站顶部购物车数据
     */
    @ApiOperation("网站顶部购物车数据（旧：/shoppingCart/header/cart）")
    @ApiImplicitParam(required = true, name = "userId", value = "用户Id")
    @GetMapping("/cart/head")
    public ResponseJson<Map<String, Object>> getShoppingCartHead(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return cartClubService.getShoppingCartHead(userId);
    }

    /**
     * 添加购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     * }
     */
    @ApiOperation("添加购物车（旧：/shoppingCart/addCart）")
    @PostMapping("/cart/add")
    public ResponseJson<Integer> addShoppingCart(CartDto cartDto){
        if (null == cartDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == cartDto.getProductId()) {
            return ResponseJson.error("商品Id不能为空！", null);
        }
        if (null == cartDto.getProductCount()) {
            return ResponseJson.error("商品数量不能为空！", null);
        }
        return cartClubService.addShoppingCart(cartDto);
    }

    /**
     * 更新购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     * }
     */
    @ApiOperation("更新购物车（旧：/shoppingCart/update）")
    @PostMapping("/cart/update")
    public ResponseJson<Integer> updateShoppingCart(CartDto cartDto){
        if (null == cartDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == cartDto.getProductId()) {
            return ResponseJson.error("商品Id不能为空！", null);
        }
        if (null == cartDto.getProductCount()) {
            return ResponseJson.error("商品数量不能为空！", null);
        }
        return cartClubService.updateShoppingCart(cartDto);
    }

    /**
     * 删除购物车
     *
     * @param cartDto {
     *                userId       用户ID
     *                productIds   商品ids,逗号隔开
     * }
     */
    @ApiOperation("删除购物车（旧：/shoppingCart/delete）")
    @PostMapping("/cart/delete")
    public ResponseJson<Integer> deleteShoppingCart(CartDto cartDto){
        if (null == cartDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (StringUtils.isEmpty(cartDto.getProductIds())) {
            return ResponseJson.error("商品Id集合不能为空！", null);
        }
        return cartClubService.deleteShoppingCart(cartDto);
    }

    /**
     * 购物车结算
     */
    @ApiOperation("购物车结算（旧：/order/confirm）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "productIds", value = "商品ids,逗号隔开"),
            @ApiImplicitParam(required = false, name = "source", value = "来源 : 1 网站 ; 2 小程序")
    })
    @GetMapping("/cart/settlement")
    public ResponseJson<Map<String, Object>> settlementShoppingCart(Integer userId, String productIds, Integer source) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (StringUtils.isEmpty(productIds)) {
            return ResponseJson.error("商品Id集合不能为空！", null);
        }
        return cartClubService.settlementShoppingCart(userId, productIds, source);
    }

    /**
     * 立即购买
     *
     * @param cartDto {
     *                userId       用户ID
     *                productId    商品id
     *                productCount 商品数量
     *                source       来源 : 1 网站 ; 2 小程序
     * }
     */
    @ApiOperation("立即购买（旧：/order/confirm）")
    @GetMapping("/product/settlement")
    public ResponseJson<Map<String, Object>> buyNowProduct(CartDto cartDto){
        if (null == cartDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == cartDto.getProductId()) {
            return ResponseJson.error("商品Id不能为空！", null);
        }
        if (null == cartDto.getProductCount()) {
            return ResponseJson.error("商品数量不能为空！", null);
        }
        return cartClubService.buyNowProduct(cartDto);
    }

    /**
     * 获取结算商品运费
     */
    @ApiOperation("计算运费（旧：/order/postage）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "productIds", value = "商品ids,逗号隔开"),
            @ApiImplicitParam(required = false, name = "townId", value = "地区Id")
    })
    @GetMapping("/postage")
    public ResponseJson<Map<String, Object>> orderPostage(Integer userId, String productIds, Integer townId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (StringUtils.isEmpty(productIds)) {
            return ResponseJson.error("商品Id集合不能为空！", null);
        }
        if (null == townId) {
            return ResponseJson.error("地区Id不能为空！", null);
        }
        return cartClubService.getProductsPostage(userId, productIds, townId);
    }
    /**
     * 获取用户发票信息
     */
    @ApiOperation("获取用户发票信息（旧：/personalCenter/findInvoice）")
    @ApiImplicitParam(required = false, name = "userId", value = "用户Id")
    @GetMapping("/invoice")
    public ResponseJson<InvoiceVo> getUserInvoice(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return cartClubService.getUserInvoice(userId);
    }

    /**
     * 更新用户发票
     *
     * @param invoiceDto {
     *                   userId:用户Id
     *                   id:id
     *                   invoiceTitle:单位名，发票抬头
     *                   corporationTaxNum: 企业税号、纳税人识别号
     *                   registeredAddress:注册地址
     *                   registeredPhone:注册电话
     *                   bankAccountNo:开户银行账户
     *                   openBank:开户银行
     *                   }
     */
    @ApiOperation("更新用户发票（旧：/personalCenter/invoice）")
    @PostMapping("/invoice/update")
    public ResponseJson<Integer> updateUserInvoice(InvoiceDto invoiceDto){
        if (null == invoiceDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return cartClubService.updateUserInvoice(invoiceDto);
    }

    /**
     * 供应商优惠券
     *
     * @param userId 用户id
     * @param shopId 供应商id
     * @param source 来源 : 1 网站 ; 2 小程序
     * @param status 状态: 1未领取 2已领取
     */
    @ApiOperation("领取供应商优惠券（旧：/shoppingCart/get/coupon）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "shopId", value = "供应商Id"),
            @ApiImplicitParam(required = false, name = "status", value = "状态: 1未领取, 2已领取"),
            @ApiImplicitParam(required = false, name = "source", value = "来源: 1网站, 2小程序")
    })
    @GetMapping("/coupon")
    public ResponseJson<Map<String, Object>>  getShopCoupons(Integer userId, Integer shopId, Integer source,
                                   @RequestParam(name = "status", defaultValue = "1") Integer status) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == shopId) {
            return ResponseJson.error("供应商Id不能为空！", null);
        }
        if (null == source) {
            return ResponseJson.error("来源source不能为空！", null);
        }
        return cartClubService.getShopCoupons(userId, shopId, source, status);
    }
}
