package com.caimei365.order.controller;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.AgainBuyDto;
import com.caimei365.order.model.dto.SellerCartDto;
import com.caimei365.order.service.CartSellerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 协销购物车API
 *
 * @author : Charles
 * @date : 2021/6/28
 */
@Api(tags="协销购物车API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/seller")
public class CartSellerApi {

    private final CartSellerService cartSellerService;

    /**
     * 协销购物车列表数据
     */
    @ApiOperation("协销购物车列表数据（旧：/seller/shoppingInfo）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "serviceProviderId", value = "协销Id"),
            @ApiImplicitParam(required = false, name = "clubId", value = "机构Id"),
            @ApiImplicitParam(required = false, name = "againBuyProductIds", value = "再来一单商品Ids"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/cart/list")
    public ResponseJson<Map<String,Object>> getShoppingCarts(Integer serviceProviderId, Integer clubId, String againBuyProductIds,
                                                                @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (null == serviceProviderId) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (null == clubId) {
            return ResponseJson.error("机构Id不能为空！", null);
        }
        return cartSellerService.getSellerCartList(serviceProviderId, clubId, againBuyProductIds, pageNum, pageSize);
    }

    /**
     * 获取协销购物车商品种类
     */
    @ApiOperation("获取协销购物车数量（旧：/seller/productNum）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "serviceProviderId", value = "协销Id"),
            @ApiImplicitParam(required = false, name = "clubId", value = "机构Id")
    })
    @GetMapping("/cart/count")
    public ResponseJson<Integer> getSellerCarts(Integer serviceProviderId, Integer clubId) {
        if (null == serviceProviderId) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (null == clubId) {
            return ResponseJson.error("机构Id不能为空！", null);
        }
        return cartSellerService.getSellerCartCount(serviceProviderId, clubId);
    }

    /**
     * 协销添加购物车
     *
     * @param sellerCartDto {
     *                      serviceProviderId 协销Id
     *                      clubId       机构ID
     *                      type         商品类型：1单一商品，2组合商品
     *                      productId    单一商品id
     *                      productCount 单一商品数量
     *                      productInfo  组合商品信息：Json字符串格式[{"id":4351,"count":1},{}]
     * }
     */
    @ApiOperation("协销添加购物车（旧：/seller/addCart）（/seller/batchAddCart）")
    @PostMapping("/cart/add")
    public ResponseJson<Integer> addSellerCart(SellerCartDto sellerCartDto){
        if (null == sellerCartDto.getServiceProviderId()) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (null == sellerCartDto.getClubId()) {
            return ResponseJson.error("机构Id不能为空！", null);
        }
        if (null == sellerCartDto.getType()) {
            return ResponseJson.error("商品类型(单一/组合)不能为空！", null);
        }
        if (1 == sellerCartDto.getType()) {
            if (null == sellerCartDto.getProductId()) {
                return ResponseJson.error("单一商品Id不能为空！", null);
            }
            if (null == sellerCartDto.getProductCount()) {
                return ResponseJson.error("单一商品数量不能为空！", null);
            }
        } else if (2 == sellerCartDto.getType()) {
            if (StringUtils.isEmpty(sellerCartDto.getProductInfo())) {
                return ResponseJson.error("组合商品信息不能为空！", null);
            }
        } else {
            return ResponseJson.error("商品类型(1单一/2组合)不正确！", null);
        }
        return cartSellerService.addShoppingCart(sellerCartDto);
    }

    /**
     * 协销更新购物车
     *
     * @param sellerCartDto {
     *                      id                购物车Id
     *                      serviceProviderId 协销Id
     *                      productCount        商品数量
     *                      }
     */
    @ApiOperation("协销更新购物车（旧：/seller/addProductNum）")
    @PostMapping("/cart/update")
    public ResponseJson<Integer> updateSellerCart(SellerCartDto sellerCartDto){
        if (null == sellerCartDto.getServiceProviderId()) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (null == sellerCartDto.getId()) {
            return ResponseJson.error("购物车Id不能为空！", null);
        }
        if (null == sellerCartDto.getProductCount()) {
            return ResponseJson.error("商品数量不能为空！", null);
        }
        return cartSellerService.updateSellerCart(sellerCartDto);
    }

    /**
     * 协销删除购物车
     *
     * @param sellerCartDto {
     *                      serviceProviderId 协销Id
     *                      cartIds           购物车IDs
     * }
     */
    @ApiOperation("协销删除购物车（旧：/seller/deleteSellerCart）")
    @PostMapping("/cart/delete")
    public ResponseJson<Integer> deleteSellerCart(SellerCartDto sellerCartDto){
        if (null == sellerCartDto.getServiceProviderId()) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (StringUtils.isEmpty(sellerCartDto.getCartIds())) {
            return ResponseJson.error("购物车Id集合不能为空！", null);
        }
        return cartSellerService.deleteSellerCart(sellerCartDto);
    }

    /**
     * 协销购物车结算
     */
    @ApiOperation("协销购物车结算（旧：/seller/settlement）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "serviceProviderId", value = "协销Id"),
            @ApiImplicitParam(required = false, name = "clubId", value = "机构Id"),
            @ApiImplicitParam(required = false, name = "productIds", value = "商品ids,逗号隔开")
    })
    @GetMapping("/cart/settlement")
    public ResponseJson<Map<String, Object>> settlementSellerCart(Integer serviceProviderId, Integer clubId, String productIds) {
        if (null == serviceProviderId) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (null == clubId) {
            return ResponseJson.error("机构Id不能为空！", null);
        }
        if (StringUtils.isEmpty(productIds)) {
            return ResponseJson.error("商品Id集合不能为空！", null);
        }
        return cartSellerService.settlementSellerCart(serviceProviderId, clubId, productIds);
    }

    /**
     * 协销二手商品立即购买
     */
    @ApiOperation("协销二手商品立即购买（旧：/seller/settlementBySencondProduct）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "serviceProviderId", value = "协销Id"),
            @ApiImplicitParam(required = false, name = "clubId", value = "机构Id"),
            @ApiImplicitParam(required = false, name = "productId", value = "商品Id"),
            @ApiImplicitParam(required = false, name = "productCount", value = "商品数量")
    })
    @GetMapping("/second/settlement")
    public ResponseJson<Map<String, Object>> buyNowProduct(Integer serviceProviderId, Integer clubId, Integer productId, Integer productCount){
        if (null == serviceProviderId) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (null == clubId) {
            return ResponseJson.error("购物车Id不能为空！", null);
        }
        if (null == productId) {
            return ResponseJson.error("商品Id不能为空！", null);
        }
        if (null == productCount) {
            return ResponseJson.error("商品数量不能为空！", null);
        }
        return cartSellerService.settlementSecond(serviceProviderId, clubId, productId, productCount);
    }

    /**
     * 协销再来一单（订单商品加入购物车）
     *
     * @param againBuyDto {
     *                    serviceProviderId      协销Id
     *                    orderId                订单Id
     *                    confirmFlag            确认标识 0未确认 1已确认-将失效商品以外的正常商品加入购物车
     *                    }
     */
    @ApiOperation("协销再来一单(订单商品加入购物车)（旧：/seller/order/again）")
    @PostMapping("/cart/again")
    public ResponseJson<Map<String, Object>> addCartBuyAgain(AgainBuyDto againBuyDto){
        if (null == againBuyDto.getServiceProviderId()) {
            return ResponseJson.error("协销Id不能为空！", null);
        }
        if (null == againBuyDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return cartSellerService.addCartBuyAgain(againBuyDto);
    }


}
