package com.caimei365.order.controller;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.OrderDto;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.OrderClubService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 机构订单API
 *
 * @author : Charles
 * @date : 2021/7/19
 */
@Api(tags="机构订单API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/club")
public class OrderClubApi {

    private final OrderClubService orderClubService;

    /**
     * 机构订单列表
     */
    @ApiOperation("机构订单列表（旧：/order/list）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "orderState", value = "1:待确认,2:待付款,3:待发货,4:已发货,5:退货款"),
            @ApiImplicitParam(required = false, name = "orderNo", value = "订单编号"),
            @ApiImplicitParam(required = false, name = "beginTime", value = "开始时间"),
            @ApiImplicitParam(required = false, name = "endTime", value = "截止时间"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/list")
    public ResponseJson<PageInfo<OrderVo>> getOrderList(Integer userId, Integer orderState, String orderNo, String beginTime, String endTime,
                                                        @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                        @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return orderClubService.getOrderList(userId, orderState, orderNo, beginTime, endTime, pageNum, pageSize);
    }

    /**
     * 根据关键词搜索订单
     */
    @ApiOperation("根据关键词搜索订单（旧：/order/search）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "searchWord", value = "搜索关键词"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/search")
    public ResponseJson<PageInfo<OrderVo>> getOrderListByKeyword(Integer userId, String searchWord,
                                                        @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                        @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (StringUtils.isEmpty(searchWord)) {
            return ResponseJson.error("请输入搜索关键词！", null);
        }
        return orderClubService.getOrderListByKeyword(userId, searchWord, pageNum, pageSize);
    }

    /**
     * 机构订单详情
     */
    @ApiOperation("机构订单详情（旧：/order/detail）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = true, name = "orderId", value = "订单Id")
    })
    @GetMapping("/detail")
    public ResponseJson<Map<String, Object>> getOrderDetail(Integer userId, Integer orderId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == orderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return orderClubService.getOrderDetail(userId, orderId);
    }

    /**
     * 机构确认订单
     *
     * @param orderDto {
     *                orderId      订单ID
     * }
     */
    @ApiOperation("机构确认订单（旧：/order/affirmOrder）")
    @PostMapping("/confirm")
    public ResponseJson<Void> confirmOrder(OrderDto orderDto){
        if (null == orderDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return orderClubService.confirmMainOrder(orderDto.getOrderId());
    }

    /**
     * 机构取消订单
     *
     * @param orderDto {
     *                orderId      订单ID
     *                remark       备注（订单取消原因）
     * }
     */
    @ApiOperation("机构取消订单（旧：/order/cancel）")
    @PostMapping("/cancel")
    public ResponseJson<Void> cancelOrder(OrderDto orderDto){
        if (null == orderDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return orderClubService.cancelMainOrder(orderDto.getOrderId(), orderDto.getRemark());
    }

    /**
     * 机构删除订单
     *
     * @param orderDto {
     *                orderId      订单ID
     * }
     */
    @ApiOperation("机构删除订单（旧：/order/delete）")
    @PostMapping("/delete")
    public ResponseJson<Void> deleteOrder(OrderDto orderDto){
        if (null == orderDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return orderClubService.deleteMainOrder(orderDto.getOrderId());
    }

    /**
     * 机构确认收货
     *
     * @param orderDto {
     *                orderId      订单ID
     * }
     */
    @ApiOperation("机构确认收货（旧：/order/affirm）")
    @PostMapping("/receive")
    public ResponseJson<Void> receiveGoods(OrderDto orderDto){
        if (null == orderDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return orderClubService.receiveGoods(orderDto.getOrderId());
    }

    /**
     * 根据用户Id查找订单搜索历史记录
     */
    @ApiOperation("根据用户Id查找订单搜索历史记录（旧：/order/searchHistory）")
    @ApiImplicitParam(required = true, name = "userId", value = "用户Id")
    @GetMapping("/search/history")
    public ResponseJson<List<SearchHistoryVo>> getOrderSearchHistory(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return orderClubService.getOrderSearchHistory(userId);
    }

    /**
     * 根据用户Id删除订单搜索历史记录
     */
    @ApiOperation("根据用户Id删除订单搜索历史记录（旧：/order/searchHistory/delete）")
    @ApiImplicitParam(required = true, name = "userId", value = "用户Id")
    @GetMapping("/search/history/delete")
    public ResponseJson<Void> deleteOrderSearchHistory(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return orderClubService.deleteOrderSearchHistory(userId);
    }

    /**
     * 物流详情
     */
    @ApiOperation("物流详情（旧：/order/logistics）")
    @ApiImplicitParam(required = true, name = "orderId", value = "订单Id")
    @GetMapping("/logistics")
    public ResponseJson<List<LogisticsBatchVo>> getOrderLogistics(Integer orderId) {
        if (null == orderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return orderClubService.getOrderLogistics(orderId);
    }

    /**
     * 分享订单，获取分享码
     */
    @ApiOperation("分享订单，获取分享码（旧：/order/share）")
    @ApiImplicitParam(required = true, name = "orderId", value = "订单Id")
    @GetMapping("/share/code")
    public ResponseJson<String> getOrderShareCode(Integer orderId) {
        if (null == orderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return orderClubService.getOrderShareCode(orderId);
    }

    /**
     * 分享订单，用户身份
     */
    @ApiOperation("分享订单，用户身份（旧：/order/shareCode）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "orderId", value = "订单Id"),
            @ApiImplicitParam(required = false, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = false, name = "serviceProviderId", value = "协销Id"),
            @ApiImplicitParam(required = false, name = "code", value = "微信code"),
            @ApiImplicitParam(required = false, name = "encryptedData", value = "微信加密数据"),
            @ApiImplicitParam(required = false, name = "iv", value = "微信加密算法的初始向量")
    })
    @GetMapping("/share/code/identity")
    public ResponseJson<String> identityOrderShareCode( Integer orderId, Integer userId, Integer serviceProviderId, String code, String encryptedData, String iv, @RequestHeader HttpHeaders headers) {
        //head可能报错，不确定原因，版本回退

        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == orderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        if (StringUtils.isEmpty(code)) {
            return ResponseJson.error("微信code不能为空！", null);
        }
        return orderClubService.identityOrderShareCode(orderId, userId, serviceProviderId, code, encryptedData, iv, headers);
    }

    /**
     * 分享订单，校验分享码
     */
    @ApiOperation("分享订单，校验分享码（旧：/order/shareCode）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "orderId", value = "订单Id"),
            @ApiImplicitParam(required = false, name = "shareCode", value = "分享码"),
            @ApiImplicitParam(required = false, name = "code", value = "微信code"),
            @ApiImplicitParam(required = false, name = "encryptedData", value = "微信加密数据"),
            @ApiImplicitParam(required = false, name = "iv", value = "微信加密算法的初始向量")
    })
    @GetMapping("/share/code/check")
    public ResponseJson<String> checkOrderShareCode( Integer orderId, String shareCode, String code, String encryptedData, String iv, @RequestHeader HttpHeaders headers) {
        if (null == orderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        if (StringUtils.isEmpty(shareCode)) {
            return ResponseJson.error("分享码不能为空！", null);
        }
        if (StringUtils.isEmpty(code)) {
            return ResponseJson.error("微信code不能为空！", null);
        }
        return orderClubService.checkOrderShareCode(orderId, shareCode, code, encryptedData, iv, headers);
    }


    /**
     * 分享订单，初始商品数据
     */
    @ApiOperation("分享订单，初始商品数据（旧：/order/commodityData）")
    @ApiImplicitParam(required = true, name = "orderId", value = "订单Id")
    @GetMapping("/share/product")
    public ResponseJson<List<OrderProductVo>> getOrderShareInitProduct(Integer orderId) {
        if (null == orderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return orderClubService.getOrderShareInitProduct(orderId);
    }

    /**
     * 二手订单 确认打款供应商
     *
     * @param orderDto {
     *                orderId      订单ID
     *                userId       用户Id
     * }
     */
    @ApiOperation("二手订单,确认打款供应商（旧：/order/affirm/payment）")
    @PostMapping("/second/payment/confirm")
    public ResponseJson<Void> secondPaymentConfirm(OrderDto orderDto){
        if (null == orderDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == orderDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return orderClubService.secondPaymentConfirm(orderDto.getOrderId(), orderDto.getUserId());
    }

    /**
     * 根据售后条款ID查看售后条款内容
     */
    @ApiOperation("根据售后条款ID查看售后条款内容（旧：/seller/clause）")
    @ApiImplicitParam(required = false, name = "clauseId", value = "条款Id")
    @GetMapping("/clause")
    public ResponseJson<ClauseVo> getClauseById(Integer clauseId){
        if (null == clauseId) {
            return ResponseJson.error("售后条款Id不能为空！", null);
        }
        return orderClubService.getClauseById(clauseId);
    }


}
