package com.caimei365.order.controller;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.PaySecondDto;
import com.caimei365.order.model.dto.PayVipDto;
import com.caimei365.order.service.PayNonOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;

import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;

/**
 * 二手发布支付，升级超级会员支付
 *
 * @author : Charles
 * @date : 2021/9/27
 */
@Api(tags="其他(非订单)支付API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/pay")
public class PayNonOrderApi {

    private final PayNonOrderService payNonOrderService;

    /**
     * 二手发布-微信线上支付
     * @param paySecondDto {
     *               productId      二手发布商品id
     *               returnUrl     页面回调地址
     *               code          微信小程序code
     *               state         微信公众号state参数
     * }
     */
    @ApiOperation("二手发布-微信线上支付（旧：/PayOrder/appletsSecondHandPay）(/PayOrder/secondHandPay[WEIXIN])")
    @PostMapping("/second/wechat")
    public ResponseJson<JSONObject> paySecondByWeChat(PaySecondDto paySecondDto, @RequestHeader HttpHeaders headers){
        if (null == paySecondDto.getProductId()) {
            return ResponseJson.error("二手商品Id不能为空！", null);
        }
        if (StringUtils.isEmpty(paySecondDto.getCode())) {
            return ResponseJson.error("微信code不能为空！", null);
        }
        return payNonOrderService.paySecondByWeChat(paySecondDto, headers);
    }

    /**
     * 二手发布-支付宝线上支付
     * @param paySecondDto {
     *               productId      二手发布商品id
     *               returnUrl     页面回调地址
     * }
     */
    @ApiOperation("二手发布-支付宝线上支付（旧：/PayOrder/secondHandPay[ALIPAY]）")
    @PostMapping("/second/alipay")
    public ResponseJson<JSONObject> paySecondByAlipay(PaySecondDto paySecondDto, @RequestHeader HttpHeaders headers){
        if (null == paySecondDto.getProductId()) {
            return ResponseJson.error("二手商品Id不能为空！", null);
        }
        if (StringUtils.isEmpty(paySecondDto.getReturnUrl())) {
            return ResponseJson.error("回调地址不能为空！", null);
        }
        return payNonOrderService.paySecondByAlipay(paySecondDto, headers);
    }

    /**
     * 二手发布-银联线上支付
     * @param paySecondDto {
     *               productId     二手发布商品id
     *               returnUrl     页面回调地址
     *               bankCode      银行编码(银联支付使用)
     *               userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     * }
     */
    @ApiOperation("二手发布-银联线上支付（旧：/PayOrder/secondHandPay[UNIONPAY]）")
    @PostMapping("/second/union")
    public ResponseJson<JSONObject> paySecondByUnionPay(PaySecondDto paySecondDto, @RequestHeader HttpHeaders headers){
        if (null == paySecondDto.getProductId()) {
            return ResponseJson.error("二手商品Id不能为空！", null);
        }
        if (StringUtils.isEmpty(paySecondDto.getReturnUrl())) {
            return ResponseJson.error("回调地址不能为空！", null);
        }
        if (StringUtils.isEmpty(paySecondDto.getBankCode())) {
            return ResponseJson.error("银行编码不能为空！", null);
        }
        if (StringUtils.isEmpty(paySecondDto.getUserType())) {
            return ResponseJson.error("银行用户类型不能为空！", null);
        }
        return payNonOrderService.paySecondByUnionPay(paySecondDto, headers);
    }

    /**
     * 二手发布-支付回调
     */
    @ApiOperation("二手发布-支付回调（旧：/PayOrder/secondHandPayCallBack）")
    @GetMapping("/second/callback")
    public String paymentSecondCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException {
        if (StringUtils.isBlank(data)) {
            return "回调参数失败";
        }
        return payNonOrderService.paymentSecondCallback(data);
    }

    /**
     * 升级超级会员-微信线上支付
     * @param payVipDto {
     *                  userId        用户Id
     *                  vipId         会员套餐Id
     *                  returnUrl     页面回调地址
     *                  code          微信小程序code
     *                  state         微信公众号state参数
     * }
     */
    @ApiOperation("升级超级会员-微信线上支付")
    @PostMapping("/vip/wechat")
    public ResponseJson<JSONObject> paySuperVipByWeChat(PayVipDto payVipDto, @RequestHeader HttpHeaders headers){
        if (null == payVipDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == payVipDto.getVipId()) {
            return ResponseJson.error("会员套餐Id不能为空！", null);
        }
        if (StringUtils.isEmpty(payVipDto.getCode())) {
            return ResponseJson.error("微信code不能为空！", null);
        }
        return payNonOrderService.paySuperVipByWeChat(payVipDto, headers);
    }

    /**
     * 升级超级会员-支付宝线上支付
     * @param payVipDto {
     *                  userId        用户Id
     *                  vipId         会员套餐Id
     *                  returnUrl     页面回调地址
     * }
     */
    @ApiOperation("升级超级会员-支付宝线上支付")
    @PostMapping("/vip/alipay")
    public ResponseJson<JSONObject> paySuperVipByAlipay(PayVipDto payVipDto, @RequestHeader HttpHeaders headers){
        if (null == payVipDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == payVipDto.getVipId()) {
            return ResponseJson.error("会员套餐Id不能为空！", null);
        }
        if (StringUtils.isEmpty(payVipDto.getReturnUrl())) {
            return ResponseJson.error("回调地址不能为空！", null);
        }
        return payNonOrderService.paySuperVipByAlipay(payVipDto, headers);
    }

    /**
     * 升级超级会员-银联线上支付
     * @param payVipDto {
     *               userId        用户Id
     *               vipId         会员套餐Id
     *               returnUrl     页面回调地址
     *               bankCode      银行编码(银联支付使用)
     *               userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     * }
     */
    @ApiOperation("升级超级会员-银联线上支付（旧：/PayOrder/secondHandPay[UNIONPAY]）")
    @PostMapping("/vip/union")
    public ResponseJson<JSONObject> paySuperVipByUnionPay(PayVipDto payVipDto, @RequestHeader HttpHeaders headers){
        if (null == payVipDto.getUserId()) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        if (null == payVipDto.getVipId()) {
            return ResponseJson.error("会员套餐Id不能为空！", null);
        }
        if (StringUtils.isEmpty(payVipDto.getReturnUrl())) {
            return ResponseJson.error("回调地址不能为空！", null);
        }
        if (StringUtils.isEmpty(payVipDto.getBankCode())) {
            return ResponseJson.error("银行编码不能为空！", null);
        }
        if (StringUtils.isEmpty(payVipDto.getUserType())) {
            return ResponseJson.error("银行用户类型不能为空！", null);
        }
        return payNonOrderService.paySuperVipByUnionPay(payVipDto, headers);
    }

    /**
     * 升级超级会员-支付回调
     */
    @ApiOperation("升级超级会员-支付回调（旧：/PayOrder/secondHandPayCallBack）")
    @GetMapping("/vip/callback")
    public String paymentSuperVipCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException {
        if (StringUtils.isBlank(data)) {
            return "回调参数失败";
        }
        return payNonOrderService.paymentSuperVipCallback(data);
    }
}
