package com.caimei365.order.controller;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.PayDto;
import com.caimei365.order.model.dto.PayLinkDto;
import com.caimei365.order.service.PayOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.util.Map;

/**
 * 订单支付API
 *
 * @author : Charles
 * @date : 2021/7/29
 */
@Api(tags="订单支付API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/pay")
public class PayOrderApi {
    private final PayOrderService payOrderService;

    /**
     * 订单支付前效验付款规则
     */
    @ApiOperation("订单支付前效验付款规则（旧：/order/paymentValidation）")
    @ApiImplicitParam(required = false, name = "orderId", value = "订单Id")
    @GetMapping("/check")
    public ResponseJson<Map<String, Object>> paymentValidation(Integer orderId) {
        if (null == orderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return payOrderService.payBeforeCheck(orderId);
    }

    /**
     * 订单支付,余额抵扣
     * @param payDto {
     *                orderId      订单ID
     * }
     */
    @ApiOperation("订单支付,余额抵扣（旧：/order/balanceDeduction）")
    @PostMapping("/balance/deduction")
    public ResponseJson<Map<String, Object>> balanceDeduction(PayDto payDto){
        if (null == payDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return payOrderService.balanceDeduction(payDto.getOrderId());
    }

    /**
     * 获取线上支付全局开关状态
     */
    @ApiOperation("获取线上支付全局开关状态（旧：/PayOrder/onLineSwitch）")
    @GetMapping("/online/switch")
    public ResponseJson<Integer> getPayOnLineSwitch() {
        return payOrderService.getPayOnLineSwitch();
    }

    /**
     * 收银台数据显示
     */
    @ApiOperation("收银台数据（旧：/PayOrder/checkoutCounter）")
    @ApiImplicitParam(required = false, name = "orderId", value = "订单Id")
    @GetMapping("/checkout/counter")
    public ResponseJson<Map<String, Object>> getCheckoutCounter(Integer orderId) {
        if (null == orderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return payOrderService.getCheckoutCounter(orderId);
    }


    /**
     *  生成网银支付链接
     * @param payLinkDto {
     *          orderId       订单ID
     * }
     */
    @ApiOperation("生成网银支付链接（旧：/PayOrder/payLink）")
    @PostMapping("/link")
    public ResponseJson<String> getPayLink(PayLinkDto payLinkDto) {
        if (null == payLinkDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        return payOrderService.getPayLink(payLinkDto);
    }

    /**
     * 支付链接重定向到页面
     * @param linkLogo 链接标识
     */
    @ApiOperation("支付链接重定向到页面（旧：/PayOrder/jumpPage）")
    @ApiImplicitParam(required = false, name = "linkLogo", value = "链接标识")
    @GetMapping("/link/jump")
    public void jumpPayPage(String linkLogo, HttpServletResponse response) throws IOException {
        payOrderService.jumpPayPage(linkLogo, response);
    }

    /**
     * 重定向页面的支付数据
     */
    @ApiOperation("重定向页面的支付数据（旧：/PayOrder/linkData）")
    @GetMapping("/link/data")
    public ResponseJson<Map<String, Object>> linkPayData(String linkLogo) {
        if (StringUtils.isBlank(linkLogo)) {
            return ResponseJson.error("链接标识不能为空！", null);
        }
        return payOrderService.linkPayData(linkLogo);
    }

    /**
     * 微信线上支付
     * @param payDto {
     *               orderId      订单ID
     *               payAmount     支付金额，单位分，必须大于2
     *               returnUrl     页面回调地址
     *               code          微信小程序code
     *               state         微信公众号state参数
     * }
     */
    @ApiOperation("微信线上支付（旧：/PayOrder/miniWxPay）")
    @PostMapping("/wechat")
    public ResponseJson<JSONObject> payByWeChat(PayDto payDto, @RequestHeader HttpHeaders headers){
        if (null == payDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        if (null == payDto.getPayAmount() || payDto.getPayAmount() < 2) {
            return ResponseJson.error("支付金额不正确！", null);
        }
        if (StringUtils.isEmpty(payDto.getCode())) {
            return ResponseJson.error("微信code不能为空！", null);
        }
        return payOrderService.payByWeChat(payDto, headers);
    }

    /**
     * 支付宝线上支付
     * @param payDto {
     *               orderId      订单ID
     *               payAmount     支付金额，单位分，必须大于2
     *               returnUrl     页面回调地址
     * }
     */
    @ApiOperation("支付宝线上支付（旧：/PayOrder/pcMallPay[ALIPAY]）")
    @PostMapping("/alipay")
    public ResponseJson<JSONObject> payByAlipay(PayDto payDto, @RequestHeader HttpHeaders headers){
        if (null == payDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        if (null == payDto.getPayAmount() || payDto.getPayAmount() < 2) {
            return ResponseJson.error("支付金额不正确！", null);
        }
        if (StringUtils.isEmpty(payDto.getReturnUrl())) {
            return ResponseJson.error("回调地址不能为空！", null);
        }
        return payOrderService.payByAlipay(payDto, headers);
    }

    /**
     * 银联线上支付
     * @param payDto {
     *               orderId       订单ID
     *               payAmount     支付金额，单位分，必须大于2
     *               returnUrl     页面回调地址
     *               bankCode      银行编码(银联支付使用)
     *               userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     * }
     */
    @ApiOperation("银联线上支付（旧：/PayOrder/pcMallPay[UNIONPAY]）")
    @PostMapping("/union")
    public ResponseJson<JSONObject> payByUnionPay(PayDto payDto, @RequestHeader HttpHeaders headers){
        if (null == payDto.getOrderId()) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        if (null == payDto.getPayAmount() || payDto.getPayAmount() < 2) {
            return ResponseJson.error("支付金额不正确！", null);
        }
        if (StringUtils.isEmpty(payDto.getReturnUrl())) {
            return ResponseJson.error("回调地址不能为空！", null);
        }
        if (StringUtils.isEmpty(payDto.getBankCode())) {
            return ResponseJson.error("银行编码不能为空！", null);
        }
        if (StringUtils.isEmpty(payDto.getUserType())) {
            return ResponseJson.error("银行用户类型不能为空！", null);
        }
        return payOrderService.payByUnionPay(payDto, headers);
    }

    /**
     * 支付异步通知回调
     */
    @ApiOperation("支付异步通知回调（旧：/PayOrder/paymentCallback）")
    @GetMapping("/callback")
    public String paymentCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException {
        if (StringUtils.isBlank(data)) {
            return "回调参数失败";
        }
        return payOrderService.paymentCallback(data);
    }

    /**
     * 判断此次支付是否完成
     * @param orderId           订单id
     * @param paySuccessCounter 付款次数
     */
    @ApiOperation("判断此次支付是否完成（旧：/PayOrder/payWhetherSuccess）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "orderId", value = "订单id"),
            @ApiImplicitParam(required = false, name = "paySuccessCounter", value = "付款次数")
    })
    @GetMapping("/result/check")
    public ResponseJson<String> payWhetherSuccess(Integer orderId, Integer paySuccessCounter) {
        if (null == orderId) {
            return ResponseJson.error("订单Id不能为空！", null);
        }
        if (null == paySuccessCounter) {
            return ResponseJson.error("付款次数不能为空！", null);
        }
        return payOrderService.payWhetherSuccess(orderId, paySuccessCounter);
    }

    /**
     * 查询本次支付订单结果
     * @param mbOrderId 平台唯一流水号
     */
    @ApiOperation("查询本次支付订单结果（旧：/PayOrder/findOrderStatus）")
    @ApiImplicitParam(required = false, name = "mbOrderId", value = "平台唯一流水号")
    @GetMapping("/result/json")
    public ResponseJson<JSONObject> getPayOrderResult(String mbOrderId) {
        if (null == mbOrderId) {
            return ResponseJson.error("平台唯一流水号不能为空！", null);
        }
        return payOrderService.getPayOrderResult(mbOrderId);
    }

    /**
     * 延时分账异步通知回调
     */
    @ApiOperation("延时分账异步通知回调（旧：/PayOrder/delayedSplittingCallback）")
    @GetMapping("/delay/split/callback")
    public String delayedSplittingCallback(String data) {
        if (StringUtils.isBlank(data)) {
            return "回调参数失败";
        }
        return payOrderService.delayedSplittingCallback(data);
    }

//    /**
//     * 手动临时分账
//     */
//    @PostMapping("/temp/split")
//    public String temporarySplitting(){
//        return payOrderService.temporarySplitting();
//    }
//
//    /**
//     * 手动临时分账异步通知回调
//     */
//    @ApiOperation("手动临时分账异步通知回调")
//    @GetMapping("/temp/split/callback")
//    public String tempSplittingCallback(String data) {
//        if (StringUtils.isBlank(data)) {
//            return "回调参数失败";
//        }
//        return payOrderService.tempSplittingCallback(data);
//    }

}
