package com.caimei365.order.service;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.JsonParamsDto;
import com.caimei365.order.model.dto.LogisticsBatchDto;
import com.caimei365.order.model.dto.LogisticsDto;
import com.caimei365.order.model.vo.*;
import com.github.pagehelper.PageInfo;
import org.springframework.http.HttpHeaders;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/8/5
 */
public interface ShipService {
    /**
     * 供应商订单列表
     * @param shopId         供应商Id
     * @param sendOutStatus  发货状态
     * @param payStatus      结算状态
     * @param shopOrderNo    订单编号
     * @param receiver       买家名称
     * @param pageNum        页码
     * @param pageSize       每页数量
     */
    ResponseJson<PageInfo<ShopOrderVo>> getShopOrderList(Integer shopId, Integer sendOutStatus, Integer payStatus, String shopOrderNo, String receiver, int pageNum, int pageSize);
    /**
     * 供应商结算列表
     * @param shopId         供应商Id
     * @param payStatus      结算状态
     * @param shopOrderNo    订单编号
     * @param name       买家名称
     * @param pageNum        页码
     * @param pageSize       每页数量
     */
    ResponseJson<PageInfo<SettlementVo>> getSettlementList(Integer shopId, Integer payStatus, String shopOrderNo, String name, int pageNum, int pageSize);
    /**
     * 供应商子订单详情
     * @param shopOrderId 子订单Id
     */
    ResponseJson<Map<String, Object>> getShopOrderDetail(Integer shopOrderId);
    /**
     * 供应商售货清单
     * @param shopOrderId 子订单Id
     */
    ResponseJson<Map<String, Object>> getSalesList(Integer shopOrderId);
    /**
     * 发货页面子订单数据
     * @param shopOrderId 子订单Id
     */
    ResponseJson<ShopOrderVo> getShopOrderInfo(Integer shopOrderId);
    /**
     * 物流公司
     * @param value 快递公司代码
     */
    ResponseJson<List<CompanyVo>> getLogisticsCompany(String value);
    /**
     * 发货
     * @param logisticsDto {
     *                     shopOrderId: 子订单Id,
     *                     note: 备注信息,
     *                     image: 图片备注以##隔开,
     *                     logistics：[  // 物流信息
     *                          {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'},
     *                          {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'}
     *                        ],
     *                     products：[   // 商品信息
     *                          {orderProductId:订单商品id,num:此次发货数量},
     *                          {orderProductId:订单商品id,num:此次发货数量}
     *                        ],
     *                     }
     */
    ResponseJson<Integer> addDelivery(LogisticsDto logisticsDto);
    /**
     * 供应商发货记录
     * @param shopOrderId    子订单Id
     * @param pageNum        页码
     * @param pageSize       每页数量
     */
    ResponseJson<Map<String, Object>> getDeliveryRecord(Integer shopOrderId, int pageNum, int pageSize);
    /**
     * 供应商撤销发货
     * @param logisticsBatchId 发货物流批次Id
     */
    ResponseJson<Void> cancelDelivery(Integer logisticsBatchId);
    /**
     * 当前批次发货商品数据
     * @param logisticsBatchId 发货物流批次Id
     */
    ResponseJson<List<LogisticsRecordVo>> getLogisticsRecord(Integer logisticsBatchId);
    /**
     * 发货记录-添加物流
     * @param logisticsBatchDto {
     *                              logisticsBatchId: 发货批次表Id,
     *                              remark: 备注,
     *                              remarkImage: 图片备注,
     *                              logistics：[  // 物流信息
     *                                  {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'},
     *                                  {number:'45646464646',logisticsCompanyName:'顺丰物流',logisticsCompanyCode:'物流英文代码'}
     *                              ]
     *                          }
     */
    ResponseJson<Void> addLogisticsInfo(LogisticsBatchDto logisticsBatchDto);
    /**
     * 扫码添加物流
     * @param number      快递单号
     * @param mobile      收/寄件人的电话号码(顺丰必填)
     */
    ResponseJson<CompanyVo> scanGetLogisticsInfo(String number, String mobile);
    /**
     * 供应商查看物流
     * @param shopOrderId       子订单Id
     * @param logisticsBatchId  发货物流批次Id
     */
    ResponseJson<Map<String, Object>> getLogisticsInfo(Integer shopOrderId, Integer logisticsBatchId);
    /**
     * 删除物流信息
     * @param logisticsInfoId  发货物流信息Id
     */
    ResponseJson<Void> deleteLogisticsInfo(Integer logisticsInfoId);
    /**
     * 小程序分享发货-获取分享码
     * @param shopOrderId 子订单Id
     */
    ResponseJson<String> getShipShareCode(Integer shopOrderId);
    /**
     * 小程序分享发货-分享权限控制
     * @param source 来源：1:客服扫码,2:供应商运营人员查看,3:分享码查看
     * @param shopOrderId 子订单Id
     * @param shareCode 分享码
     * @param code 微信code
     * @param encryptedData 微信加密数据
     * @param iv 微信加密算法的初始向量
     * @param headers HttpHeaders
     */
    ResponseJson<Map<String, Object>> checkShipShareCode(Integer source, Integer shopOrderId, String shareCode, String code, String encryptedData, String iv, HttpHeaders headers);
    /**
     * 查看商品资质信息
     * @param logisticsBatchId 发货物流批次Id
     */
    ResponseJson<List<QualificationVo>> getProductQualification(Integer logisticsBatchId);
     /**
     * 保存商品资质
     *
     * @param jsonParamsDto params:[{
     *                         recordId: 商品发货记录id,
     *                         sn:       sn码,
     *                         files:    [{fileName:"文件名称",ossName:"oss文件名称"},{fileName:"文件名称",ossName:"oss文件名称"}...]
     *                         images:   ["图片","图片",...]
     *                         }, {}]
     */
    ResponseJson<Void> saveProductQualification(JsonParamsDto jsonParamsDto);
}
