package com.caimei365.order.service.impl;

import com.caimei365.order.components.OrderCommonService;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.OrderSellerMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.vo.OrderVo;
import com.caimei365.order.service.OrderSellerService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/19
 */
@Slf4j
@Service
public class OrderSellerServiceImpl implements OrderSellerService {
    @Resource
    private OrderSellerMapper orderSellerMapper;
    @Resource
    private OrderCommonService orderCommonService;
    @Resource
    private BaseMapper baseMapper;

    /**
     * 协销订单列表
     *
     * @param serviceProviderId 协销Id
     * @param listType          1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderSubmitType   1：自主订单， 2:协销或后台订单
     * @param orderNo           订单编号
     * @param pageNum           页码
     * @param pageSize          每页数量
     */
    @Override
    public ResponseJson<PageInfo<OrderVo>> getOrderList(Integer serviceProviderId, Integer listType, Integer orderSubmitType, String orderNo, int pageNum, int pageSize) {
        // 获取该协销下的机构用户Id
        List<Integer> clubUserIds = orderSellerMapper.getClubUserIds(serviceProviderId);
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        // 获取订单列表
        List<OrderVo> orderList = orderSellerMapper.getOrderList(clubUserIds, listType, orderSubmitType, orderNo);
        //设置主订单数据
        orderList.forEach(this::setSellerOrder);
        PageInfo<OrderVo> pageInfo = new PageInfo(orderList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * 设置协销订单
     */
    private void setSellerOrder(OrderVo order) {
        // 设置订单状态
        orderCommonService.setOrderStatus(order);
        // 设置子订单数据
        orderCommonService.getShopOrderData(order);
        // 设置付款金额
        orderCommonService.getDiscernReceiptAndSetOrder(order);
        // 设置机构名称
        String clubName = baseMapper.getClubNameById(order.getClubId());
        order.setClubName(clubName);
        // 设置机构类型
        Integer svipUserId = baseMapper.getSvipUserIdByClubId(order.getClubId());
        order.setSvipUserFlag(null != svipUserId ? 1 : 0);
    }

    /**
     * 协销下机构总订单列表
     *
     * @param serviceProviderId 协销Id
     * @param clubId            机构Id
     * @param listType          1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderSubmitType   1：自主订单， 2:协销或后台订单
     * @param pageNum           页码
     * @param pageSize          每页数量
     */
    @Override
    public ResponseJson<PageInfo<OrderVo>> getOrderListClub(Integer serviceProviderId, Integer clubId, Integer listType, Integer orderSubmitType, int pageNum, int pageSize) {
        // 判断该机构是不是该协销下的
        Integer spId = orderSellerMapper.getSpIdByClubId(clubId);
        if (!serviceProviderId.equals(spId)) {
            return ResponseJson.error("该机构不是该协销下的机构！", null);
        }
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        // 获取订单列表
        List<OrderVo> orderList = orderSellerMapper.getOrderListClub(clubId, listType, orderSubmitType);
        //设置主订单数据
        orderList.forEach(this::setSellerOrder);
        PageInfo<OrderVo> pageInfo = new PageInfo(orderList);
        // 更新协销查看机构订单时间
        orderSellerMapper.updateClubLastCheckOrderDate(clubId, new Date());
        return ResponseJson.success(pageInfo);
    }

    /**
     * 协销根据关键词搜索订单
     *
     * @param serviceProviderId 协销Id
     * @param searchWord        搜索关键词
     * @param pageNum           页码
     * @param pageSize          每页数量
     */
    @Override
    public ResponseJson<PageInfo<OrderVo>> getOrderListByKeyword(Integer serviceProviderId, String searchWord, int pageNum, int pageSize) {
        // 获取协销用户Id
        Integer spUserId = orderSellerMapper.getUserIdBySpId(serviceProviderId);
        if (null == spUserId) {
            return ResponseJson.error("协销用户异常！", null);
        }
        // 设置搜索关键词历史记录
        orderCommonService.setHistoryRecord(spUserId, searchWord);
        // 获取该协销下的机构用户Id
        List<Integer> clubUserIds = orderSellerMapper.getClubUserIds(serviceProviderId);
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        // 获取订单列表
        List<OrderVo> orderList = orderSellerMapper.getOrderListByKeyword(clubUserIds, searchWord);
        //设置主订单数据
        orderList.forEach(this::setSellerOrder);
        PageInfo<OrderVo> pageInfo = new PageInfo(orderList);
        return ResponseJson.success(pageInfo);
    }

}
