package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.WeChatService;
import com.caimei365.order.mapper.PayOrderMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.bo.PayParamBo;
import com.caimei365.order.model.dto.PaySecondDto;
import com.caimei365.order.model.dto.PayVipDto;
import com.caimei365.order.model.po.SplitAccountPo;
import com.caimei365.order.service.PayNonOrderService;
import com.caimei365.order.utils.MathUtil;
import com.caimei365.order.utils.PayUtil;
import com.caimei365.order.utils.pay.RSAUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/9/27
 */
@Slf4j
@Service
public class PayNonOrderServiceImpl implements PayNonOrderService {
    @Resource
    private PayOrderMapper payOrderMapper;
    @Resource
    private WeChatService weChatService;
    @Value("${pay.second-notify-url}")
    private String secondHandUrl;
    @Value("${pay.vip-notify-url}")
    private String superVipUrl;
    /**
     * 二手发布微信线上支付
     *
     * @param paySecondDto {
     *                     productId      二手发布商品id
     *                     returnUrl     页面回调地址
     *                     code          微信小程序code
     *                     state         微信公众号state参数
     *                     }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> paySecondByWeChat(PaySecondDto paySecondDto, HttpHeaders headers) {
        PayParamBo tempParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(paySecondDto, tempParam);
        ResponseJson<PayParamBo> jsonParam = getWeChatResponseJson(tempParam, headers);
        if (-1 == jsonParam.getCode()){
            return ResponseJson.error(jsonParam.getMsg(), null);
        }
        PayParamBo payParam = jsonParam.getData();
        // 二手发布默认100元
        payParam.setPayAmount(10000);
        payParam.setNotifyUrl(secondHandUrl);
        log.info("【二手发布微信支付】>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>获取openId:" + payParam.getOpenId());
        return toPaySecond(payParam, headers);
    }

    /**
     * 微信线上支付-获取微信支付信息
     *
     * @param payParam 支付参数
     * @param headers  HttpHeaders
     * @return payParam获取微信支付信息
     */
    private ResponseJson<PayParamBo> getWeChatResponseJson(PayParamBo payParam, HttpHeaders headers) {
        // 微信线上支付
        payParam.setPayWay("WEIXIN");
        String openId = null;
        if (null == payParam.getState()) {
            // 小程序微信授权获取登录信息
            ResponseJson<Map<String, Object>> appletsInfo = weChatService.getInfoMapByApplets(payParam.getCode(), headers, 1);
            if (appletsInfo.getCode() == -1) {
                return ResponseJson.error(appletsInfo.getMsg(), null);
            }
            Map<String, Object> infoData = appletsInfo.getData();
            openId = (String) infoData.get(WeChatService.Keys.OPEN_ID);
            if (StringUtils.isEmpty(openId)) {
                return ResponseJson.error("微信小程序获取openId失败！", null);
            }
            //小程序微信快捷支付
            payParam.setPayType("MINIAPP_WEIXIN");
        } else {
            try {
                // 微信公众号，通过code获取微信用户信息
                Map<String, Object> map = weChatService.getInfoMapByWeb(payParam.getCode(), "crm");
                openId = (String) map.get(WeChatService.Keys.OPEN_ID);
            } catch (Exception e) {
                log.error("try-catch:",e);
            }
            if (StringUtils.isEmpty(openId)) {
                return ResponseJson.error("微信公众号获取openId失败！", null);
            }
            //pc微信扫码支付,微信公众号支付
            payParam.setPayType("JSAPI_WEIXIN");
        }
        payParam.setOpenId(openId);
        return ResponseJson.success(payParam);
    }

    /**
     * 二手发布支付宝线上支付
     *
     * @param paySecondDto {
     *                     productId      二手发布商品id
     *                     returnUrl     页面回调地址
     *                     }
     * @param headers HttpHeaders
     */
    @Override
    public ResponseJson<JSONObject> paySecondByAlipay(PaySecondDto paySecondDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(paySecondDto, payParam);
        //支付宝支付
        payParam.setPayWay("ALIPAY");
        payParam.setPayType("ALIPAY_H5");
        // 二手发布默认100元
        payParam.setPayAmount(10000);
        payParam.setNotifyUrl(secondHandUrl);
        log.info("【二手发布支付宝支付】>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>productId:" + payParam.getProductId());
        return toPaySecond(payParam, headers);
    }

    /**
     * 二手发布银联线上支付
     *
     * @param paySecondDto {
     *                     productId       二手发布商品id
     *                     returnUrl     页面回调地址
     *                     bankCode      银行编码(银联支付使用)
     *                     userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     *                     }
     * @param headers HttpHeaders
     */
    @Override
    public ResponseJson<JSONObject> paySecondByUnionPay(PaySecondDto paySecondDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(paySecondDto, payParam);
        // 银联支付
        payParam.setPayWay("UNIONPAY");
        payParam.setPayType("GATEWAY_UNIONPAY");
        // 二手发布默认100元
        payParam.setPayAmount(10000);
        payParam.setNotifyUrl(secondHandUrl);
        log.info("【二手发布银联支付】>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>productId:" + payParam.getProductId());
        return toPaySecond(payParam, headers);
    }

    /**
     * 二手发布支付
     */
    private ResponseJson<JSONObject> toPaySecond(PayParamBo payParam, HttpHeaders headers) {
        // 定义支付请求结果
        JSONObject result = null;
        try {
            // 时间戳
            long time = System.currentTimeMillis() / 1000;
            // 用户IP地址
            JSONObject json = getPayJsonObject(payParam, headers, time);
            String environment = "";
            if (payParam.getNotifyUrl().contains("18002")) {
                environment = "DEV";
            } else if (payParam.getNotifyUrl().contains("core-b")) {
                environment = "BETA";
            }
            // 商户订单号
            String orderId = payParam.getProductId() + "T" + time + environment;
            json.put("orderId", orderId);
            //商品名称
            String product = "采美订单" + orderId;
            json.put("product", product);
            String attach = payParam.getProductId() + "," + payParam.getPayType();
            json.put("attach", attach);
            // 二手发布默认100元,默认私账
            List<Map<String, String>> list = new ArrayList<>();
            Map<String, String> map = new HashMap<>(3);
            map.put("subUserNo", PayUtil.privateAccountNo);
            map.put("splitBillType", "1");
            map.put("splitBillValue", "1");
            list.add(map);
            String splitBillDetail = JSONObject.toJSONString(list);
            json.put("splitBillDetail", splitBillDetail);
            String sign = PayUtil.getPaySign(json, PayUtil.merKey);
            json.put("sign", sign);
            // 私钥加密
            String data = RSAUtil.privateEncrypt(json.toJSONString(), PayUtil.merKey);
            // 提交
            result = PayUtil.httpGet("https://platform.mhxxkj.com/paygateway/mbpay/order/v1", PayUtil.merAccount, data);
        } catch (Exception e) {
            log.error("错误信息", e);
            return ResponseJson.error("支付失败！", null);
        }
        String code = result.getString("code");
        if (!"000000".equals(code)) {
            String msg = result.getString("msg");
            log.info("第三方支付失败>>>>>>>msg:" + msg);
            return ResponseJson.error(msg, null);
        }
        // 保存二手发布分账参数
        SplitAccountPo splitAccount = new SplitAccountPo();
        splitAccount.setOrderId(payParam.getOrderId());
        splitAccount.setProductId(payParam.getProductId());
        // 二手交易供应商
        splitAccount.setShopId(1252);
        splitAccount.setProductType(4);
        splitAccount.setType(2);
        if ("UNIONPAY".equals(payParam.getPayWay())) {
            splitAccount.setSplitAccount(92.00);
        } else {
            splitAccount.setSplitAccount(99.62);
        }
        JSONObject data = result.getJSONObject("data");
        splitAccount.setMbOrderId(data.getString("mbOrderId"));
        splitAccount.setOrderRequestNo(data.getString("orderId"));
        splitAccount.setPayStatus(0);
        // 保存分账详情
        payOrderMapper.insertSplitAccount(splitAccount);

        return ResponseJson.success(result);
    }

    /**
     * 二手发布支付回调
     */
    @Override
    public String paymentSecondCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException {
        log.info("******************** 二手发布支付异步回调 start *******************");
        // 公钥解密
        JSONObject json = PayUtil.publicKeyDecrypt(data, PayUtil.publicKey);
        log.info("【二手发布支付异步回调】>>>>>>>>>>>>>>公钥解密:" + json);
        // 公钥验签
        String signaa = json.getString("sign");
        json.remove("sign");
        String signbb = PayUtil.getPaySign(json, PayUtil.publicKey);
        if (!signaa.equals(signbb)) {
            return "验签失败";
        }
        // 订单状态
        String orderStatus = json.getString("orderStatus");
        // 平台唯一流水号
        String mbOrderId = json.getString("mbOrderId");
        // 附加数据，下单时若有传输则原样返回，下单时为空，则不返回该数据
        String attach = json.getString("attach");
        // 订单金额，以元为单位
        BigDecimal amount = json.getBigDecimal("amount");
        log.info("【二手发布支付异步回调】>>>>>>>>>>>>>>支付订单状态:" + orderStatus);
        if ("FAILED".equals(orderStatus)) {
            return "支付失败";
        }
        log.info("【二手发布支付异步回调】>>>>>>>>>>>>>>本次支付金额：" + amount);
        String[] split = attach.split(",");
        // 订单id
        Integer productId = Integer.valueOf(split[0]);
        // 支付类型
        String payType = split[1];
        double payAmount = amount.doubleValue();
        String payFormData = json.toJSONString();
        payOrderMapper.updateSeconHandDetail(productId, payType, payAmount, payFormData);
        //修改分账付款状态
        payOrderMapper.updateSplitAccountByPay(mbOrderId);
        return "SUCCESS";
    }

    /**
     * 设置支付基础参数
     */
    private JSONObject getPayJsonObject(PayParamBo payParam, HttpHeaders headers, long time) {
        // 用户IP地址
        String userIp = headers.getFirst("X-CLIENT-IP");
        JSONObject json = new JSONObject();
        json.put("merAccount", PayUtil.merAccount);
        json.put("merNo", PayUtil.merNo);
        json.put("time", time);
        //支付金额
        json.put("amount", payParam.getPayAmount());
        json.put("payWay", payParam.getPayWay());
        json.put("payType", payParam.getPayType());
        json.put("userIp", userIp);
        json.put("returnUrl", payParam.getReturnUrl());
        json.put("notifyUrl", payParam.getNotifyUrl());
        if (StringUtils.isNotBlank(payParam.getBankCode())) {
            json.put("bankCode", payParam.getBankCode());
        }
        if (StringUtils.isNotBlank(payParam.getUserType())) {
            json.put("userType", payParam.getUserType());
        }
        if (StringUtils.isNotBlank(payParam.getOpenId())) {
            json.put("openId", payParam.getOpenId());
        }
        return json;
    }
    /**
     * 升级超级会员-微信线上支付
     *
     * @param payVipDto {
     *                  userId        用户Id
     *                  vipId         会员套餐Id
     *                  returnUrl     页面回调地址
     *                  code          微信小程序code
     *                  state         微信公众号state参数
     *                  }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> paySuperVipByWeChat(PayVipDto payVipDto, HttpHeaders headers) {
        PayParamBo tempParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payVipDto, tempParam);
        ResponseJson<PayParamBo> jsonParam = getWeChatResponseJson(tempParam, headers);
        if (-1 == jsonParam.getCode()){
            return ResponseJson.error(jsonParam.getMsg(), null);
        }
        PayParamBo payParam = jsonParam.getData();
        // 获取会员套餐价格
        Double price = payOrderMapper.getVipPackagePriceById(payParam.getVipId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(superVipUrl);
        log.info("【升级超级会员微信支付】>>>>>>>>>>>vipId:" + payParam.getVipId()+"，获取openId:" + payParam.getOpenId());
        return toPaySuperVip(payParam, headers);
    }

    /**
     * 升级超级会员-支付宝线上支付
     *
     * @param payVipDto {
     *                  userId        用户Id
     *                  vipId         会员套餐Id
     *                  returnUrl     页面回调地址
     *                  }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> paySuperVipByAlipay(PayVipDto payVipDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payVipDto, payParam);
        //支付宝支付
        payParam.setPayWay("ALIPAY");
        payParam.setPayType("ALIPAY_H5");
        // 获取会员套餐价格
        Double price = payOrderMapper.getVipPackagePriceById(payParam.getVipId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(superVipUrl);
        log.info("【升级超级会员支付宝支付】>>>>>>>>>>>vipId:" + payParam.getVipId());
        return toPaySuperVip(payParam, headers);
    }

    /**
     * 升级超级会员-银联线上支付
     *
     * @param payVipDto {
     *                  userId        用户Id
     *                  vipId         会员套餐Id
     *                  returnUrl     页面回调地址
     *                  bankCode      银行编码(银联支付使用)
     *                  userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     *                  }
     * @param headers
     */
    @Override
    public ResponseJson<JSONObject> paySuperVipByUnionPay(PayVipDto payVipDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payVipDto, payParam);
        // 银联支付
        payParam.setPayWay("UNIONPAY");
        payParam.setPayType("GATEWAY_UNIONPAY");
        // 获取会员套餐价格
        Double price = payOrderMapper.getVipPackagePriceById(payParam.getVipId());
        payParam.setPayAmount(MathUtil.mul(price, 100).intValue());
        payParam.setNotifyUrl(superVipUrl);
        log.info("【升级超级会员银联线上支付】>>>>>>>>>>>vipId:" + payParam.getVipId());
        return toPaySuperVip(payParam, headers);
    }


    /**
     * 升级超级会员支付
     */
    private ResponseJson<JSONObject> toPaySuperVip(PayParamBo payParam, HttpHeaders headers) {
        // 定义支付请求结果
        JSONObject result = null;
        try {
            // 时间戳
            long time = System.currentTimeMillis() / 1000;
            // 设置支付基础参数
            JSONObject json = getPayJsonObject(payParam, headers, time);
            // 支付环境
            String environment = "";
            if (payParam.getNotifyUrl().contains("18002")) {
                environment = "DEV";
            } else if (payParam.getNotifyUrl().contains("core-b")) {
                environment = "BETA";
            }
            // 商户订单号
            String orderId = payParam.getUserId() + "V" + payParam.getVipId() + "T" + time + environment;
            json.put("orderId", orderId);
            //商品名称
            String product = "采美订单" + orderId;
            json.put("product", product);
            String attach = payParam.getVipId() + "," + payParam.getPayType()+ "," + payParam.getUserId();
            json.put("attach", attach);
            // 升级超级会员默认公账
            List<Map<String, String>> list = new ArrayList<>();
            Map<String, String> map = new HashMap<>(3);
            map.put("subUserNo", PayUtil.publicAccountNo);
            // 1按比例分账
            map.put("splitBillType", "1");
            // 1比例值总额
            map.put("splitBillValue", "1");
            list.add(map);
            String splitBillDetail = JSONObject.toJSONString(list);
            json.put("splitBillDetail", splitBillDetail);
            String sign = PayUtil.getPaySign(json, PayUtil.merKey);
            json.put("sign", sign);
            // 私钥加密
            String data = RSAUtil.privateEncrypt(json.toJSONString(), PayUtil.merKey);
            // 提交
            result = PayUtil.httpGet("https://platform.mhxxkj.com/paygateway/mbpay/order/v1", PayUtil.merAccount, data);
        } catch (Exception e) {
            log.error("错误信息", e);
            return ResponseJson.error("支付失败！", null);
        }
        String code = result.getString("code");
        if (!"000000".equals(code)) {
            String msg = result.getString("msg");
            log.info("第三方支付失败>>>>>>>msg:" + msg);
            return ResponseJson.error(msg, null);
        }
        // 保存升级超级会员分账参数
        JSONObject data = result.getJSONObject("data");
        SplitAccountPo splitAccount = new SplitAccountPo();
        splitAccount.setVipPackageId(payParam.getVipId());
        // 超级会员 公账-专票
        splitAccount.setType(1);
        // 待分账总金额
        double splitAmount = MathUtil.div(payParam.getPayAmount(), 100).doubleValue();
        // 总手续费
        double procedureFee;
        if ("UNIONPAY".equals(payParam.getPayWay())) {
            procedureFee = 8.00;
        } else {
            //手续费
            procedureFee = MathUtil.mul(splitAmount, 0.0038, 2).doubleValue();
            if (MathUtil.compare(procedureFee, 0) == 0) {
                procedureFee = 0.01;
            }
        }
        splitAmount = MathUtil.sub(splitAmount, procedureFee).doubleValue();
        splitAccount.setSplitAccount(splitAmount);
        splitAccount.setMbOrderId(data.getString("mbOrderId"));
        splitAccount.setOrderRequestNo(data.getString("orderId"));
        splitAccount.setPayStatus(0);
        // 保存分账详情
        payOrderMapper.insertSplitAccount(splitAccount);

        return ResponseJson.success(result);
    }

    /**
     * 升级超级会员-支付回调
     *
     * @param data
     */
    @Override
    public String paymentSuperVipCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException {
        log.info("******************** 升级超级会员异步回调 start *******************");
        // 公钥解密
        JSONObject json = PayUtil.publicKeyDecrypt(data, PayUtil.publicKey);
        log.info("【升级超级会员异步回调】>>>>>>>>>>>公钥解密：" + json);
        // 公钥验签
        String signaa = json.getString("sign");
        json.remove("sign");
        String signbb = PayUtil.getPaySign(json, PayUtil.publicKey);
        if (!signaa.equals(signbb)) {
            return "验签失败";
        }
        // 订单状态
        String orderStatus = json.getString("orderStatus");
        // 平台唯一流水号
        String mbOrderId = json.getString("mbOrderId");
        // 金额，以元为单位
        BigDecimal amount = json.getBigDecimal("amount");
        log.info("【升级超级会员异步回调】>>>>>>>>>>>支付订单状态：" + orderStatus);
        if ("FAILED".equals(orderStatus)) {
            return "支付失败";
        }
        log.info("【升级超级会员异步回调】>>>>>>>>>>>>>>本次支付金额：" + amount);
        // 附加数据，支付时若有传输则原样返回(vipId,payType,userId)，下单时为空，则不返回该数据
        String attach = json.getString("attach");
        String[] split = attach.split(",");
        // 会员套餐Id
        Integer vipId = Integer.valueOf(split[0]);
        // 支付类型
        String payType = split[1];
        // 用户Id
        Integer userId = Integer.valueOf(split[2]);
        // 金额
        double payAmount = amount.doubleValue();
        String payFormData = json.toJSONString();

        // todo 支付成功-超级会员数据库修改

        //修改分账付款状态
        payOrderMapper.updateSplitAccountByPay(mbOrderId);
        return "SUCCESS";
    }

}
