package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.OrderCommonService;
import com.caimei365.order.components.ProductService;
import com.caimei365.order.components.WeChatService;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.OrderCommonMapper;
import com.caimei365.order.mapper.PayOrderMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.bo.PayParamBo;
import com.caimei365.order.model.dto.PayDto;
import com.caimei365.order.model.dto.PayLinkDto;
import com.caimei365.order.model.enums.OrderStatus;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.PayOrderService;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.utils.CodeUtil;
import com.caimei365.order.utils.MathUtil;
import com.caimei365.order.utils.Md5Util;
import com.caimei365.order.utils.PayUtil;
import com.caimei365.order.utils.pay.RSAUtil;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/29
 */
@Slf4j
@Service
public class PayOrderServiceImpl implements PayOrderService {
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private PayOrderMapper payOrderMapper;
    @Resource
    private OrderCommonMapper orderCommonMapper;
    @Resource
    private OrderCommonService orderCommonService;
    @Resource
    private ProductService productService;
    @Resource
    private WeChatService weChatService;
    @Resource
    private RemoteCallService remoteCallService;
    @Value("${pay.redirect-link}")
    private String redirectLink;
    @Value("${pay.link-page}")
    private String linkPage;
    @Value("${pay.notify-url}")
    private String notifyUrl;

    /**
     * 订单支付前效验付款规则
     *
     * @param orderId 订单Id
     */
    @Override
    public ResponseJson<Map<String, Object>> payBeforeCheck(Integer orderId) {
        // 订单信息
        OrderVo order = orderCommonMapper.getOrderByOrderId(orderId);
        if (null == order) {
            return ResponseJson.error("订单不存在", null);
        }
        // 返回数据
        Map<String, Object> map = new HashMap<>();
        map.put("orderId", orderId);
        // 查询审核中的退款退货数量
        int refundCount = payOrderMapper.getRefundCount(orderId);
        if (refundCount > 0) {
            map.put("code", -1);
            map.put("msg", "订单已申请全部退款，无需再付款！");
            return ResponseJson.success(map);
        }
        // 付款金额
        orderCommonService.getDiscernReceiptAndSetOrder(order);
        map.put("pendingPayments", order.getPendingPayments());
        map.put("onlinePayFlag", order.getOnlinePayFlag());
        // 商品数据
        List<OrderProductVo> orderProductList = orderCommonMapper.getOrderProductByOrderId(orderId);
        for (OrderProductVo orderProduct : orderProductList) {
            // 是否充值商品
            boolean recharge = productService.isRechargeProduct(orderProduct.getProductId());
            if (recharge) {
                return ResponseJson.success(map);
            }
        }
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(order.getUserId());
        if (MathUtil.compare(availableMoney, 0) > 0) {
            if (MathUtil.compare(availableMoney, order.getPendingPayments()) >= 0) {
                map.put("code", 2);
                map.put("msg", "余额大于或等于待付金额");
                map.put("ableUserMoney", availableMoney);
            } else {
                map.put("code", 1);
                map.put("msg", "余额不为0且小于待付金额");
                map.put("ableUserMoney", availableMoney);
                map.put("pendingPayments", MathUtil.sub(order.getPendingPayments(), availableMoney));
            }
        }
        return ResponseJson.success(map);
    }

    /**
     * 订单支付,余额抵扣
     *
     * @param orderId 订单Id
     */
    @Override
    public ResponseJson<Map<String, Object>> balanceDeduction(Integer orderId) {
        // 订单信息
        OrderVo order = orderCommonMapper.getOrderByOrderId(orderId);
        if (null == order) {
            return ResponseJson.error("订单不存在", null);
        }
        // 付款金额
        orderCommonService.getDiscernReceiptAndSetOrder(order);
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(order.getUserId());
        Double balancePayFee = 0d;
        if (MathUtil.compare(availableMoney, 0) > 0) {
            if (MathUtil.compare(availableMoney, order.getPendingPayments()) >= 0) {
                if (OrderStatus.UNCONFIRMED.getCode() != order.getStatus()) {
                    if (1 == order.getSendOutStatus()) {
                        order.setStatus(31);
                    } else if (2 == order.getSendOutStatus()) {
                        order.setStatus(32);
                    } else {
                        order.setStatus(33);
                    }
                }
                order.setReceiptStatus(3);
                balancePayFee = order.getPendingPayments();
            } else {
                if (OrderStatus.UNCONFIRMED.getCode() != order.getStatus()) {
                    //待确认状态,不修改订单状态
                    if (1 == order.getSendOutStatus()) {
                        order.setStatus(21);
                    } else if (2 == order.getSendOutStatus()) {
                        order.setStatus(22);
                    } else {
                        order.setStatus(23);
                    }
                }
                order.setReceiptStatus(2);
                balancePayFee = availableMoney;
            }

            order.setBalancePayFee(MathUtil.add(order.getBalancePayFee(), balancePayFee).doubleValue());
            order.setPayableAmount(MathUtil.sub(order.getPayableAmount(), balancePayFee).doubleValue());
            Date date = new Date();
            String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
            order.setUpdateDate(curDateStr);
            // 更新订单支付状态
            payOrderMapper.updateOrderStatus(order);
            // 剩余待付金额
            order.setPendingPayments(MathUtil.sub(order.getPendingPayments(), balancePayFee).doubleValue());
            // 修改账户余额
            double userMoney = baseMapper.getUserMoney(order.getUserId());
            if (OrderStatus.UNCONFIRMED.getCode() != order.getStatus()) {
                userMoney = MathUtil.sub(userMoney, balancePayFee).doubleValue();
                //保存收款记录
                productService.saveDiscernReceipt(balancePayFee, orderId, "【订单支付,余额抵扣】");
            }
            availableMoney = MathUtil.sub(availableMoney, balancePayFee).doubleValue();
            payOrderMapper.updateMoneyByUserId(userMoney, availableMoney, order.getUserId());
            log.info("【订单支付,余额抵扣】>>>>>>>>>>>更新用户余额(update[user])userId：" + order.getUserId() + "，orderId：" + orderId);
            //保存余额到余额收支记录
            BalanceRecordPo balanceRecord = new BalanceRecordPo();
            balanceRecord.setUserId(order.getUserId());
            balanceRecord.setType(2);
            balanceRecord.setBalanceType(1);
            balanceRecord.setAddDate(new Date());
            balanceRecord.setAmount(balancePayFee);
            balanceRecord.setOrderId(orderId);
            balanceRecord.setRemark("订单支付,余额抵扣");
            balanceRecord.setDelFlag(0);
            // 保存 余额收支记录
            baseMapper.insertBalanceRecord(balanceRecord);
            log.info("【订单支付,余额抵扣】>>>>>>>>>>>>>>>>>>>>>>>>>>新增用户余额收支记录(insert[cm_user_balance_record])orderId：" + orderId);
        }
        Map<String, Object> map = new HashMap<>(2);
        map.put("order", order);
        map.put("balancePayFee", balancePayFee);
        return ResponseJson.success(map);
    }

    /**
     * 获取线上支付全局开关状态
     */
    @Override
    public ResponseJson<Integer> getPayOnLineSwitch() {
        Integer status = payOrderMapper.getPayOnLineSwitch();
        return ResponseJson.success(status);
    }

    /**
     * 收银台数据显示
     *
     * @param orderId 订单Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getCheckoutCounter(Integer orderId) {
        // 订单信息
        OrderVo order = orderCommonMapper.getOrderByOrderId(orderId);
        if (null == order) {
            return ResponseJson.error("订单不存在", null);
        }
        // 机构信息
        String userName = baseMapper.getUserNameByUserId(order.getUserId());
        // 支付记录
        List<DiscernReceiptVo> discernReceiptList = orderCommonMapper.getDiscernReceipt(order.getOrderId(), order.getShopOrderIds());
        if (!discernReceiptList.isEmpty()) {
            AtomicDouble receiptAmount = new AtomicDouble(0d);
            AtomicBoolean offlineFlag = new AtomicBoolean(false);
            discernReceiptList.forEach(discernReceipt -> {
                if (2 == discernReceipt.getPayWay()) {
                    offlineFlag.set(true);
                }
                if (3 == discernReceipt.getReceiptStatus()) {
                    receiptAmount.set(MathUtil.add(receiptAmount.get(), discernReceipt.getAssociateAmount()).doubleValue());
                }
            });
            if (offlineFlag.get()) {
                return ResponseJson.error("已经线下支付过，只能线下支付！", null);
            }
            order.setReceiptAmount(receiptAmount.get());
        }
        // 商品数据
        List<OrderProductVo> orderProductList = orderCommonMapper.getOrderProductByOrderId(orderId);
        //过滤运费商品
        orderProductList.removeIf(product -> product.getShopId() == 998);
        // 是否有商品发票属性的限制
        orderProductList.forEach(orderProduct -> {
            // 商品含税未知 or 订单选择开企业发票,商品不含税不能开票
            boolean productTaxFlag = (null == orderProduct.getIncludedTax()) || (null == orderProduct.getInvoiceType()) || (Integer.valueOf(2).equals(orderProduct.getIncludedTax()))
                                     || (Integer.valueOf(2).equals(order.getInvoiceFlag()) && (Integer.valueOf(0).equals(orderProduct.getIncludedTax()) && Integer.valueOf(3).equals(orderProduct.getInvoiceType())));
            if (productTaxFlag) {
                order.setInvoiceStatus(true);
            }
        });
        // 返回数据
        Map<String, Object> map = new HashMap<>();
        map.put("userName", userName);
        map.put("order", order);
        map.put("discernReceipt", discernReceiptList);
        map.put("orderProductList", orderProductList);

        return ResponseJson.success(map);
    }

    /**
     * 生成网银支付链接
     *
     * @param payLinkDto {
     *                   orderId       订单ID
     *                   }
     */
    @Override
    public ResponseJson<String> getPayLink(PayLinkDto payLinkDto) {
        // 订单信息
        OrderVo order = orderCommonMapper.getOrderByOrderId(payLinkDto.getOrderId());
        if (null == order) {
            return ResponseJson.error("订单不存在", null);
        }
        // 时间戳
        long time = System.currentTimeMillis() / 1000;
        String linkLogo = CodeUtil.randomLetter(4);
        try {
            linkLogo = Md5Util.MD5To16Bit(linkLogo + time);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Date date = new Date();
        GregorianCalendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, 1);
        OrderPayLinkVo orderPayLink = payOrderMapper.getOrderPayLink(payLinkDto.getOrderId(), null);
        if (null == orderPayLink) {
            orderPayLink = new OrderPayLinkVo();
        }
        orderPayLink.setOrderId(payLinkDto.getOrderId());
        orderPayLink.setPayType(payLinkDto.getPayType());
        orderPayLink.setUnpaidAmount(payLinkDto.getUnpaidAmount());
        orderPayLink.setRedirectLink(redirectLink);
        orderPayLink.setLinkLogo(linkLogo);
        orderPayLink.setGenerateTime(date);
        orderPayLink.setEffectiveTime(calendar.getTime());
        orderPayLink.setPayStatus(0);
        orderPayLink.setDelFlag(0);
        if (null == orderPayLink.getId()) {
            payOrderMapper.insertOrderPayLink(orderPayLink);
        } else {
            payOrderMapper.updateOrderPayLink(orderPayLink);
        }
        String link = orderPayLink.getRedirectLink() + "?linkLogo=" + linkLogo;
        return ResponseJson.success(link);
    }

    /**
     * 支付链接重定向到页面
     *
     * @param linkLogo 链接标识
     */
    @Override
    public void jumpPayPage(String linkLogo, HttpServletResponse response) throws IOException {
        response.sendRedirect(linkPage + "?linkLogo=" + linkLogo);
    }

    /**
     * 重定向页面的支付数据
     *
     * @param linkLogo 链接标识
     */
    @Override
    public ResponseJson<Map<String, Object>> linkPayData(String linkLogo) {
        OrderPayLinkVo orderPayLink = payOrderMapper.getPayLinkByLinkLogo(linkLogo);
        if (null == orderPayLink) {
            // 已重新生成链接
            return ResponseJson.error("链接已更新,请重新获取", null);
        }
        //链接状态,0成功
        int code = 0;
        // 订单信息
        OrderVo order = orderCommonMapper.getOrderByOrderId(orderPayLink.getOrderId());
        // 支付记录
        List<DiscernReceiptVo> discernReceiptList = orderCommonMapper.getDiscernReceipt(order.getOrderId(), order.getShopOrderIds());
        if (orderPayLink.getEffectiveTime().compareTo(new Date()) < 0) {
            //链接失效
            code = -3;
        }
        if (null != discernReceiptList && discernReceiptList.size() > 0) {
            for (DiscernReceiptVo discernReceipt : discernReceiptList) {
                if (2 == discernReceipt.getPayWay()) {
                    // 已线下支付
                    code = -2;
                    break;
                }
            }
        }
        if (1 == orderPayLink.getPayStatus()) {
            //链接已支付
            code = 5;
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String time = dateFormat.format(orderPayLink.getEffectiveTime());
        Map<String, Object> map = new HashMap<>(3);
        map.put("orderPayLink", orderPayLink);
        map.put("code", code);
        map.put("time", time);
        return ResponseJson.success(map);
    }

    /**
     * 微信线上支付
     *
     * @param payDto  {
     *                orderId      订单ID
     *                payAmount     支付金额，单位分，必须大于2
     *                code          微信小程序code
     *                state         微信公众号state参数
     *                productId     二手发布商品id
     *                returnUrl     页面回调地址
     *                }
     * @param headers HttpHeaders
     */
    @Override
    public ResponseJson<JSONObject> payByWeChat(PayDto payDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payDto, payParam);
        // 微信线上支付
        payParam.setPayWay("WEIXIN");
        String openId = null;
        if (null == payParam.getState()) {
            // 小程序微信授权获取登录信息
            ResponseJson<Map<String, Object>> appletsInfo = weChatService.getInfoMapByApplets(payParam.getCode(), headers, 1);
            if (appletsInfo.getCode() == -1) {
                return ResponseJson.error(appletsInfo.getMsg(), null);
            }
            Map<String, Object> infoData = appletsInfo.getData();
            openId = (String) infoData.get(WeChatService.Keys.OPEN_ID);
            if (StringUtils.isEmpty(openId)) {
                return ResponseJson.error("微信小程序获取openId失败！", null);
            }
            //小程序微信快捷支付
            payParam.setPayType("MINIAPP_WEIXIN");
        } else {
            try {
                // 微信公众号，通过code获取微信用户信息
                Map<String, Object> map = weChatService.getInfoMapByWeb(payParam.getCode(), "crm");
                openId = (String) map.get(WeChatService.Keys.OPEN_ID);
            } catch (Exception e) {
                log.error("try-catch:",e);
            }
            if (StringUtils.isEmpty(openId)) {
                return ResponseJson.error("微信公众号获取openId失败！", null);
            }
            //pc微信扫码支付,微信公众号支付
            payParam.setPayType("JSAPI_WEIXIN");
        }
        payParam.setOpenId(openId);
        payParam.setNotifyUrl(notifyUrl);
        log.info("【微信支付】>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>获取openId:" + openId);
        return toPayOrder(payParam, headers);
    }

    /**
     * 支付宝线上支付
     *
     * @param payDto  {
     *                orderId      订单ID
     *                payAmount     支付金额，单位分，必须大于2
     *                returnUrl     页面回调地址
     *                }
     * @param headers HttpHeaders
     */
    @Override
    public ResponseJson<JSONObject> payByAlipay(PayDto payDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payDto, payParam);
        //支付宝支付
        payParam.setPayWay("ALIPAY");
        payParam.setPayType("ALIPAY_H5");
        payParam.setNotifyUrl(notifyUrl);
        log.info("【支付宝支付】>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>orderId:" + payParam.getOrderId());
        return toPayOrder(payParam, headers);
    }

    /**
     * 银联线上支付
     *
     * @param payDto  {
     *                orderId       订单ID
     *                payAmount     支付金额，单位分，必须大于2
     *                returnUrl     页面回调地址
     *                bankCode      银行编码(银联支付使用)
     *                userType      用户类型(银联支付使用)企业:ENTERPRISE，个人:USER
     *                }
     * @param headers HttpHeaders
     */
    @Override
    public ResponseJson<JSONObject> payByUnionPay(PayDto payDto, HttpHeaders headers) {
        PayParamBo payParam = new PayParamBo();
        // payDto -> payParam
        BeanUtils.copyProperties(payDto, payParam);
        //支付宝支付
        payParam.setPayWay("UNIONPAY");
        payParam.setPayType("GATEWAY_UNIONPAY");
        payParam.setNotifyUrl(notifyUrl);
        log.info("【银联支付】>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>orderId:" + payParam.getOrderId());
        return toPayOrder(payParam, headers);
    }

    /**
     * 订单线上支付
     */
    private ResponseJson<JSONObject> toPayOrder(PayParamBo payParam, HttpHeaders headers) {
        // 订单信息
        OrderVo order = orderCommonMapper.getOrderByOrderId(payParam.getOrderId());
        if (null == order) {
            return ResponseJson.error("订单不存在", null);
        }
        if (OrderStatus.UNCONFIRMED.getCode() == order.getStatus() ||
                OrderStatus.RECEIVED_AND_FULL_SHIPPED.getCode() == order.getStatus() ||
                OrderStatus.TRANSACTION_COMPLETED.getCode() == order.getStatus() ||
                OrderStatus.ORDER_COMPLETED.getCode() == order.getStatus() ||
                OrderStatus.CLOSED.getCode() == order.getStatus() ||
                OrderStatus.FULL_RETURNED.getCode() == order.getStatus()) {
            return ResponseJson.error("订单状态错误", null);
        }
        // 支付记录
        List<DiscernReceiptVo> discernReceiptList = orderCommonMapper.getDiscernReceipt(order.getOrderId(), order.getShopOrderIds());
        if (!discernReceiptList.isEmpty()) {
            AtomicReference<Boolean> offlineFlag = new AtomicReference<>(false);
            discernReceiptList.forEach(discernReceipt -> {
                if (2 == discernReceipt.getPayWay()) {
                    offlineFlag.set(true);
                }
            });
            if (offlineFlag.get()) {
                return ResponseJson.error("已经线下支付过，只能线下支付！", null);
            }
        }
        log.info(">>>>>>>>>>>>>>本次付款金额:" + payParam.getPayAmount());
        if (MathUtil.compare(MathUtil.mul(order.getPayableAmount(), 100), payParam.getPayAmount()) < 0) {
            return ResponseJson.error("付款金额错误！", null);
        }
        // 定义支付请求结果
        JSONObject result;
        try {
            // 时间戳
            long time = System.currentTimeMillis() / 1000;
            // 用户IP地址
            String userIp = headers.getFirst("X-CLIENT-IP");
            JSONObject json = new JSONObject();
            json.put("merAccount", PayUtil.merAccount);
            json.put("merNo", PayUtil.merNo);
            json.put("time", time);
            // 支付金额
            json.put("amount", payParam.getPayAmount());
            json.put("payWay", payParam.getPayWay());
            json.put("payType", payParam.getPayType());
            json.put("userIp", userIp);
            json.put("returnUrl", payParam.getReturnUrl());
            json.put("notifyUrl", payParam.getNotifyUrl());
            if (org.apache.commons.lang.StringUtils.isNotBlank(payParam.getBankCode())) {
                json.put("bankCode", payParam.getBankCode());
            }
            if (org.apache.commons.lang.StringUtils.isNotBlank(payParam.getUserType())) {
                json.put("userType", payParam.getUserType());
            }
            if (org.apache.commons.lang.StringUtils.isNotBlank(payParam.getOpenId())) {
                json.put("openId", payParam.getOpenId());
            }
            String environment = "";
            if (payParam.getNotifyUrl().contains("18002")) {
                environment = "DEV";
            } else if (payParam.getNotifyUrl().contains("core-b")) {
                environment = "BETA";
            }
            // 商户订单号
            String orderId = order.getOrderNo() + "T" + time + environment;
            json.put("orderId", orderId);
            //商品名称
            String product = "采美订单" + order.getOrderNo();
            json.put("product", product);
            //支付类型
            String payType = PayUtil.getPayTypeId(payParam.getPayType(), payParam.getUserType());
            String attach = order.getOrderId() + "," + payType;
            json.put("attach", attach);
            String sign = PayUtil.getPaySign(json, PayUtil.merKey);
            json.put("sign", sign);
            // 私钥加密
            String data = RSAUtil.privateEncrypt(json.toJSONString(), PayUtil.merKey);
            // 提交
            result = PayUtil.httpGet("https://platform.mhxxkj.com/paygateway/mbpay/order/v1", PayUtil.merAccount, data);
        } catch (Exception e) {
            log.error("错误信息", e);
            return ResponseJson.error("支付失败！", null);
        }
        String code = result.getString("code");
        if (!"000000".equals(code)) {
            String msg = result.getString("msg");
            log.info("第三方支付失败>>>>>>>msg:" + msg);
            return ResponseJson.error(msg, null);
        }
        return ResponseJson.success(result);
    }

    /**
     * 支付异步通知回调
     */
    @Override
    public String paymentCallback(String data) throws NoSuchAlgorithmException, InvalidKeySpecException {
        log.info("******************** 支付异步回调 start *******************");
        // 公钥解密
        JSONObject json = PayUtil.publicKeyDecrypt(data, PayUtil.publicKey);
        log.info("【支付异步回调】>>>>>>>>>>>>>>公钥解密:" + json);
        // 公钥验签
        String signaa = json.getString("sign");
        json.remove("sign");
        String signbb = PayUtil.getPaySign(json, PayUtil.publicKey);
        if (!signaa.equals(signbb)) {
            return "验签失败";
        }
        // 订单状态
        String orderStatus = json.getString("orderStatus");
        // 附加数据，下单时若有传输则原样返回，下单时为空，则不返回该数据
        String attach = json.getString("attach");
        // 平台唯一流水号
        String mbOrderId = json.getString("mbOrderId");
        // 商户唯一订单号
        String orderRequestNo = json.getString("orderId");
        // 订单金额，以元为单位
        Double amount = json.getDouble("amount");
        log.info("【支付异步回调】>>>>>>>>>>>>>>支付订单状态:" + orderStatus);
        if ("FAILED".equals(orderStatus)) {
            return "支付失败";
        }
        String[] split = attach.split(",");
        //订单id
        Integer orderId = Integer.valueOf(split[0]);
        //支付类型
        String payType = split[1];
        // 订单信息
        OrderVo order = orderCommonMapper.getOrderByOrderId(orderId);
        if (null == order) {
            return "订单不存在";
        }
        // 支付记录
        List<DiscernReceiptVo> discernReceiptList = orderCommonMapper.getDiscernReceipt(order.getOrderId(), order.getShopOrderIds());
        double receiptAmount = 0d;
        if (null != discernReceiptList && discernReceiptList.size() > 0) {
            for (DiscernReceiptVo discernReceipt : discernReceiptList) {
                receiptAmount = MathUtil.add(receiptAmount, discernReceipt.getAssociateAmount()).doubleValue();
            }
        }
        order.setReceiptAmount(MathUtil.add(receiptAmount, amount).doubleValue());
        log.info("【支付异步回调】>>>>>>>>>>>>>>已付金额+本次支付金额：" + order.getReceiptAmount());
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        if (MathUtil.compare(order.getPayableAmount(), order.getReceiptAmount()) == 0) {
            /*
             * 订单全部支付
             * 0待确认,11待收待发,12待收部发,13待收全发,21部收待发,22部收部发,23部收全发,
             * 31已收待发,32已收部发,33已收全发,4交易完成,5订单完成,6已关闭,7交易全退
             */
            if (OrderStatus.UNRECEIVED_AND_UNSHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_UNSHIPPED.getCode() == order.getStatus()) {
                order.setStatus(31);
            } else if (OrderStatus.UNRECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus()) {
                order.setStatus(32);
            } else {
                order.setStatus(33);
            }
            order.setPayFlag(1);
            order.setOnlinePayFlag(0);
            //（收款买家）收款状态：1待收款、2部分收款、3已收款
            order.setReceiptStatus(3);
            log.info("【支付异步回调】>>>>>>>>>>>>>>订单(全部支付),修改订单状态：" + order.getStatus() + "，orderId：" + orderId);
        } else {
            // 部分支付
            if (OrderStatus.UNRECEIVED_AND_UNSHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_UNSHIPPED.getCode() == order.getStatus()) {
                order.setStatus(21);
            } else if (OrderStatus.UNRECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus() || OrderStatus.PART_RECEIVED_AND_PART_SHIPPED.getCode() == order.getStatus()) {
                order.setStatus(22);
            } else {
                order.setStatus(23);
            }
            order.setOnlinePayFlag(0);
            //（收款买家）收款状态：1待收款、2部分收款、3已收款
            order.setReceiptStatus(2);
            log.info("【支付异步回调】>>>>>>>>>>>>>>订单(部分支付),修改订单状态：" + order.getStatus() + "，orderId：" + orderId);
        }
        // 更新付款次数
        Integer paySuccessCounter = (null == order.getPaySuccessCounter()) ? 1 : order.getPaySuccessCounter() +1;
        order.setPaySuccessCounter(paySuccessCounter);
        order.setUpdateDate(curDateStr);
        // 更新订单支付状态
        payOrderMapper.updateOrderStatus(order);
        // 修改支付链接状态
        OrderPayLinkVo orderPayLink = payOrderMapper.getOrderPayLink(orderId, amount);
        if (null != orderPayLink && ("12".equals(payType) || "17".equals(payType))) {
            orderPayLink.setPayStatus(1);
            payOrderMapper.updateOrderPayLinkStatus(orderPayLink);
        }
        // 保存收款记录
        DiscernReceiptPo discernReceipt = new DiscernReceiptPo();
        discernReceipt.setPayWay(1);
        discernReceipt.setPayType(Integer.valueOf(payType));
        discernReceipt.setReceiptType(1);
        discernReceipt.setReceiptStatus(3);
        discernReceipt.setReceiptAmount(amount);
        discernReceipt.setConfirmType(4);
        discernReceipt.setRePayFlag(1);
        discernReceipt.setFormData(json.toJSONString());
        discernReceipt.setReceiptDate(curDateStr);
        discernReceipt.setConfirmDate(curDateStr);
        discernReceipt.setReviewDate(curDateStr);
        discernReceipt.setUpdateDate(curDateStr);
        discernReceipt.setDelFlag(0);
        // 保存 收款记录
        baseMapper.insertDiscernReceipt(discernReceipt);
        log.info("【支付异步回调】>>>>>>>>>>>保存识别款项(insert[cm_discern_receipt])id：" + discernReceipt.getId() + "，orderId：" + orderId);
        // 收款项和订单关系表
        OrderReceiptRelationPo relation = new OrderReceiptRelationPo();
        relation.setReceiptId(discernReceipt.getId());
        relation.setOrderId(orderId);
        relation.setAssociateAmount(amount);
        relation.setMbOrderId(mbOrderId);
        relation.setOrderRequestNo(orderRequestNo);
        relation.setSplitStatus(0);
        relation.setRelationType(2);
        relation.setDelFlag(0);
        // 保存 收款项和订单关系
        baseMapper.insertOrderReceiptRelation(relation);
        log.info("【支付异步回调】>>>>>>>>>>>收款项和订单关系(insert[cm_receipt_order_relation])id：" + relation.getId() + "，orderId：" + orderId);

        // 商品数据
        List<OrderProductVo> orderProductList = orderCommonMapper.getOrderProductByOrderId(orderId);
        // 判断是否是充值商品
        int rechargeFlag = 0;
        // 缴纳订金订单
        int[] productId1 = {6060, 6061, 6062, 6063, 6064};
        // 充值余额订单
        int[] productId2 = {6065, 6066, 6067, 6068, 6069};
        for (OrderProductVo product : orderProductList) {
            if (ArrayUtils.contains(productId1, product.getProductId())) {
                rechargeFlag = 1;
                break;
            }
            if (ArrayUtils.contains(productId2, product.getProductId())) {
                rechargeFlag = 2;
                break;
            }
        }
        if (rechargeFlag > 0) {
            // 账户余额
            double oldUserMoney = baseMapper.getUserMoney(order.getUserId());
            double userMoney = MathUtil.add(oldUserMoney, amount).doubleValue();
            // 可用余额
            Double oldAvailableMoney = baseMapper.getAbleUserMoney(order.getUserId());
            double availableMoney = MathUtil.add(oldAvailableMoney, amount).doubleValue();
            payOrderMapper.updateMoneyByUserId(userMoney, availableMoney, order.getUserId());
            log.info("【支付异步回调】>>>>>>>>>>>更新用户余额(update[user])userId：" + order.getUserId() + "，orderId：" + orderId);
            //保存余额到余额收支记录
            BalanceRecordPo balanceRecord = new BalanceRecordPo();
            balanceRecord.setUserId(order.getUserId());
            balanceRecord.setType(1);
            if (rechargeFlag == 1) {
                balanceRecord.setBalanceType(6);
            } else {
                balanceRecord.setBalanceType(7);
            }
            balanceRecord.setAddDate(new Date());
            balanceRecord.setAmount(amount);
            balanceRecord.setOrderId(orderId);
            balanceRecord.setRemark("订单商品充值余额");
            balanceRecord.setDelFlag(0);
            // 保存 余额收支记录
            baseMapper.insertBalanceRecord(balanceRecord);
            log.info("【支付异步回调】>>>>>>>>>>>>>>>>>>>>>>>>>>订单商品充值余额(insert[cm_user_balance_record])orderId：" + orderId);
        }

        // 线上支付与自主下单送豆
        if (3 == order.getReceiptStatus() && 0 == order.getSecondHandOrderFlag()) {
            UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
            beansHistory.setUserId(order.getUserId());
            beansHistory.setOrderId(orderId);
            beansHistory.setBeansType(6);
            beansHistory.setType(1);
            beansHistory.setNum(200);
            beansHistory.setPushStatus(0);
            beansHistory.setAddTime(date);
            beansHistory.setDelFlag(0);
            // 保存 采美豆使用记录
            baseMapper.insertBeansHistory(beansHistory);
            // 用户采美豆
            Integer userBeans = baseMapper.getUserBeans(order.getUserId());
            // 更新用户剩余采美豆数量
            userBeans = userBeans + beansHistory.getNum();

            // 自主下单且没有促销活动送豆
            Integer orderPromotionsId = payOrderMapper.getOrderPromotionsId(order.getOrderId());
            if (1 == order.getOrderType() && null == orderPromotionsId) {
                Double payTotalFee = order.getPayTotalFee();
                int num;
                if (MathUtil.compare(1000, payTotalFee) > -1) {
                    num = 0;
                } else if (MathUtil.compare(5000, payTotalFee) > -1) {
                    num = 1000;
                } else if (MathUtil.compare(10000, payTotalFee) > -1) {
                    num = 2500;
                } else if (MathUtil.compare(25000, payTotalFee) > -1) {
                    num = 5000;
                } else if (MathUtil.compare(100000, payTotalFee) > -1) {
                    num = 7500;
                } else if (MathUtil.compare(250000, payTotalFee) > -1) {
                    num = 10000;
                } else if (MathUtil.compare(500000, payTotalFee) > -1) {
                    num = 12500;
                } else {
                    num = 20000;
                }
                beansHistory.setBeansType(5);
                beansHistory.setNum(num);
                if (num > 0) {
                    // 保存 采美豆使用记录
                    baseMapper.insertBeansHistory(beansHistory);
                    // 更新用户剩余采美豆数量
                    userBeans = userBeans + beansHistory.getNum();
                }
            }
            baseMapper.updateUserBeans(beansHistory.getUserId(), userBeans);
            log.info("【支付异步回调】>>>>>更新用户采美豆(update[user(userBeans:"+ userBeans +")])，userId:" + beansHistory.getUserId());
        }
        // 已支付短信推送
        boolean smsPushFlag = !orderRequestNo.contains("BETA") && !orderRequestNo.contains("DEV");
        String bindMobile = baseMapper.getBindMobileByUserId(order.getUserId());
        if (smsPushFlag && StringUtils.isNotBlank(bindMobile)) {
            String shortLink = remoteCallService.getShortLink(8, 6, "https://www.caimei365.com/user/order/detail.html?orderId=" + orderId);
            String content = "您已成功支付订单（订单编号：" + order.getOrderNo() + "）全部款项，支付总金额￥" + order.getPayTotalFee() + "，采美平台将立即安排发货。您可关注采美公众号或者访问采美微信小程序和网站查看订单。" +
                    "平台公众号：微信搜索“采美365网”; 微信小程序：微信搜索“采美采购商城”；网址：www.caimei365.com/t/" + shortLink;
            boolean sendSms = remoteCallService.getSendSms(6, bindMobile, content);
            if (!sendSms) {
                log.info("取消订单推送失败，orderId>>>>" + orderId);
            }
        }
        return "SUCCESS";
    }

    /**
     * 判断此次支付是否完成
     *
     * @param orderId           订单id
     * @param paySuccessCounter 付款次数
     */
    @Override
    public ResponseJson<String> payWhetherSuccess(Integer orderId, Integer paySuccessCounter) {
        Integer dbPayCounter = payOrderMapper.getDbPaySuccessCounter(orderId);
        if (paySuccessCounter.equals(dbPayCounter)) {
            return ResponseJson.error(-2, "支付失败", "付款次数异常");
        } else if (dbPayCounter > paySuccessCounter) {
            return ResponseJson.success("支付成功");
        } else {
            return ResponseJson.error("支付异常", null);
        }
    }

    /**
     * 查询本次支付订单结果
     *
     * @param mbOrderId 平台唯一流水号
     */
    @Override
    public ResponseJson<JSONObject> getPayOrderResult(String mbOrderId) {
        // 时间戳
        long time = System.currentTimeMillis() / 1000;
        JSONObject json = new JSONObject();
        json.put("merAccount", PayUtil.merAccount);
        json.put("mbOrderId", mbOrderId);
        json.put("time", time);
        // 生成RSA签名
        String sign = PayUtil.getPaySign(json, PayUtil.merKey);
        json.put("sign", sign);
        String data = "";
        try {
            // 私钥加密
            data = PayUtil.privateKeyEncrypt(json, PayUtil.merKey);
        } catch (Exception e) {
            log.error("【查询本次支付订单结果】>>>>>错误信息:", e);
        }
        JSONObject result = PayUtil.httpGet("https://platform.mhxxkj.com/paygateway/mbpay/order/query/v1_1", PayUtil.merAccount, data);
        String code = result.getString("code");
        if (!"000000".equals(code)) {
            String msg = result.getString("msg");
            log.info("【查询本次支付订单结果】>>>>>>>>>>>>>>第三方查询订单失败>msg:" + msg);
            return ResponseJson.error(msg, null);
        }
        return ResponseJson.success(result);
    }

    /**
     * 延时分账
     *
     * @param callUrl  回调地址
     */
    @Override
    public void delayedSplitting(String callUrl) {
        log.info("【延时分账】>>>>>>>>>>延时分账,每一小时执行一次");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String currentTime = format.format(calendar.getTime());
        // 查询未分账已支付收款
        List<OrderReceiptRelationPo> orderRelations = payOrderMapper.getUndividedPaidReceipt(currentTime);
        if (orderRelations != null && orderRelations.size() > 0) {
            for (OrderReceiptRelationPo orderRelation : orderRelations) {
                log.info("【延时分账】>>>>>>>>>>订单id:" + orderRelation.getOrderId() + ",进入延时分账");
                // 订单信息
                OrderVo order = orderCommonMapper.getOrderByOrderId(orderRelation.getOrderId());
                PayParamBo payParam = new PayParamBo();
                payParam.setPayAmount(MathUtil.mul(orderRelation.getAssociateAmount(), 100).intValue());
                if (12 == orderRelation.getPayType()) {
                    // 网银支付
                    payParam.setPayWay("UNIONPAY");
                }
                List<SplitAccountPo> splitBillDetail = setSplitAccountDetail(order, payParam);
                List<Map<String, String>> maps = new ArrayList<>();
                List<ShopOrderVo> shopOrderList = orderCommonMapper.getShopOrderListByOrderId(order.getOrderId());
                //供应商子商户总金额
                for (ShopOrderVo shopOrder : shopOrderList) {
                    double shopTotalAmount = 0.00;
                    String subUserNo = "";
                    for (SplitAccountPo account : splitBillDetail) {
                        if (4 == account.getType() && shopOrder.getShopId().equals(account.getShopId())) {
                            shopTotalAmount = MathUtil.add(shopTotalAmount, account.getSplitAccount()).doubleValue();
                            subUserNo = account.getSubUserNo();
                        }
                    }
                    addMaps(maps, shopTotalAmount, subUserNo);
                }
                //公账-专票总金额,私账-无票总金额,公账-普票总金额
                double totalAmount1 = 0.00;
                double totalAmount2 = 0.00;
                double totalAmount3 = 0.00;
                for (SplitAccountPo account : splitBillDetail) {
                    if (1 == account.getType()) {
                        totalAmount1 = MathUtil.add(totalAmount1, account.getSplitAccount()).doubleValue();
                    } else if (2 == account.getType()) {
                        totalAmount2 = MathUtil.add(totalAmount2, account.getSplitAccount()).doubleValue();
                    } else if (3 == account.getType()) {
                        totalAmount3 = MathUtil.add(totalAmount3, account.getSplitAccount()).doubleValue();
                    }
                }
                addMaps(maps, totalAmount1, PayUtil.publicAccountNo);
                addMaps(maps, totalAmount2, PayUtil.privateAccountNo);
                addMaps(maps, totalAmount3, PayUtil.commonInvoiceNo);
                String parameters = JSON.toJSONString(maps);
                log.info("【延时分账】>>>>>>>>>>分账参数: " + parameters);

                //第三方分账接口
                JSONObject result = null;
                try {
                    // 时间戳
                    long time = System.currentTimeMillis() / 1000;
                    JSONObject json = new JSONObject();
                    json.put("merAccount", PayUtil.merAccount);
                    json.put("orderId", orderRelation.getOrderRequestNo());
                    json.put("requestNo", orderRelation.getOrderRequestNo());
                    json.put("mbOrderId", orderRelation.getMbOrderId());
                    json.put("time", time);
                    json.put("splitBillDetail", parameters);
                    json.put("notifyUrl", callUrl);
                    log.info("回调接口>>>" + callUrl);
                    String sign = PayUtil.getPaySign(json, PayUtil.merKey);
                    json.put("sign", sign);
                    String data = PayUtil.privateKeyEncrypt(json, PayUtil.merKey);
                    result = PayUtil.httpGet("https://platform.mhxxkj.com/paygateway/mbpay/splitOrder/v1", PayUtil.merAccount, data);
                } catch (Exception e) {
                    log.error("【延时分账】>>>>>>>>>>错误信息", e);
                }
                if (result != null) {
                    String code = result.getString("code");
                    if (!"000000".equals(code)) {
                        String msg = result.getString("msg");
                        log.info("【延时分账】>>>>>>>>>>第三方延迟分账失败>>>>>>>msg:" + msg);
                    } else {
                        for (SplitAccountPo splitAccount : splitBillDetail) {
                            splitAccount.setMbOrderId(orderRelation.getMbOrderId());
                            splitAccount.setOrderRequestNo(orderRelation.getOrderRequestNo());
                            splitAccount.setPayStatus(1);
                            // 保存分账详情
                            payOrderMapper.insertSplitAccount(splitAccount);
                        }
                        log.info("【延时分账】>>>>>>>>>>此订单分账结束");
                    }
                }
            }
        }
    }

    /**
     * 分账参数添加
     */
    private void addMaps(List<Map<String, String>> maps, Double shopTotalAmount, String subUserNo) {
        if (MathUtil.compare(shopTotalAmount, 0) > 0) {
            Map<String, String> map = new HashMap<>(3);
            map.put("subUserNo", subUserNo);
            map.put("splitBillType", "2");
            map.put("splitBillValue", MathUtil.mul(shopTotalAmount, 100).toString());
            maps.add(map);
        }
    }

    /**
     * 分账详情
     */
    private List<SplitAccountPo> setSplitAccountDetail(OrderVo order, PayParamBo payParam) {
        List<SplitAccountPo> list = new ArrayList<>();
        // 本次支付金额,单位/元
        double payAmount = MathUtil.div(payParam.getPayAmount(), 100).doubleValue();
        // 待分账总金额
        double splitAmount = payAmount;
        // 总手续费
        double procedureFee;
        if ("UNIONPAY".equals(payParam.getPayWay())) {
            procedureFee = 8.00;
        } else {
            //手续费
            procedureFee = MathUtil.mul(payAmount, 0.0038, 2).doubleValue();
            if (MathUtil.compare(procedureFee, 0) == 0) {
                procedureFee = 0.01;
            }
        }
        splitAmount = MathUtil.sub(splitAmount, procedureFee).doubleValue();
        // 商品数据
        List<OrderProductVo> orderProductList = orderCommonMapper.getOrderProductByOrderId(order.getOrderId());
        for (OrderProductVo orderProduct : orderProductList) {
            double costPrice = MathUtil.mul(orderProduct.getCostPrice(), orderProduct.getNum()).doubleValue();
            // 不含税能开发票
            if (Integer.valueOf(0).equals(orderProduct.getIncludedTax()) && !Integer.valueOf(3).equals(orderProduct.getInvoiceType())) {
                //应付总税费
                Double payableTax = MathUtil.mul(orderProduct.getSingleShouldPayTotalTax(), orderProduct.getNum()).doubleValue();
                costPrice = MathUtil.add(costPrice, payableTax).doubleValue();
            }
            // 判断是否支付过
            Double paidAmount = payOrderMapper.getOrderProductPaidAmount(orderProduct.getOrderProductId());
            if (paidAmount != null && MathUtil.compare(paidAmount, 0) > 0) {
                costPrice = MathUtil.sub(costPrice, paidAmount).doubleValue();
            }
            if (paidAmount == null || MathUtil.compare(paidAmount, costPrice) < 0) {
                // 待分账金额>=本次待分账金额
                if (MathUtil.compare(splitAmount, costPrice) > -1) {
                    splitAmount = MathUtil.sub(splitAmount, costPrice).doubleValue();
                } else {
                    costPrice = splitAmount;
                    splitAmount = 0.00;
                }
                String commercialCode = payOrderMapper.getShopCommercialCode(orderProduct.getShopId());
                SplitAccountPo splitAccount = new SplitAccountPo();
                splitAccount.setOrderId(order.getOrderId());
                splitAccount.setOrderProductId(orderProduct.getOrderProductId());
                splitAccount.setShopId(orderProduct.getShopId());
                splitAccount.setSplitAccount(costPrice);
                splitAccount.setProductType(1);
                if (StringUtils.isNotBlank(commercialCode)) {
                    // 供应商拥有子商户号
                    splitAccount.setType(4);
                    splitAccount.setSubUserNo(commercialCode);
                } else {
                    if (3 == orderProduct.getInvoiceType()) {
                        // 不能开票，则分账到私账-无票
                        splitAccount.setType(2);
                        splitAccount.setSubUserNo(PayUtil.privateAccountNo);
                    } else if (1 == orderProduct.getInvoiceType()) {
                        // 开增值税发票，则分账到公账-专票
                        splitAccount.setType(1);
                        splitAccount.setSubUserNo(PayUtil.publicAccountNo);
                    } else if (2 == orderProduct.getInvoiceType()) {
                        // 开普通发票，则分账到公账-普票
                        splitAccount.setType(3);
                        splitAccount.setSubUserNo(PayUtil.commonInvoiceNo);
                    }
                }
                list.add(splitAccount);
                if (MathUtil.compare(splitAmount, 0) == 0) {
                    break;
                }
            }
        }
        // 付供应商运费,是以供应商为单位的
        if (MathUtil.compare(splitAmount, 0) > 0) {
            List<ShopOrderVo> shopOrderList = orderCommonMapper.getShopOrderListByOrderId(order.getOrderId());
            for (ShopOrderVo shopOrder : shopOrderList) {
                // 运费
                Double shopPostFee = shopOrder.getShopPostFee();
                if (MathUtil.compare(shopPostFee, 0) > 0) {
                    // 查询已支付运费
                    Double shipping = payOrderMapper.getPaidShipping(order.getOrderId(), shopOrder.getShopId());
                    shopPostFee = MathUtil.sub(shopPostFee, shipping).doubleValue();
                    if (MathUtil.compare(splitAmount, shopPostFee) > -1) {
                        splitAmount = MathUtil.sub(splitAmount, shopPostFee).doubleValue();
                    } else {
                        shopPostFee = splitAmount;
                        splitAmount = 0.00;
                    }
                    String commercialCode = payOrderMapper.getShopCommercialCode(shopOrder.getShopId());
                    SplitAccountPo splitAccount = new SplitAccountPo();
                    splitAccount.setOrderId(order.getOrderId());
                    splitAccount.setShopId(shopOrder.getShopId());
                    splitAccount.setSplitAccount(shopPostFee);
                    splitAccount.setProductType(2);
                    if (StringUtils.isNotBlank(commercialCode)) {
                        //供应商拥有子商户号
                        splitAccount.setType(4);
                        splitAccount.setSubUserNo(commercialCode);
                    } else {
                        //私账
                        splitAccount.setType(2);
                        splitAccount.setSubUserNo(PayUtil.privateAccountNo);
                    }
                    list.add(splitAccount);
                }
            }
        }
        //佣金,公账
        if (MathUtil.compare(splitAmount, 0) > 0) {
            SplitAccountPo splitAccount = new SplitAccountPo();
            splitAccount.setOrderId(order.getOrderId());
            splitAccount.setSplitAccount(splitAmount);
            splitAccount.setProductType(3);
            splitAccount.setType(1);
            splitAccount.setSubUserNo(PayUtil.publicAccountNo);
            list.add(splitAccount);
        }
        return list;
    }

    /**
     * 延时分账异步通知回调
     */
    @Override
    public String delayedSplittingCallback(String data) {
        try {
            log.info("******************** 延时分账异步通知回调 start *******************");
            // 公钥解密
            JSONObject json = PayUtil.publicKeyDecrypt(data, PayUtil.publicKey);
            log.info("公钥解密>>>>>>" + json);
            // 公钥验签
            String signaa = json.getString("sign");
            json.remove("sign");
            String signbb = PayUtil.getPaySign(json, PayUtil.publicKey);
            if (!signaa.equals(signbb)) {
                return "验签失败";
            }
            String mbOrderId = json.getString("mbOrderId");
            String status = json.getString("status");
            log.info("【延时分账回调】>>>>>>>>>>分账状态:" + status);
            if ("FAILED".equals(status)) {
                return "分账失败";
            }
            // 修改收款分账状态
            payOrderMapper.updateBySplitStatus(mbOrderId);

            List<SplitAccountPo> splitAccountList = payOrderMapper.getSplitAccountList(mbOrderId);
            if (splitAccountList != null && splitAccountList.size() > 0) {
                Integer orderId = splitAccountList.get(0).getOrderId();
                List<ShopOrderVo> shopOrderList = orderCommonMapper.getShopOrderListByOrderId(orderId);
                Integer shopOrderId = null;
                String shopOrderNo = "";
                for (SplitAccountPo account : splitAccountList) {
                    log.info("【延时分账回调】>>>>>>>>>>保存应付付供应商:" + account.getShopId());
                    // 本次付供应商金额(分账金额)
                    Double splitAmount = account.getSplitAccount();
                    orderId = account.getOrderId();
                    Integer shopId = account.getShopId();
                    for (ShopOrderVo shopOrder : shopOrderList) {
                        if (shopId.equals(shopOrder.getShopId())) {
                            shopOrderId = shopOrder.getShopOrderId();
                            shopOrderNo = shopOrder.getShopOrderNo();
                            // 已付供应商金额
                            Double paidAmount = payOrderMapper.getPaidShopAmount(shopOrderId);
                            Double paidShop = MathUtil.add(paidAmount, splitAmount).doubleValue();
                            shopOrder.setPayedShopAmount(paidShop);
                            if (MathUtil.compare(shopOrder.getShouldPayShopAmount(), paidShop) == 0) {
                                shopOrder.setPayStatus(3);
                            } else {
                                shopOrder.setPayStatus(2);
                            }
                            // 修改子订单付款状态及付款金额
                            payOrderMapper.updateShopOrderByPayStatus(shopOrderId, paidShop, shopOrder.getPayStatus());
                        }
                    }
                    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    String currentTime = format.format(new Date());
                    // 保存付款单表
                    PayShopPo payShop = new PayShopPo();
                    payShop.setShopId(shopId);
                    payShop.setName("线上支付分账");
                    payShop.setTotalAmount(splitAmount);
                    payShop.setWipePayment(0d);
                    payShop.setPayType(6);
                    payShop.setStatus(1);
                    payShop.setDelFlag(0);
                    payShop.setApplyTime(currentTime);
                    payShop.setReviewTime(currentTime);
                    payShop.setPayTime(currentTime);
                    payOrderMapper.insertPayShop(payShop);
                    // 保存 付供应商记录
                    PayShopRecordPo shopRecord = new PayShopRecordPo();
                    shopRecord.setShopId(shopId);
                    shopRecord.setShopOrderId(shopOrderId);
                    shopRecord.setShopOrderNo(shopOrderNo);
                    shopRecord.setPayAmount(splitAmount);
                    shopRecord.setWipePayment(0d);
                    shopRecord.setPayType(6);
                    shopRecord.setPayTime(currentTime);
                    shopRecord.setPayShopId(payShop.getId());
                    shopRecord.setStatus(1);
                    shopRecord.setDelFlag(0);
                    payOrderMapper.insertPayShopRecord(shopRecord);
                }
                // 子订单是否全部付款
                boolean isPay = true;
                for (ShopOrderVo shopOrder : shopOrderList) {
                    if (3 != shopOrder.getPayStatus()) {
                        isPay = false;
                        break;
                    }
                }
                // 修改主订单付款状态
                if (isPay) {
                    payOrderMapper.updateOrderByPayStatus(orderId, 3);
                } else {
                    payOrderMapper.updateOrderByPayStatus(orderId, 2);
                }
            }
        } catch (Exception e) {
            log.error("【延时分账回调】>>>>>>>>>>分账异步通知异常", e);
            return "分账失败";
        }
        return "SUCCESS";
    }

//    /**
//     * 手动临时分账异步通知回调
//     *
//     * @param data
//     */
//    @Override
//    public String tempSplittingCallback(String data) {
//        try {
//            log.info("******************** 临时手动分账异步通知回调 start *******************");
//            // 公钥解密
//            JSONObject json = PayUtil.publicKeyDecrypt(data, PayUtil.publicKey);
//            log.info("公钥解密>>>>>>" + json);
//            // 公钥验签
//            String signaa = json.getString("sign");
//            json.remove("sign");
//            String signbb = PayUtil.getPaySign(json, PayUtil.publicKey);
//            if (!signaa.equals(signbb)) {
//                return "验签失败";
//            }
//            String mbOrderId = json.getString("mbOrderId");
//            String status = json.getString("status");
//            log.info("【临时手动分账回调】>>>>>>>>>>分账状态:" + status);
//            if ("FAILED".equals(status)) {
//                return "分账失败";
//            }
//            // 修改收款分账状态
//            payOrderMapper.updateBySplitStatus(mbOrderId);
//        } catch (Exception e) {
//            log.error("【临时手动分账回调】>>>>>>>>>>分账异步通知异常", e);
//            return "分账失败";
//        }
//        return "SUCCESS";
//    }
//
//    /**
//     * 临时手动分账
//     *
//     * 订单：14814
//     *
//     * ￥730.00 分账给 20001793 (米花分账号)      公账-专票     10033578851 (易宝商户号)
//     * ￥740.00 分账给 20001924 (米花分账号)      私账-无票     10033806951 (易宝商户号)
//     */
//    @Override
//    public String temporarySplitting(){
//        log.info("【临时手动分账】订单Id：14814");
//        // 订单信息
//        List<SplitAccountPo> splitBillDetail = new ArrayList<>();
//        double splitAmount0 = 1470.00d;
//        double splitAmount1 = 730.00d;
//        double splitAmount2 = 740.00d;
//        // 手续费
//        double procedureFee = MathUtil.mul(splitAmount0, 0.0038, 4).doubleValue();
//        if (MathUtil.compare(splitAmount0, 0) == 0) {
//            procedureFee = 0.01;
//        }
//        splitAmount1 = MathUtil.sub(splitAmount1, procedureFee).doubleValue();
//
//        SplitAccountPo splitAccount1 = new SplitAccountPo();
//        splitAccount1.setOrderId(14814);
//        splitAccount1.setSplitAccount(splitAmount1);
//        splitAccount1.setProductType(3);
//        splitAccount1.setType(1);
//        splitAccount1.setSubUserNo(PayUtil.publicAccountNo);
//        splitBillDetail.add(splitAccount1);
//
//        SplitAccountPo splitAccount2 = new SplitAccountPo();
//        splitAccount2.setOrderId(14814);
//        splitAccount2.setSplitAccount(splitAmount2);
//        splitAccount2.setProductType(1);
//        splitAccount2.setType(2);
//        splitAccount2.setSubUserNo(PayUtil.privateAccountNo);
//        splitBillDetail.add(splitAccount2);
//
//        List<Map<String, String>> maps = new ArrayList<>();
//        //公账-专票总金额,私账-无票总金额,公账-普票总金额
//        double totalAmount1 = 0.00d;
//        double totalAmount2 = 0.00d;
//        double totalAmount3 = 0.00d;
//        for (SplitAccountPo account : splitBillDetail) {
//            if (1 == account.getType()) {
//                totalAmount1 = MathUtil.add(totalAmount1, account.getSplitAccount()).doubleValue();
//            } else if (2 == account.getType()) {
//                totalAmount2 = MathUtil.add(totalAmount2, account.getSplitAccount()).doubleValue();
//            } else if (3 == account.getType()) {
//                totalAmount3 = MathUtil.add(totalAmount3, account.getSplitAccount()).doubleValue();
//            }
//        }
//        addMaps(maps, totalAmount1, PayUtil.publicAccountNo);
//        addMaps(maps, totalAmount2, PayUtil.privateAccountNo);
//        addMaps(maps, totalAmount3, PayUtil.commonInvoiceNo);
//        String parameters = JSON.toJSONString(maps);
//        log.info("【临时手动分账】>>>>>>>>>>分账参数: " + parameters);
//
//        //第三方分账接口
//        JSONObject result = null;
//        try {
//            // 时间戳
//            long time = System.currentTimeMillis() / 1000;
//            JSONObject json = new JSONObject();
//            json.put("merAccount", PayUtil.merAccount);
//            json.put("orderId", "X162807235471784T1628072439");
//            json.put("requestNo", "X162807235471784T1628072439");
//            json.put("mbOrderId", "gpnpac79061628072439679010001720");
//            json.put("time", time);
//            json.put("splitBillDetail", parameters);
//            json.put("notifyUrl", "https://core.caimei365.com/order/pay/temp/split/callback");
//            log.info("回调接口>>>https://core.caimei365.com/order/pay/temp/split/callback");
//            String sign = PayUtil.getPaySign(json, PayUtil.merKey);
//            json.put("sign", sign);
//            String data = PayUtil.privateKeyEncrypt(json, PayUtil.merKey);
//            result = PayUtil.httpGet("https://platform.mhxxkj.com/paygateway/mbpay/splitOrder/v1", PayUtil.merAccount, data);
//        } catch (Exception e) {
//            log.error("【临时手动分账】>>>>>>>>>>错误信息", e);
//        }
//        if (result != null) {
//            String code = result.getString("code");
//            if (!"000000".equals(code)) {
//                String msg = result.getString("msg");
//                log.info("【临时手动分账】>>>>>>>>>>第三方延迟分账失败>>>>>>>msg:" + msg);
//            } else {
//                for (SplitAccountPo splitAccount : splitBillDetail) {
//                    splitAccount.setMbOrderId("gpnpac79061628072439679010001720");
//                    splitAccount.setOrderRequestNo("X162807235471784T1628072439");
//                    splitAccount.setPayStatus(1);
//                    // 保存分账详情
//                    payOrderMapper.insertSplitAccount(splitAccount);
//                    log.info("【临时手动分账】>>>>>>>>>>id:"+splitAccount.getId());
//                }
//                log.info("【临时手动分账】>>>>>>>>>>此订单分账结束");
//            }
//        }
//        return "临时手动分账";
//    };

}
