package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.feign.ToolsFeign;
import com.caimei365.order.feign.UserFeign;
import com.caimei365.order.mapper.MessagePushMapper;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.utils.CodeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 远程调用数据处理
 *
 * @author : Charles
 * @date : 2021/7/15
 */
@Slf4j
@Service
public class RemoteCallServiceImpl implements RemoteCallService {
    @Value("${spring.cloud.config.profile}")
    private String profile;
    @Resource
    private MessagePushMapper messagePushMapper;
    @Resource
    private ToolsFeign toolsFeign;
    @Resource
    private UserFeign userFeign;

    /**
     * 生成短链接
     *
     * @param length 链接长度
     * @param markId 跳转类型
     * @param url    跳转地址
     */
    @Override
    public String getShortLink(int length, int markId, String url) {
        String shortLink = CodeUtil.generateShortLink(length);
        Integer id = messagePushMapper.findIdByShortLink(shortLink);
        if (id != null && id > 0) {
            getShortLink(length, markId, url);
        }
        messagePushMapper.insertShortLink(markId, shortLink, url);
        return shortLink;
    }

    /**
     * 请求发短信接口
     *
     * @param markId  跳转类型
     * @param mobile  手机号
     * @param content 内容
     */
    @Override
    public boolean getSendSms(int markId, String mobile, String content) {
        AtomicBoolean returnValue = new AtomicBoolean(false);
        try {
            //测试环境手机号允许发短信
            List<String> list = new ArrayList<>();
            list.add("15917362709");
            list.add("15814011616");
            list.add("13100721916");
            list.add("15113936829");
            if ("prod".equals(profile) || list.contains(mobile)) {
                if (StringUtils.isNotBlank(mobile) && mobile.length() == 11) {
                    String regex = "^(1[3-9]\\d{9}$)";
                    Pattern pattern = Pattern.compile(regex);
                    Matcher matcher = pattern.matcher(mobile);
                    if (matcher.matches()) {
                        // 调用 ToolsFeign 发送短信
                        String jsonStr = toolsFeign.getSendSms(mobile, content);
                        JSONObject parseObject = JSONObject.parseObject(jsonStr);
                        if (0 == parseObject.getInteger("code")){
                            // 保存短信发送条数+count
                            messagePushMapper.updateSmsSendCount(markId, 1);
                            returnValue.set(true);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("短信推送异常:", e);
        }
        return returnValue.get();
    }

    /**
     * 微信小程序授权登录（调用user服务，发送服务间调用POST请求）
     *
     * @param code          微信code
     * @param encryptedData 微信加密数据
     * @param iv            微信加密算法的初始向量
     * @param headers       HttpHeaders
     */
    @Override
    public String appletsAuthorization(String code, String encryptedData, String iv, HttpHeaders headers) {
        AtomicReference<String> resultData = new AtomicReference<>("");
        // 获取当前微信小程序的环境
        String referer = headers.getFirst("Referer");
        try {
            // 调用 UserFeign 获取物流
            String jsonStr = userFeign.appletsAuthorization(code, encryptedData, iv, referer);
            JSONObject parseObject = JSONObject.parseObject(jsonStr);
            if (0 == parseObject.getInteger("code") || -4 == parseObject.getInteger("code")){
                // 授权登录成功（游客-4）
                resultData.set(parseObject.getString("data"));
            }
        } catch (Exception e) {
            log.error("获取微信小程序授权失败:", e);
        }
        return resultData.get();
    }

    /**
     * 智能获取物流信息
     *
     * @param number      快递单号
     * @param companyCode 快递公司的编码
     * @param mobile      收/寄件人的电话号码(顺丰必填)
     */
    @Override
    public String getLogisticsByNumber(String number, String companyCode, String mobile) {
        AtomicReference<String> resultData = new AtomicReference<>("");
        try {
            // 调用 ToolsFeign 获取物流
            String jsonStr = toolsFeign.getLogisticsByNumber(number, companyCode, mobile);
            log.info("智能获取物流信息:"+jsonStr);
            JSONObject parseObject = JSONObject.parseObject(jsonStr);
            if (0 == parseObject.getInteger("code")){
                // 获取物流信息成功
                resultData.set(parseObject.getString("data"));
            } else {
                // 获取物流失败，返回错误信息
                resultData.set(jsonStr);
            }
        } catch (Exception e) {
            log.error("获取物流信息失败:", e);
        }
        return resultData.get();
    }
}
