package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.ProductService;
import com.caimei365.order.mapper.*;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.RechargeDto;
import com.caimei365.order.model.po.*;
import com.caimei365.order.model.dto.SubmitDto;
import com.caimei365.order.model.bo.OrderParamBo;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.service.SubmitService;
import com.caimei365.order.utils.CodeUtil;
import com.caimei365.order.utils.ImageUtil;
import com.caimei365.order.utils.MathUtil;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;

import static com.alibaba.fastjson.JSON.parseArray;
import static com.alibaba.fastjson.JSON.parseObject;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/6
 */
@Slf4j
@Service
public class SubmitServiceImpl implements SubmitService {
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private SubmitMapper submitMapper;
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private ProductService productService;
    @Resource
    private OrderClubMapper orderClubMapper;
    @Resource
    private CartClubMapper cartClubMapper;
    @Resource
    private CartSellerMapper cartSellerMapper;
    @Resource
    private RemoteCallService remoteCallService;
    /**
     * 生成订单
     *
     * @param submitDto {
     *                  "clubId": 10708,            //机构ID
     *                  "unionId":"",               //微信unionId
     *                  "addressId": 2732,          //地址ID
     *                  "orderSource": 2,           //订单来源：1WWW、2CRM、4客服[适用后台下单]、5外单[适用后台下单]、6小程序[采美,星范]、7呵呵商城小程序、8维沙小程序
     *                  "cartType":3,               //购买类型:（1自主下单(购物车),2自主下单(立即购买)，3协销下单）
     *                  "serviceProviderId": 1378,  //协销ID（小程序忽略）
     *                  "clubCouponId": ""          //优惠券Id
     *                  "orderInfo": [              //【订单商品】
     *                      { "shopId":1001,                         // 供应商Id
     *                          "note":备注,
     *                          "productInfo":[                      // 商品id，数量，赠品数,商品类型
     *                              {"productId": 2789, "productNum": 1, "presentNum":0,"productType":2},
     *                              {"productId": 2789, "productNum": 1, "presentNum":0,"productType":0}
     *                          ]
     *                      },{多个供应商商品数据结构同上}
     *                  ],
     *                  "payInfo": {                //【订单金额】
     *                      "orderShouldPayFee": 609.11,
     *                      "balancePayFlag": 0,
     *                      "clauseId": "2",
     *                      "postage": "15",
     *                      "postageFlag": 1, //运费标志：0包邮 -1到付 1遵循运费规则
     *                      "userBeans": 100,//抵扣采美豆数量
     *                      "rebateFlag":0
     *                  },
     *                  "orderInvoice":             //【发票信息】
     *                          {"type": 0 // 不开发票  }
     *                      或： { // 普通发票
     *                              "type": 1,
     *                              "invoiceTitle": "企业抬头",
     *                              "invoiceTitleType": 1,
     *                              "corporationTaxNum": "XXX4156465465",
     *                              "invoiceContent": "明细 "
     *                          }
     *                      或： { // 增值税发票
     *                              "type": 2,
     *                              "invoiceTitle": "单位名称",
     *                              "corporationTaxNum": "NSRSBM97897",
     *                              "registeredAddress": "注册地址",
     *                              "registeredPhone": "15814011616",
     *                              "openBank": "开户银行",
     *                              "bankAccountNo": "987987465465464"
     *                          }
     *              }
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<Map<String, Object>> generateOrder(SubmitDto submitDto) {
        // 获取机构用户Id
        Integer clubUserId = baseMapper.getUserIdByClubId(submitDto.getClubId());
        if (null == clubUserId || clubUserId == 0){
            return ResponseJson.error("机构用户信息异常！", null);
        }
        JSONArray orderInfo = null;
        JSONObject payInfo = null;
        JSONObject orderInvoice = null;
        try {
            orderInfo = parseArray(submitDto.getOrderInfo());
            payInfo = parseObject(submitDto.getPayInfo());
            orderInvoice = parseObject(submitDto.getOrderInvoice());
        } catch (Exception e) {
            log.error("订单参数解析异常try-catch:", e);
            return ResponseJson.error("订单参数解析异常！", null);
        }
        if (null == orderInfo || orderInfo.isEmpty()){
            return ResponseJson.error("订单商品数据异常！", null);
        }
        if (null == payInfo || payInfo.isEmpty()){
            return ResponseJson.error("订单金额数据异常！", null);
        }
        if (null == orderInvoice || orderInvoice.isEmpty()){
            return ResponseJson.error("发票信息数据异常！", null);
        }
        // 打印参数
        if (3 == submitDto.getCartType()){
            log.info("******************** 【协销订单】提交订单参数："+ submitDto);
        } else if (2 == submitDto.getCartType()) {
            log.info("******************** 【自主订单】立即购买提交订单参数："+ submitDto);
        } else {
            log.info("******************** 【自主订单】购物车提交订单参数："+ submitDto);
        }
        // 机构用户
        OrderParamBo orderParamBo = submitMapper.getOrderUserBoById(clubUserId);
        orderParamBo.setOrderSource(submitDto.getOrderSource());
        orderParamBo.setOrderInfo(orderInfo);
        // 余额支付标识，0不使用，1使用
        Integer balancePayFlag = (Integer) payInfo.get("balancePayFlag");
        // 判断用户可用余额是否wei0
        if (1 == balancePayFlag && null != orderParamBo.getAbleUserMoney() && MathUtil.compare(orderParamBo.getAbleUserMoney(), BigDecimal.ZERO) == 0) {
            return ResponseJson.error("用户可用余额为0.00元！", null);
        }
        orderParamBo.setBalancePayFlag(balancePayFlag);
        String orderShouldPayFee = (String) payInfo.get("orderShouldPayFee");
        orderParamBo.setOrderShouldPayFee(Double.parseDouble(orderShouldPayFee));
        // 运费标志：0包邮 -1到付 1遵循运费规则                               
        Integer postageFlag = (Integer) payInfo.get("postageFlag");
        // 运费
        String postage = (String) payInfo.get("postage");
        if (null == postageFlag || null == postage) {
            return ResponseJson.error("运费数据异常！", null);
        }
        orderParamBo.setPostageFlag(postageFlag);
        orderParamBo.setPostage(Double.parseDouble(postage));
        // 采美豆抵扣运费，1：100，到付默认30元，抵扣3000豆
        Integer userBeans = (Integer) payInfo.get("userBeans");
        if (null != userBeans && userBeans > 0) {
            // 计算需抵扣采美豆数量
            int offsetBeans = MathUtil.mul(postage, 100).intValue();
            if (-1 == postageFlag) {
                // 到付默认30元，抵扣3000豆
                offsetBeans = 3000;
            }
            // 与前端传入采美豆数量比较
            if (null == orderParamBo.getUserBeans() || !userBeans.equals(offsetBeans)){
                return ResponseJson.error("采美豆数据异常！", null);
            }
            if (orderParamBo.getUserBeans()<=0) {
                return ResponseJson.error("用户没有剩余采美豆！", null);
            }
        } else {
            userBeans = 0;
        }
        orderParamBo.setOffsetBeans(userBeans);
        // 发票类型 0不开发票 1普通发票 2增值税发票
        Integer invoiceType = (Integer) orderInvoice.get("type");
        if (null == invoiceType) {
            return ResponseJson.error("发票类型不能为空！", null);
        }
        orderParamBo.setInvoiceType(invoiceType);
        if (1== invoiceType || 2 == invoiceType) {
            InvoicePo invoice = new InvoicePo();
            String invoiceTitle = (String) orderInvoice.get("invoiceTitle");
            if (StringUtils.isEmpty(invoiceTitle)){
                return ResponseJson.error("发票抬头信息不正确！", null);
            }
            String corporationTaxNum = (String) orderInvoice.get("corporationTaxNum");
            if (StringUtils.isEmpty(corporationTaxNum)){
                return ResponseJson.error("纳税人识别号信息不正确！", null);
            }
            invoice.setType(invoiceType);
            invoice.setInvoiceTitle(invoiceTitle);
            invoice.setCorporationTaxNum(corporationTaxNum);
            if (1== invoiceType) {
                // 普通发票：发票类型、发票内容（商品明细）、抬头(公司名称)、纳税人识别号[普通发票的公司]
                String invoiceContent = (String) orderInvoice.get("invoiceContent");
                Integer invoiceTitleType = (Integer) orderInvoice.get("invoiceTitleType");
                if (StringUtils.isEmpty(invoiceContent) || null == invoiceTitleType) {
                    return ResponseJson.error("发票信息不完整！", null);
                }
                invoice.setInvoiceContent(invoiceContent);
                invoice.setInvoiceTitleType(invoiceTitleType);
            }
            if (2== invoiceType) {
                // 增值税发票：发票类型、发票、抬头(公司名称)、纳税人识别号、注册地址、注册电话、开户银行、开户银行账户
                String registeredAddress = (String) orderInvoice.get("registeredAddress");
                String registeredPhone = (String) orderInvoice.get("registeredPhone");
                String openBank = (String) orderInvoice.get("openBank");
                String bankAccountNo = (String) orderInvoice.get("bankAccountNo");
                if (StringUtils.isEmpty(registeredAddress) || StringUtils.isEmpty(registeredPhone) || StringUtils.isEmpty(openBank) || StringUtils.isEmpty(bankAccountNo)) {
                    return ResponseJson.error("发票信息不完整！", null);
                }
                invoice.setRegisteredAddress(registeredAddress);
                invoice.setRegisteredPhone(registeredPhone);
                invoice.setOpenBank(openBank);
                invoice.setBankAccountNo(bankAccountNo);
            }
            orderParamBo.setOrderInvoice(invoice);
        }
        // 返佣订单标识 0非返佣订单，1返佣订单
        Integer rebateFlag = (Integer) payInfo.get("rebateFlag");
        orderParamBo.setRebateFlag(rebateFlag);
        // 售后条款id
        Integer clauseId = (Integer) payInfo.get("clauseId");
        orderParamBo.setClauseId(clauseId);
        // 购买类型:（1自主下单,2直接购买 3协销下单）
        orderParamBo.setCartType(submitDto.getCartType());
        // 下单人
        Integer buyUserId = null;
        if (1 == submitDto.getCartType() || 2 == submitDto.getCartType()){
            // 自主下单
            buyUserId = clubUserId;
            if (StringUtils.isNotEmpty(submitDto.getUnionId())) {
                // 运营人员Id
                Integer operationId = submitMapper.getOperationIdByUnionId(submitDto.getUnionId(), clubUserId);
                if (null != operationId && operationId>0) {
                    buyUserId = operationId;
                }
            }
        } else if (3 == submitDto.getCartType()){
            // 协销下单, 获取协销用户Id
            Integer spUserId = submitMapper.getServiceProviderUserId(submitDto.getServiceProviderId());
            if (null != spUserId && spUserId>0) {
                buyUserId = spUserId;
            } else {
                return ResponseJson.error("协销用户异常！", null);
            }
        } else {
            return ResponseJson.error("购买类型不正确！", null);
        }
        orderParamBo.setBuyUserId(buyUserId);
        // 超级会员用户标识
        Integer svipUserId = baseMapper.getSvipUserIdByUserId(clubUserId);
        boolean svipUserFlag = null != svipUserId;
        orderParamBo.setSvipUserFlag(svipUserFlag);
        orderParamBo.setServiceProviderId(submitDto.getServiceProviderId());
        orderParamBo.setAddressId(submitDto.getAddressId());
        orderParamBo.setClubCouponId(submitDto.getClubCouponId());
        /*
         * 保存订单
         */
        return saveOrder(orderParamBo);
    }

    public ResponseJson<Map<String, Object>> saveOrder(OrderParamBo orderParamBo) {
        log.info("******************** 提交订单逻辑处理 start *******************");
        /*
         * 初始化主订单
         */
        OrderPo mainOrder = new OrderPo();
        // 订单来源
        mainOrder.setOrderSource(orderParamBo.getOrderSource());
        // 订单号
        String orderNo = CodeUtil.generateOrderNo(orderParamBo.getOrderSource());
        mainOrder.setOrderNo(orderNo);
        // 用户Id
        mainOrder.setUserId(orderParamBo.getUserId());
        mainOrder.setClubId(orderParamBo.getClubId());
        mainOrder.setBuyUserId(orderParamBo.getBuyUserId());
        // 订单提交时间
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        mainOrder.setOrderTime(curDateStr);
        mainOrder.setUpdateDate(curDateStr);
        // 订单状态 0 有效  其它无效
        mainOrder.setDelFlag(0);
        if (3 == orderParamBo.getCartType()) {
            mainOrder.setSpId(orderParamBo.getServiceProviderId());
            // 协销订单
            mainOrder.setOrderType(0);
            // 3:协销帮会所下单
            mainOrder.setOrderSubmitType(3);
            // 订单未确认
            mainOrder.setConfirmFlag(0);
        } else {
            // 普通订单
            mainOrder.setOrderType(1);
            // 其他默认 2:员工帮会所下单
            mainOrder.setOrderSubmitType(2);
            // 2买家确认
            mainOrder.setConfirmFlag(2);
        }
        // 抵扣采美豆数量
        mainOrder.setUserBeans(orderParamBo.getOffsetBeans());
        // 默认可以线上支付：0可以 1不可以
        mainOrder.setOnlinePayFlag(0);
        // 默认订单可以拆分：1可拆分 0不可拆分
        mainOrder.setSplitFlag(1);
        // 默认订单未支付：未支付0 已支付1
        mainOrder.setPayFlag(0);
        // 默认待向买家收款：1待收款、2部分收款、3已收款
        mainOrder.setReceiptStatus(1);
        // 默认未付款供应商：1待付款、2部分付款、3已付款
        mainOrder.setPayStatus(1);
        // 默认有成本
        mainOrder.setZeroCostFlag(0);
        // 发货状态：1待发货、2部分发货、3已发货
        mainOrder.setSendOutStatus(1);
        // 退货退款类型：0未发生退款、1部分退、2全部退
        mainOrder.setRefundType(0);
        // 未确认付款供应商
        mainOrder.setAffirmPaymentFlag(0);
        // 是否开发票
        mainOrder.setInvoiceFlag(orderParamBo.getInvoiceType());

        /*
         * 整理订单商品信息
         */
        // 商品总数量
        AtomicInteger productCount = new AtomicInteger(0);
        // 赠品数量
        AtomicInteger presentCount = new AtomicInteger(0);
        // 促销赠品数量
        AtomicInteger promotionalGiftsCount = new AtomicInteger(0);
        // 促销满减优惠
        AtomicDouble promotionFullReduction = new AtomicDouble(0);
        // 超级会员优惠
        AtomicDouble svipFullReduction = new AtomicDouble(0);
        // 商品总金额 （商品单价乘以数量，再加上税费）
        AtomicDouble productTotalFee = new AtomicDouble(0);
        // 订单总额（小计金额减去经理折扣后，再加上运费[默认0])
        AtomicDouble payTotalFee = new AtomicDouble(0);
        // 真实支付金额（订单总额减去抵扣的账户余额）
        AtomicDouble payableAmount = new AtomicDouble(0);
        // 余额支付金额
        AtomicDouble balancePayFee = new AtomicDouble(0);

        // 二手订单标记（二手订单不能同正常商品下单，只能单个商品立即购买下单）
        boolean secondHandOrderFlag = false;
        // 是否包含活动商品
        boolean hasActProductFlag = false;
        //促销活动ids
        List<Integer> promotionsIds = new ArrayList<>();
        // 促销活动信息
        List<PromotionsVo> promotionList= new ArrayList<>();
        // 订单商品列表
        List<OrderProductPo> orderProductList = new ArrayList<>();
        List<String> productIdList = new ArrayList<>();

        // 子订单订单列表
        List<OrderShopPo> shopOrderList = new ArrayList<>();
        JSONArray orderInfo = orderParamBo.getOrderInfo();
        for (Object infoObject: orderInfo) {
            JSONObject shopInfo = (JSONObject) infoObject;
            Integer shopId = (Integer) shopInfo.get("shopId");
            String shopNote = (String) shopInfo.get("note");
            if (null == shopId) {
                return ResponseJson.error("供应商数据异常！", null);
            }
            JSONArray productArr = (JSONArray) shopInfo.get("productInfo");
            if (null == productArr) {
                return ResponseJson.error("订单商品数据异常！", null);
            }
            // 供应商名称
            String shopName = baseMapper.getShopNameById(shopId);
            /*
             * 初始化子订单
             */
            OrderShopPo shopOrder = new OrderShopPo();
            shopOrder.setShopId(shopId);
            shopOrder.setShopName(shopName);
            shopOrder.setNote(shopNote);
            // 用户Id
            shopOrder.setUserId(orderParamBo.getUserId());
            shopOrder.setClubId(orderParamBo.getClubId());
            if (3 == orderParamBo.getCartType()) {
                // 协销订单
                shopOrder.setSpId(orderParamBo.getBuyUserId());
            }
            // 商品总数量
            AtomicInteger shopProductCount = new AtomicInteger(0);
            // 佣金 =  应付采美
            AtomicDouble brokerage = new AtomicDouble(0);
            // 需要支付金额
            AtomicDouble needPayAmount = new AtomicDouble(0);
            // 商品总金额
            AtomicDouble shopAmount = new AtomicDouble(0);
            // 付供应商 商品费
            AtomicDouble shopProductAmount = new AtomicDouble(0);
            // 付供应商税费
            AtomicDouble shopTaxFee = new AtomicDouble(0);

            // 店铺促销活动状态
            PromotionsVo shopPromotions = baseMapper.getPromotionByShopId(shopId);
            // 商品费
            AtomicDouble shopProductFee = new AtomicDouble(0);
            // 供应商下参与店铺促销的商品总价
            AtomicDouble shopPromotionFee = new AtomicDouble(0);
            // 供应商超级会员优惠
            AtomicDouble svipShopReduction = new AtomicDouble(0);
            // 遍历所有商品
            for (Object productObject : productArr) {
                JSONObject productTemp = (JSONObject) productObject;
                Integer productId = (Integer) productTemp.get("productId");
                Integer productNum = (Integer) productTemp.get("productNum");
                Integer presentNum = (Integer) productTemp.get("presentNum");
                Integer productType = (Integer) productTemp.get("productType");
                productType = (null == productType) ? 0 : productType;
                if (null == productId) {
                    return ResponseJson.error("订单商品数据异常！", null);
                }
                if (null == productNum || productNum == 0) {
                    return ResponseJson.error("商品购买数量异常！", null);
                }
                // 统计商品总数量
                productCount.updateAndGet(v -> v + productNum);
                // 赠品数
                presentCount.updateAndGet(v -> v + presentNum);
                // 获取数据库商品信息
                OrderProductPo product = submitMapper.getProductDetails(productId);
                if (null == product) {
                    return ResponseJson.error("订单商品不存在！", null);
                }
                // 是否二手商品
                if (null != product.getProductCategory() && 2 == product.getProductCategory()) {
                    secondHandOrderFlag = true;
                }
                // 成本价
                Double costPrice = null;
                // 判断是否选中固定成本价
                if (null != product.getCostPrice() && 1 == product.getCostCheckFlag() && product.getCostPrice() >= 0d) {
                    costPrice = product.getCostPrice();
                }
                // 判断是否选中比例成本价
                if (null != product.getCostProportional() && 2 == product.getCostCheckFlag() && product.getCostProportional() > 0d) {
                    // 通过售价*比例得到成本价
                    costPrice = MathUtil.div(MathUtil.mul(product.getPrice(), product.getCostProportional()), 100).doubleValue();
                }
                if (null == costPrice) {
                    return ResponseJson.error("订单商品成本异常！", null);
                }
                // 是否添加税费，不含税商品 开票需添加税费
                boolean taxFlag = (Integer.valueOf(0).equals(product.getIncludedTax()) && (Integer.valueOf(1).equals(product.getInvoiceType()) || Integer.valueOf(2).equals(product.getInvoiceType())));
                Double shopTax = 0d;
                // 图片路径
                String image = ImageUtil.getImageUrl("product", product.getImage(), domain);
                product.setImage(image);
                product.setShopName(shopName);
                product.setCostPrice(costPrice);
                product.setNum(productNum);
                product.setPresentNum(presentNum);
                product.setProductType(productType);
                // 超级会员优惠商品详情
                SvipProductPo svipProductPo = submitMapper.getSvipProductDetails(productId);
                // 是否是促销赠品
                if (productType == 2) {
                    // 促销赠品数+1
                    promotionalGiftsCount.incrementAndGet();
                    product.setPrice(0d);
                    product.setDiscountPrice(0d);
                    product.setDiscount(0d);
                    product.setTotalAmount(0d);
                    product.setTotalFee(0d);
                    product.setTaxRate(0d);
                    product.setAddedValueTax(0d);
                    product.setTotalAddedValueTax(0d);
                }
                else {
                    // 获取商品购买价格(超级会员优惠商品价格>>>活动价格>>>阶梯价格>>>复购价格库>>>商品原始价)
                    Double productPrice = product.getPrice();
                    Double discountPrice = product.getPrice();
                    // 商品税费
                    Double productTax = 0d;
                    Double discountTax = 0d;
                    // 商品是否处于活动状态
                    PromotionsVo promotions = null;
                    // 店铺促销优先
                    if (null == shopPromotions) {
                        promotions = baseMapper.getPromotionByProductId(productId);
                    }
                    // 计算单价
                    if (null != svipProductPo && 1 == svipProductPo.getSvipProductFlag()) {
                        // 超级会员优惠商品,不参与促销活动（普通机构购买该商品可参与店铺促销）/阶梯价/复购价
                        if (orderParamBo.getSvipUserFlag()) {
                            if (1 == svipProductPo.getSvipPriceType()) {
                                // 折扣价
                                discountPrice = MathUtil.div(MathUtil.mul(productPrice, svipProductPo.getSvipDiscount()), 100, 2).doubleValue();
                            } else if (2 == svipProductPo.getSvipPriceType()) {
                                // 直接优惠价
                                discountPrice = svipProductPo.getSvipDiscountPrice().doubleValue();
                            }
                            BigDecimal reductionFee = MathUtil.mul(MathUtil.sub(productPrice, discountPrice), productNum, 2);
                            // 统计超级会员优惠
                            svipShopReduction.set(MathUtil.add(svipShopReduction, reductionFee).doubleValue());
                            svipFullReduction.set(MathUtil.add(svipFullReduction, reductionFee).doubleValue());
                            // 保存订单商品svip数据
                            product.setSvipProductFlag(1);
                            product.setSvipPriceType(svipProductPo.getSvipPriceType());
                            product.setSvipDiscount(1 == svipProductPo.getSvipPriceType() ? svipProductPo.getSvipDiscount() : 0.00d);
                        }
                    } else if (null != promotions || null != shopPromotions) {
                        // 是否包含活动商品(受订单未支付自动关闭时间影响)  0 否 1 是
                        hasActProductFlag = true;
                        // 关闭阶梯价格，活动优先
                        product.setLadderPriceFlag(0);
                        product.setActProduct(1);
                        if (null != promotions && promotions.getType() == 1 && promotions.getMode() == 1) {
                            discountPrice = promotions.getTouchPrice();
                        }
                    } else if (1 == product.getLadderPriceFlag()) {
                        // 启用了阶梯价格
                        List<LadderPriceVo> ladderPrices = baseMapper.getLadderPriceList(productId);
                        // 判断阶梯价格的购买数量校验
                        int minBuyNumber = null != ladderPrices.get(0) ? ladderPrices.get(0).getBuyNum() : 0;
                        if (productNum < minBuyNumber) {
                            return ResponseJson.error("商品购买量低于最小起订量！", null);
                        }
                        //根据商品购买数量获取商品对应阶梯价格
                        for (LadderPriceVo ladderPrice : ladderPrices) {
                            if (productNum >= ladderPrice.getBuyNum()) {
                                discountPrice = ladderPrice.getBuyPrice();
                            }
                        }
                        product.setLadderPrices(ladderPrices);
                        product.setActProduct(2);
                    } else {
                        // 复购价
                        Double repurchase = baseMapper.getRepurchasePrice(productId, orderParamBo.getUserId());
                        if (null != repurchase && repurchase > 0) {
                            discountPrice = repurchase;
                        }
                        product.setActProduct(0);
                    }
                    if (MathUtil.compare(discountPrice, BigDecimal.ZERO) == 0) {
                        return ResponseJson.error("商品购买价格不能为0！", null);
                    }
                    // 不含税可开票商品计算税费
                    if (null == product.getTaxRate() || product.getTaxRate() <= 0) {
                        product.setTaxRate(0d);
                    }
                    if (taxFlag) {
                        productTax = MathUtil.div(MathUtil.mul(productPrice, product.getTaxRate()), 100, 2).doubleValue();
                        discountTax = MathUtil.div(MathUtil.mul(discountPrice, product.getTaxRate()), 100, 2).doubleValue();
                    } else if (!Integer.valueOf(1).equals(product.getIncludedTax())) {
                        // 不含税不可开票商品和未知商品，税率置为0
                        product.setTaxRate(0d);
                    }
                    // 商品价格
                    product.setPrice(productPrice);
                    // 商品税后价格
                    Double productTaxPrice = MathUtil.add(productPrice, productTax).doubleValue();
                    // 折后单价
                    discountPrice = MathUtil.add(discountPrice, discountTax).doubleValue();
                    // 折扣率 = 折后单价/机构价
                    Double discountRate = MathUtil.mul(MathUtil.div(discountPrice, productTaxPrice), 100).doubleValue();
                    // 单个商品的金额
                    Double productAmount = MathUtil.mul(productTaxPrice, productNum).doubleValue();
                    // 单个商品的折后金额
                    Double productFee = MathUtil.mul(discountPrice, productNum).doubleValue();
                    // 单个商品总税费
                    Double taxFee = MathUtil.mul(discountTax, productNum).doubleValue();
                    if (hasActProductFlag && null != promotions) {
                        // 单品优惠
                        if (promotions.getType() == 1) {
                            // 是否满足单品优惠条件
                            if (promotions.getMode()==1 || MathUtil.compare(productFee, promotions.getTouchPrice()) >= 0) {
                                // 满减
                                if (promotions.getMode() == 2) {
                                    // 统计订单总满减金额
                                    promotionFullReduction.set(MathUtil.add(promotionFullReduction.get(), promotions.getReducedPrice()).doubleValue());
                                    // 统计单个商品的折后金额
                                    productFee = MathUtil.sub(productFee, promotions.getReducedPrice()).doubleValue();
                                }
                                // 添加到总促销
                                if (!promotionsIds.contains(promotions.getId())) {
                                    promotionsIds.add(promotions.getId());
                                    promotions.setPromotionsId(promotions.getId());
                                    promotionList.add(promotions);
                                }
                                product.setOrderPromotionsId(promotions.getId());
                            }
                        // 凑单优惠
                        } else if (promotions.getType() == 2) {
                            // 商品添加到总促销
                            PromotionPriceVo promotionPrice = new PromotionPriceVo();
                            promotionPrice.setProductId(productId);
                            promotionPrice.setNumber(productNum);
                            promotionPrice.setPrice(discountPrice);
                            if (null == promotions.getProductList()){
                                promotions.setProductList(new ArrayList<>());
                            }
                            promotions.getProductList().add(promotionPrice);
                            if (!promotionsIds.contains(promotions.getId())) {
                                promotionsIds.add(promotions.getId());
                                promotions.setPromotionsId(promotions.getId());
                                promotionList.add(promotions);
                            } else {
                                PromotionsVo finalPromotions = promotions;
                                promotionList.forEach(promotionsTemp -> {
                                    if (finalPromotions.getId().equals(promotionsTemp.getId())){
                                        promotionsTemp.getProductList().add(promotionPrice);
                                    }
                                });
                            }
                            product.setOrderPromotionsId(promotions.getId());
                        }
                    }
                    // 折后单价
                    product.setDiscountPrice(discountPrice);
                    // 折扣率 = 折后单价/机构价
                    product.setDiscount(discountRate);
                    // 单个商品的金额
                    product.setTotalAmount(productAmount);
                    // 单个商品的折后金额
                    product.setTotalFee(productFee);
                    product.setShouldPayFee(productFee);
                    // 税费
                    product.setAddedValueTax(discountTax);
                    product.setTotalAddedValueTax(taxFee);
                }
                // 付供应商税费
                if (null == product.getShopTaxRate() || product.getShopTaxRate() <= 0) {
                    product.setShopTaxRate(product.getTaxRate());
                }
                if (taxFlag) {
                    shopTax = MathUtil.div(MathUtil.mul(costPrice, product.getTaxRate()), 100, 2).doubleValue();
                } else if (!Integer.valueOf(1).equals(product.getIncludedTax())) {
                    // 不含税不可开票商品和未知商品，税率置为0
                    product.setShopTaxRate(0d);
                }
                // 单个付供应商税费
                product.setSingleShouldPayTotalTax(shopTax);
                // 单个商品付供应商总税费
                product.setShouldPayTotalTax(MathUtil.mul(shopTax, productNum).doubleValue());
                // 付供应商 商品费=成本价*(购买数量  + 赠品数量)
                Double costAmount = MathUtil.mul(costPrice, MathUtil.add(productNum, presentNum)).doubleValue();
                product.setShopProductAmount(costAmount);
                //应付供应商(单)=成本价+供应商税费(单)
                Double singleShopFee = MathUtil.add(product.getCostPrice(), shopTax).doubleValue();
                product.setSingleShopFee(singleShopFee);
                // 应付供应商(总)=应付供应商(单) * 商品数量
                Double shopFee = MathUtil.add(product.getCostPrice(), shopTax).doubleValue();
                product.setShopFee(shopFee);
                // 付第三方，默认0
                product.setOtherFee(0d);
                product.setSingleOtherFee(0d);
                //应付采美(单)=单价+机构税费(单)-(成本(单)+供应商税费(单))
                Double singleCmFee = MathUtil.sub(MathUtil.add(BigDecimal.valueOf(product.getPrice()), product.getAddedValueTax()), MathUtil.add(product.getCostPrice(), shopTax)).doubleValue();
                product.setSingleCmFee(singleCmFee);
                // 应付采美(总)=应付采美(单)*商品数量
                Double cmFee = MathUtil.mul(singleCmFee, productNum).doubleValue();
                product.setCmFee(cmFee);
                // 默认未支付
                product.setPayStatus(0);
                // 默认非再次购买商品
                product.setBuyAgainFlag(0);
                // 未出库数量
                product.setNotOutStore(productNum);
                /*
                 * 子订单金额计算
                 */
                // 供应商商品数量
                shopProductCount.set(shopProductCount.get()+product.getNum());
                // 佣金 = 应付采美
                brokerage.set(MathUtil.add(brokerage.get(), product.getCmFee()).doubleValue());
                // 需要支付金额 shouldPayFee +运费
                needPayAmount.set(MathUtil.add(needPayAmount.get(), product.getShouldPayFee()).doubleValue());
                // 统计 总金额 包括税费
                shopProductFee.set(MathUtil.add(shopProductFee.get(), product.getTotalFee()).doubleValue());
                // 统计子订单内可参与店铺促销的商品总金额
                boolean buySvipProductFlag = null != svipProductPo && 1 == svipProductPo.getSvipProductFlag() || orderParamBo.getSvipUserFlag();
                if (!buySvipProductFlag) {
                    // 超级会员购买优惠商品，不能参与店铺促销
                    shopPromotionFee.set(MathUtil.add(shopPromotionFee.get(), product.getTotalFee()).doubleValue());
                }
                // 商品总金额
                shopAmount.set(MathUtil.add(shopAmount.get(), product.getTotalAmount()).doubleValue());
                // 付供应商 商品费
                shopProductAmount.set(MathUtil.add(shopProductAmount.get(), product.getShopProductAmount()).doubleValue());
                // 付供应商税费
                shopTaxFee.set(MathUtil.add(shopTaxFee.get(), product.getShouldPayTotalTax()).doubleValue());

                // 加入订单商品列表
                orderProductList.add(product);
                productIdList.add(product.getProductId().toString());
            }

            if (null != shopPromotions) {
                // 是否满足满减满赠
                if (MathUtil.compare(shopPromotionFee.get(), shopPromotions.getTouchPrice()) >= 0) {
                    // 是否包含活动商品(受订单未支付自动关闭时间影响)  0 否 1 是
                    hasActProductFlag = true;
                    // 满减
                    if (shopPromotions.getMode() == 2) {
                        // 供应商满减金额
                        shopOrder.setPromotionFullReduction(shopPromotions.getReducedPrice());
                        // 统计订单总满减金额
                        promotionFullReduction.set(MathUtil.add(promotionFullReduction.get(), shopPromotions.getReducedPrice()).doubleValue());
                        // 统计店铺商品总金额
                        shopProductFee.set(MathUtil.sub(shopProductFee.get(), shopPromotions.getReducedPrice()).doubleValue());
                        // 统计需要支付金额
                        needPayAmount.set(MathUtil.sub(needPayAmount.get(), shopPromotions.getReducedPrice()).doubleValue());
                        // 统计需要支付金额
                        shopAmount.set(MathUtil.sub(shopAmount.get(), shopPromotions.getReducedPrice()).doubleValue());
                    }
                    // 添加到总促销
                    if (!promotionsIds.contains(shopPromotions.getId())) {
                        promotionsIds.add(shopPromotions.getId());
                        shopPromotions.setPromotionsId(shopPromotions.getId());
                        promotionList.add(shopPromotions);
                    }
                    shopOrder.setOrderPromotionsId(shopPromotions.getId());
                }
            }
            // 佣金 采美应收
            shopOrder.setBrokerage(brokerage.get());
            // 可退款金额 = 余额抵扣金额
            shopOrder.setCanRefundAmount(needPayAmount.get());
            // 购买商品数
            shopOrder.setItemCount(shopProductCount.get());
            // 超级会员优惠金额
            shopOrder.setSvipShopReduction(svipShopReduction.get());
            // 总价(含税) = totalFee
            shopOrder.setTotalAmount(shopProductFee.get());
            // 总金额 = 订单商品totalAmount
            shopOrder.setProductAmount(shopAmount.get());
            // 需要支付金额 shouldPayFee +运费
            shopOrder.setNeedPayAmount(needPayAmount.get());
            // 付供应商 商品费=成本价*(购买数量  + 赠品数量)
            shopOrder.setShopProductAmount(shopProductAmount.get());
            // 付给供应商税费
            shopOrder.setShopTaxFee(shopTaxFee.get());
            // 付供应商 = 商品费 + 运费 + 税费
            shopOrder.setShouldPayShopAmount(MathUtil.add(shopProductAmount.get(), shopTaxFee.get()).doubleValue());
            // 付给供应商运费
            shopOrder.setShopPostFee(0d);
            // 已付供应商金额
            shopOrder.setPayedShopAmount(0d);
            // (付款供应商)付款状态：1待付款、2部分付款、3已付款
            shopOrder.setPayStatus(1);

            // 添加到子订单列表
            shopOrderList.add(shopOrder);
            // 统计商品总金额
            productTotalFee.set(MathUtil.add(productTotalFee.get(), shopProductFee.get()).doubleValue());
        }
        if (orderProductList.isEmpty()){
            return ResponseJson.error("订单商品数据异常！", null);
        }
        // 设置是否是二手订单
        if (secondHandOrderFlag) {
            mainOrder.setSecondHandOrderFlag(1);
        } else {
            mainOrder.setSecondHandOrderFlag(0);
        }
        // 是否包含活动商品(受订单未支付自动关闭时间影响)  0 否 1 是
        if (!hasActProductFlag) {
            mainOrder.setHasActProduct(0);
        } else {
            mainOrder.setHasActProduct(1);
            // 促销活动优惠计算
            Iterator<PromotionsVo> promotionsIterator = promotionList.iterator();
            while (promotionsIterator.hasNext()) {
                PromotionsVo promotions = promotionsIterator.next();
                // 凑单优惠计算（单品与店铺已再前面分开计算了）
                if (promotions.getType() == 2) {
                    // 这里计算满减满赠
                    if (promotions.getMode() == 2 || promotions.getMode() == 3) {
                        // 实际该促销商品总额
                        AtomicDouble tempProductFee = new AtomicDouble(0);
                        promotions.getProductList().forEach(item -> tempProductFee.set(MathUtil.add(tempProductFee.get(), MathUtil.mul(item.getNumber(), item.getPrice())).doubleValue()));
                        //判断是否达到满减满赠要求
                        if (MathUtil.compare(tempProductFee.get(), promotions.getTouchPrice()) >= 0) {
                            if (promotions.getMode() == 2) {
                                // 满减
                                promotionFullReduction.set(MathUtil.add(promotionFullReduction.get(), promotions.getReducedPrice()).doubleValue());
                                // 统计商品总金额
                                productTotalFee.set(MathUtil.sub(productTotalFee.get(), promotions.getReducedPrice()).doubleValue());
                            }
                        } else {
                            // 删除不满足条件的凑单促销
                            promotionsIterator.remove();
                        }
                    }
                }
            }
            // 促销满减优惠
            mainOrder.setPromotionFullReduction(promotionFullReduction.get());
        }
        // 商品总数量
        mainOrder.setProductCount(productCount.get());
        // 赠品数量
        mainOrder.setPresentCount(presentCount.get());
        //促销赠品数量
        mainOrder.setPromotionalGiftsCount(promotionalGiftsCount.get());
        /*
         * 处理优惠券
         */
        Double couponAmount = 0d;
        CouponVo coupon = null;
        if (null != orderParamBo.getClubCouponId() && orderParamBo.getClubCouponId() > 0) {
            coupon = orderClubMapper.getClubCouponById(orderParamBo.getClubCouponId());
            if (null == coupon) {
                // 设置手动回滚事务
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ResponseJson.error("优惠券使用异常！", null);
            }
            couponAmount = coupon.getCouponAmount();
        }
        mainOrder.setCouponAmount(couponAmount);
        /*
         * 计算运费
         */
        if (3 != orderParamBo.getCartType()){
            // 机构用户 校验商品运费
            if (orderParamBo.getPostageFlag() != -1) {
                Integer townId = baseMapper.getTownIdByAddressId(orderParamBo.getAddressId());
                Map<String, Object> postageMap = productService.computePostage(orderParamBo.getUserId(), townId, productIdList);
                // 运费标志：0包邮 -1到付 1遵循运费规则
                Integer postageFlag = (Integer) postageMap.get("postageFlag");
                Double postage = (Double) postageMap.get("postage");
                if (!orderParamBo.getPostageFlag().equals(postageFlag) || MathUtil.compare(orderParamBo.getPostage(), postage) != 0) {
                    return ResponseJson.error("订单邮费不正确！", null);
                }
            }
        }
        // 设置运费
        mainOrder.setPostage(orderParamBo.getPostage());
        // 订单总额 = 商品费 + 运费
        payTotalFee.set(MathUtil.add(productTotalFee.get(), orderParamBo.getPostage()).doubleValue());
        // 运费标志：0包邮 -1到付 1遵循运费规则
        mainOrder.setPostageFlag(orderParamBo.getPostageFlag());
        if (1 == orderParamBo.getPostageFlag() && orderParamBo.getOffsetBeans() > 0) {
            // 订单总额 = 商品费 - 优惠券金额
            payTotalFee.set(MathUtil.sub(productTotalFee.get(), couponAmount).doubleValue());
        } else {
            // 订单总额 -= 优惠券金额
            payTotalFee.set(MathUtil.sub(payTotalFee.get(), couponAmount).doubleValue());
        }
        // 超级会员优惠
        mainOrder.setSvipFullReduction(svipFullReduction.get());
        // 商品总额
        mainOrder.setProductTotalFee(productTotalFee.get());
        mainOrder.setOrderTotalFee(productTotalFee.get());
        // 订单总额（商品金额+运费）
        mainOrder.setPayTotalFee(payTotalFee.get());
        // 订单状态
        if (3 == orderParamBo.getCartType()) {
            // 协销 状态为 待确认
            mainOrder.setStatus(0);
        } else {
            // 机构用户 状态为 待收待发
            mainOrder.setStatus(11);
            mainOrder.setConfirmTime(curDateStr);
        }

        // 是否完成支付(默认不是，只有余额抵扣才算)
        boolean isPaySuccessFlag = false;
        // 余额支付标识，0不使用，1使用
        if (1 == orderParamBo.getBalancePayFlag()) {
            // 抵扣后用户剩余可用余额
            Double lastAbleUserMoney;
            // 抵扣后账户余额
            Double lastUserMoney = orderParamBo.getUserMoney();
            // 部分抵扣
            if (MathUtil.compare(payTotalFee.get(), orderParamBo.getAbleUserMoney()) > 0) {
                balancePayFee.set(orderParamBo.getAbleUserMoney());
                payableAmount.set(MathUtil.sub(payTotalFee.get(), balancePayFee.get()).doubleValue());
                // 余额抵扣用完
                lastAbleUserMoney = 0d;
                if (3 != orderParamBo.getCartType()) {
                    // 部收款待发货
                    mainOrder.setStatus(21);
                    lastUserMoney = MathUtil.sub(orderParamBo.getUserMoney(), balancePayFee.get()).doubleValue();
                }
                mainOrder.setReceiptStatus(2);
            } else {
                // 全部用余额抵扣， 直接变成支付完成
                balancePayFee.set(payTotalFee.get());
                payableAmount.set(0d);
                lastAbleUserMoney = MathUtil.sub(orderParamBo.getAbleUserMoney(), balancePayFee.get()).doubleValue();
                if (3 != orderParamBo.getCartType()) {
                    // 已收款待发货
                    mainOrder.setStatus(31);
                    lastUserMoney = MathUtil.sub(orderParamBo.getUserMoney(), balancePayFee.get()).doubleValue();
                }
                mainOrder.setReceiptStatus(3);
                mainOrder.setPayFlag(1);
                isPaySuccessFlag = true;
            }
            // 抵扣后更新用户余额
            submitMapper.updateUserMoney(lastUserMoney, lastAbleUserMoney, orderParamBo.getUserId());
            log.info("【提交订单】>>>>>更新用户余额，抵扣后剩余：[userMoney:" + lastUserMoney + "] ,ableUserMoney:" + lastAbleUserMoney);
            // 支付时间
            mainOrder.setPayTime(curDateStr);
        } else {
            payableAmount.set(payTotalFee.get());
        }
        // 余额支付金额
        mainOrder.setBalancePayFee(balancePayFee.get());
        // 实际支付金额(商品金额+运费-余额抵扣)
        mainOrder.setPayableAmount(payableAmount.get());

        // 是否返佣订单
        Integer rebateFlag = (null == orderParamBo.getRebateFlag() ? 0 : orderParamBo.getRebateFlag());
        mainOrder.setRebateFlag(rebateFlag);

        // 判断前端传入orderShouldPayFee订单应付金额,和后台计算应付金额对比
        log.info("【提交订单】>>>>>后台计算payableAmount:" + payableAmount.get() + " ,前端传入orderShouldPayFee:" + orderParamBo.getOrderShouldPayFee());
        // compare return[-1:v1<v2, 0:v1=v2, 1:v1>v2]
        int compare = MathUtil.compare(orderParamBo.getOrderShouldPayFee(), payableAmount.get());
        if (compare != 0) {
            // 设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("订单付款金额不正确！", null);
        }

        // 售后条款
        Integer clauseId = orderParamBo.getClauseId();
        String clauseName = null;
        if (null != clauseId && clauseId > 0) {
            clauseName = submitMapper.getClauseNameById(clauseId);
        }
        if (StringUtils.isNotEmpty(clauseName)) {
            mainOrder.setClauseId(clauseId);
            mainOrder.setClauseName(clauseName);
        } else {
            mainOrder.setClauseId(1);
            mainOrder.setClauseName("无条款");
        }

        /*
         * 保存主订单数据，返回订单Id
         */
        submitMapper.insertMainOrder(mainOrder);
        log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增主订单(insert[cm_order])orderId：" + mainOrder.getOrderId());

        /*
         * 保存订单促销,返回订单促销Id
         */
        promotionList.forEach(promotions -> {
            promotions.setOrderId(mainOrder.getOrderId());
            submitMapper.insertOrderPromotions(promotions);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增订单促销活动(insert[cm_promotions_order])id：" + promotions.getId() + "，orderId：" + mainOrder.getOrderId());
        });

        /*
         * 整理 子订单信息
         */
        AtomicReference<String> shopOrderIds = new AtomicReference<>();
        AtomicInteger shopOrderIndex = new AtomicInteger(1);
        shopOrderList.forEach(shopOrder -> {
            shopOrder.setOrderId(mainOrder.getOrderId());
            shopOrder.setOrderNo(mainOrder.getOrderNo());
            shopOrder.setOrderTime(mainOrder.getOrderTime());
            shopOrder.setOrderSubmitType(mainOrder.getOrderSubmitType());
            // 生成子订单编号
            StringBuilder shopOrderNo = new StringBuilder();
            shopOrderNo.append(mainOrder.getOrderNo());
            if (shopOrderIndex.get() < 10) {
                shopOrderNo.append("0");
            }
            shopOrderNo.append(shopOrderIndex.get());
            shopOrderIndex.incrementAndGet();
            shopOrder.setShopOrderNo(shopOrderNo.toString());
            // 订单能否拆分 1 为可拆分, 0为不可拆分
            if (shopOrder.getItemCount() >1) {
                shopOrder.setSplitFlag(1);
            } else {
                shopOrder.setSplitFlag(0);
            }
            // 设置订单促销Id
            promotionList.forEach(promotions -> {
                if (promotions.getPromotionsId().equals(shopOrder.getOrderPromotionsId())) {
                    shopOrder.setOrderPromotionsId(promotions.getId());
                }
            });
            /*
             * 保存子订单
             */
            submitMapper.insertShopOrder(shopOrder);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增子订单(insert[cm_shop_order])shopOrderId：" + shopOrder.getShopOrderId() + "，orderId：" + mainOrder.getOrderId());
            // 收集子订单Id字符串
            if (StringUtils.isEmpty(shopOrderIds.get())) {
                shopOrderIds.set(shopOrder.getShopOrderId().toString());
            } else {
                shopOrderIds.set(shopOrderIds.get() + "," + shopOrder.getShopOrderId());
            }
        });

        /*
         * 整理  订单商品
         */
        for (OrderProductPo orderProduct : orderProductList) {
            // 设置商品订单Id
            orderProduct.setOrderId(mainOrder.getOrderId());
            orderProduct.setOrderNo(mainOrder.getOrderNo());
            // 设置商品子订单Id
            shopOrderList.forEach(shopOrder -> {
                if (orderProduct.getShopId().equals(shopOrder.getShopId())) {
                    orderProduct.setShopOrderId(shopOrder.getShopOrderId());
                    orderProduct.setShopOrderNo(shopOrder.getShopOrderNo());
                }
            });
            // 设置订单商品促销Id
            promotionList.forEach(promotions -> {
                if (promotions.getPromotionsId().equals(orderProduct.getOrderPromotionsId())) {
                    orderProduct.setOrderPromotionsId(promotions.getId());
                    orderProduct.setActProduct(1);
                }
            });
            /*
             * 保存 订单商品
             */
            submitMapper.insertOrderProduct(orderProduct);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单商品(insert[cm_order_product])OrderProductId：" + orderProduct.getOrderProductId() + "，orderId：" + mainOrder.getOrderId());
            /*
             * 保存 订单阶梯价格
             */
            if (orderProduct.getLadderPriceFlag() == 1) {
                // 使用阶梯价格的订单商品保存下单时的阶梯价格列表
                orderProduct.getLadderPrices().forEach(ladderPriceVo -> {
                    ladderPriceVo.setOrderProductId(orderProduct.getOrderProductId());
                    ladderPriceVo.setCreateDate(date);
                    submitMapper.insertLadderPrices(ladderPriceVo);
                    log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单阶梯价格(insert[order_product_ladder_price])id：" + ladderPriceVo.getId() + "，orderId：" + mainOrder.getOrderId());
                });
            }
            /*
             * 机构用户 写入价格库
             */
            if (3 != orderParamBo.getCartType()) {
                // 写入复购价格库
                productService.savePurchasePrice(mainOrder, orderProduct, secondHandOrderFlag, "【提交订单】");
            }
        }

        /*
         * 设置邮费子订单( 0包邮 -1到付 1遵循运费规则)
         */
        if (1 == mainOrder.getPostageFlag()) {
            // 获取 邮费商品
            OrderProductPo postageProduct = submitMapper.getProductDetails(999);
            // 生成子订单编号
            StringBuilder postageOrderNo = new StringBuilder();
            postageOrderNo.append(mainOrder.getOrderNo());
            if (shopOrderIndex.get() < 10) {
                postageOrderNo.append("0");
            }
            postageOrderNo.append(shopOrderIndex.get());
            shopOrderIndex.incrementAndGet();
            /*
             * 初始化邮费订单
             */
            OrderShopPo postageOrder = new OrderShopPo();
            postageOrder.setShopOrderNo(postageOrderNo.toString());
            postageOrder.setOrderNo(mainOrder.getOrderNo());
            postageOrder.setOrderId(mainOrder.getOrderId());
            postageOrder.setUserId(mainOrder.getUserId());
            postageOrder.setOrderSubmitType(mainOrder.getOrderSubmitType());
            postageOrder.setItemCount(1);
            //运费商品供应商Id默认998
            postageOrder.setShopId(postageProduct.getShopId());
            postageOrder.setProductAmount(mainOrder.getPostage());
            postageOrder.setTotalAmount(mainOrder.getPostage());
            postageOrder.setNeedPayAmount(mainOrder.getPostage());
            postageOrder.setOrderTime(mainOrder.getOrderTime());
            postageOrder.setCanRefundAmount(0d);
            postageOrder.setClubId(mainOrder.getClubId());
            // 已付供应商金额
            postageOrder.setPayedShopAmount(0d);
            // (付款供应商)付款状态：1待付款、2部分付款、3已付款
            postageOrder.setPayStatus(3);
            if (3 == orderParamBo.getCartType()) {
                // 协销订单
                postageOrder.setSpId(orderParamBo.getBuyUserId());
            }
            // 订单默认可拆分
            postageOrder.setSplitFlag(1);
            /*
             * 保存运费子订单
             */
            submitMapper.insertShopOrder(postageOrder);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增子订单(运费商品)(insert[cm_shop_order])shopOrderId：：" + postageOrder.getShopOrderId() + "，orderId：" + mainOrder.getOrderId());
            // 收集子订单Id字符串
            if (StringUtils.isEmpty(shopOrderIds.get())) {
                shopOrderIds.set(postageOrder.getShopOrderId().toString());
            } else {
                shopOrderIds.set(shopOrderIds.get() + "," + postageOrder.getShopOrderId());
            }
            /*
             * 设置运费商品
             */
            postageProduct.setOrderNo(mainOrder.getOrderNo());
            postageProduct.setOrderId(mainOrder.getOrderId());
            postageProduct.setShopOrderId(postageOrder.getShopOrderId());
            postageProduct.setShopOrderNo(postageOrder.getShopOrderNo());
            postageProduct.setNum(1);
            postageProduct.setPresentNum(0);
            postageProduct.setPrice(mainOrder.getPostage());
            postageProduct.setNormalPrice(mainOrder.getPostage());
            postageProduct.setTotalAmount(mainOrder.getPostage());
            postageProduct.setTotalFee(mainOrder.getPostage());
            postageProduct.setShouldPayFee(mainOrder.getPostage());
            postageProduct.setDiscount(100d);
            postageProduct.setDiscountPrice(mainOrder.getPostage());
            postageProduct.setTaxRate(100d);
            postageProduct.setAddedValueTax(0d);
            postageProduct.setTotalAddedValueTax(0d);
            postageProduct.setShopFee(0d);
            postageProduct.setOtherFee(0d);
            postageProduct.setCmFee(mainOrder.getPostage());
            postageProduct.setSingleShopFee(0d);
            postageProduct.setSingleOtherFee(0d);
            postageProduct.setSingleCmFee(mainOrder.getPostage());
            postageProduct.setNotOutStore(0);
            postageProduct.setBuyAgainFlag(0);
            /*
             * 保存 订单运费商品
             */
            submitMapper.insertOrderProduct(postageProduct);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单运费商品(insert[cm_order_product])OrderProductId：" + postageProduct.getOrderProductId() + "，orderId：" + mainOrder.getOrderId());
        }

        /*
         * 更新主订单的子订单Id信息,如:1000,1002
         */
        if (StringUtils.isNotEmpty(shopOrderIds.get())) {
            mainOrder.setShopOrderIds(shopOrderIds.get());
            submitMapper.updateShopOrderIds(shopOrderIds.get(), mainOrder.getOrderId());
        } else {
            // 设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("生成子订单异常！", null);
        }

        /*
         * 保存 订单收货用户信息
         */
        AddressVo address = submitMapper.getAddressDetailById(orderParamBo.getAddressId());
        if (null != address) {
            String userName = baseMapper.getUserNameByUserId(mainOrder.getUserId());
            //保存地址信息
            OrderUserInfoPo userInfo = new OrderUserInfoPo();
            userInfo.setOrderId(mainOrder.getOrderId());
            userInfo.setClubId(mainOrder.getClubId());
            userInfo.setUserId(mainOrder.getUserId());
            userInfo.setName(userName);
            userInfo.setReceiver(address.getReceiver());
            userInfo.setMobile(address.getMobile());
            userInfo.setTownId(address.getTownId());
            userInfo.setProvince(address.getProvince());
            userInfo.setCity(address.getCity());
            userInfo.setTown(address.getTown());
            userInfo.setAddress(address.getAddress());
            // 保存 订单收货用户信息
            submitMapper.insertOrderUserInfo(userInfo);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单用户地址(insert[bp_order_userinfo])orderId：" + mainOrder.getOrderId());
        } else {
            //设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("订单地址异常！", null);
        }

        /*
         *保存余额到余额收支记录
         */
        // 余额支付标识，0不使用，1使用
        if (1 == orderParamBo.getBalancePayFlag() && MathUtil.compare(balancePayFee.get(), 0) > 0) {
            // 余额支付标识，0不使用，1使用
            //保存余额到余额收支记录
            BalanceRecordPo balanceRecord = new BalanceRecordPo();
            balanceRecord.setUserId(mainOrder.getUserId());
            balanceRecord.setType(2);
            balanceRecord.setBalanceType(1);
            balanceRecord.setAddDate(new Date());
            balanceRecord.setAmount(balancePayFee.get());
            balanceRecord.setOrderId(mainOrder.getOrderId());
            balanceRecord.setRemark("机构自主订单");
            balanceRecord.setDelFlag(0);
            // 保存 余额收支记录
            baseMapper.insertBalanceRecord(balanceRecord);
            log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增用户余额收支记录(insert[cm_user_balance_record])orderId：" + mainOrder.getOrderId());

            // 保存余额抵扣到收款记录（自主订单）
            if (3 != orderParamBo.getCartType()) {
                Integer orderId = mainOrder.getOrderId();
                productService.saveDiscernReceipt(balancePayFee.get(), orderId, "【提交订单】");
            }
        }

        /*
         * 保存发票信息
         */
        if (1 == orderParamBo.getInvoiceType() || 2 == orderParamBo.getInvoiceType()) {
            /*
             * 更新订单增值税发票
             */
            InvoicePo orderInvoice = submitMapper.getOrderInvoice(mainOrder.getOrderId());
            if (null != orderInvoice && null != orderInvoice.getId()) {
                // 更新
                BeanUtils.copyProperties(orderParamBo.getOrderInvoice(), orderInvoice);
                orderInvoice.setOrderId(mainOrder.getOrderId());
                submitMapper.updateOrderInvoice(orderInvoice);
                log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>更新订单发票信息(update[bp_order_invoice])orderId：" + mainOrder.getOrderId());
            } else {
                // 新增
                orderInvoice = new InvoicePo();
                BeanUtils.copyProperties(orderParamBo.getOrderInvoice(), orderInvoice);
                orderInvoice.setOrderId(mainOrder.getOrderId());
                //  保存 订单发票信息
                submitMapper.insertOrderInvoice(orderInvoice);
                log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单发票信息(insert[bp_order_invoice])orderId：" + mainOrder.getOrderId());
            }
            /*
             * 更新用户增值税发票
             */
            InvoiceVo userInvoice = baseMapper.getUserInvoice(mainOrder.getUserId());
            if (null != userInvoice && null != userInvoice.getId()) {
                // 更新
                BeanUtils.copyProperties(orderInvoice, userInvoice);
                userInvoice.setUserId(mainOrder.getUserId());
                baseMapper.updateUserInvoiceByUserId(userInvoice);
                log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>更新用户增值税发票(update[cm_user_invoiceinfo])userId：" + mainOrder.getUserId());
            } else {
                // 新增
                userInvoice = new InvoiceVo();
                BeanUtils.copyProperties(orderInvoice, userInvoice);
                userInvoice.setUserId(mainOrder.getUserId());
                baseMapper.insertUserInvoiceByUserId(userInvoice);
                log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增用户增值税发票(insert[cm_user_invoiceinfo])userId：" + mainOrder.getUserId());
            }
        }

        /*
         * 保存采美豆使用记录
         */
        if (orderParamBo.getOffsetBeans() > 0) {
            UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
            beansHistory.setUserId(mainOrder.getUserId());
            beansHistory.setOrderId(mainOrder.getOrderId());
            beansHistory.setBeansType(10);
            beansHistory.setType(2);
            beansHistory.setNum(orderParamBo.getOffsetBeans());
            beansHistory.setPushStatus(0);
            beansHistory.setAddTime(date);
            beansHistory.setDelFlag(0);
            // 更新用户剩余采美豆数量
            int beans = orderParamBo.getUserBeans() - beansHistory.getNum();
            productService.updateUserBeans(beansHistory, beans,"【提交订单】");
        }

        /*
         * 保存优惠券使用情况
         */
        if (null != orderParamBo.getClubCouponId() && orderParamBo.getClubCouponId() > 0) {
            // 修改优惠券使用情况
            if (3 != orderParamBo.getCartType()) {
                // 自主订单立即确认，协销订单待确认 ，确认订单时才真正使用优惠券
                orderClubMapper.updateCouponClubStatus(orderParamBo.getClubCouponId(), mainOrder.getOrderId());
            }
            CouponOrderRecordPo orderRecord = new CouponOrderRecordPo();
            BeanUtils.copyProperties(coupon, orderRecord);
            orderRecord.setOrderId(mainOrder.getOrderId());
            orderRecord.setCreateDate(date);
            // 保存订单优惠记录
            submitMapper.insertCouponOrderRecord(orderRecord);
        }

        /*
         * 删除购物车
         */
        if (3 == orderParamBo.getCartType()) {
            // 清理当前机构的协销购物车数据
            cartSellerMapper.deleteSellerCartByProductIds(orderParamBo.getServiceProviderId(), orderParamBo.getClubId(), productIdList);
            log.info("【提交订单】>>>>删除当前机构的 协销 购物车数据！");
        } else if (1 == orderParamBo.getCartType()) {
            // 清理用户购物车
            cartClubMapper.deleteCartByProductIds(orderParamBo.getUserId(), productIdList);
            log.info("【提交订单】>>>>删除当前机构 用户 购物车数据！");
        }

        log.info("******************** 提交订单逻辑处理 end *******************");

        /*
         * 下单推送
         */
        if (StringUtils.isNotBlank(orderParamBo.getBindMobile())) {
            String shortLink = remoteCallService.getShortLink(8, 3, domain + "/user/order/detail.html?orderId=" + mainOrder.getOrderId());
            String name = orderProductList.get(0).getName();
            if (name.length() > 10) {
                name = name.substring(0, 10);
            }
            String content = "您已成功下单“" + name + "...”等" + mainOrder.getProductCount() + "件商品，订单编号：" + mainOrder.getOrderNo() + "，订单等待支付，支付完成后采美将尽快安排发货。" +
                    "您可关注采美公众号或者访问采美微信小程序和网站查看并支付订单。平台公众号：微信搜索“采美365网”; 微信小程序：微信搜索“采美采购商城”；网址：www.caimei365.com/t/" + shortLink;
            boolean sendSms = remoteCallService.getSendSms(3, orderParamBo.getBindMobile(), content);
            if (!sendSms) {
                log.info("下单推送失败，orderId>>>>" + mainOrder.getOrderId());
            }
        }

        /*
         * 构造返回参数
         */
        Map<String, Object> info = new HashMap<>();
        info.put("orderId", mainOrder.getOrderId());
        info.put("orderNo", mainOrder.getOrderNo());
        info.put("orderMark", "#" + mainOrder.getOrderId() + "#");
        //应付订单金额
        info.put("payTotalFee", mainOrder.getPayTotalFee());
        //真实需要付款金额
        info.put("payableAmount", mainOrder.getPayableAmount());
        if (isPaySuccessFlag) {
            // 1提交成功（支付完成）
            info.put("code", 1);
            info.put("msg", "提交成功且已支付完成!");
        } else {
            //2提交成功(未支付)
            info.put("code", 2);
            info.put("msg", "提交成功但未支付!");
        }
        return ResponseJson.success("操作成功!", info);
    }

    /**
     * 生成充值商品订单
     *
     * @param rechargeDto {
     *                    serviceProviderId 协销id
     *                    clubId            机构Id
     *                    name              商品名称
     *                    price             价格
     *                    note              备注
     *                    }
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<Map<String, Object>> generateRechargeOrder(RechargeDto rechargeDto) {
        // 获取机构用户Id
        Integer clubUserId = baseMapper.getUserIdByClubId(rechargeDto.getClubId());
        if (null == clubUserId || clubUserId == 0){
            return ResponseJson.error("机构用户信息异常！", null);
        }
        // 获取协销用户Id
        Integer spUserId = submitMapper.getServiceProviderUserId(rechargeDto.getServiceProviderId());
        if (null == spUserId || spUserId == 0){
            return ResponseJson.error("协销用户异常！", null);
        }
        log.info("******************** 提交充值订单逻辑处理 start *******************");
        /*
         * 初始化主订单
         */
        OrderPo mainOrder = new OrderPo();
        // 订单来源
        mainOrder.setOrderSource(6);
        // 订单号
        String orderNo = CodeUtil.generateOrderNo(6);
        mainOrder.setOrderNo(orderNo);
        // 用户Id
        mainOrder.setClubId(rechargeDto.getClubId());
        mainOrder.setUserId(clubUserId);
        mainOrder.setBuyUserId(spUserId);
        // 订单提交时间
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        mainOrder.setOrderTime(curDateStr);
        mainOrder.setUpdateDate(curDateStr);
        // 订单状态 0 有效  其它无效
        mainOrder.setDelFlag(0);
        mainOrder.setOrderType(0);
        mainOrder.setOrderSubmitType(3);
        mainOrder.setConfirmFlag(0);
        mainOrder.setPayFlag(0);
        mainOrder.setOnlinePayFlag(0);
        // 订单提交时间
        mainOrder.setOrderTime(curDateStr);
        // 订单不可以拆分
        mainOrder.setSplitFlag(0);
        // 发票类型
        mainOrder.setInvoiceFlag(0);
        mainOrder.setReceiptStatus(1);
        mainOrder.setPayStatus(1);
        mainOrder.setZeroCostFlag(0);
        mainOrder.setSendOutStatus(1);
        mainOrder.setRefundType(0);
        // 是否包含活动商品(受订单未支付自动关闭时间影响)  0 否 1 是
        mainOrder.setHasActProduct(0);
        // 是否确认付款供应商
        mainOrder.setAffirmPaymentFlag(0);
        // 设置是否是二手订单
        mainOrder.setSecondHandOrderFlag(0);
        mainOrder.setPromotionFullReduction(0d);
        // 商品总数量
        mainOrder.setProductCount(1);
        // 赠品数量
        mainOrder.setPresentCount(0);
        //促销赠品数量
        mainOrder.setPromotionalGiftsCount(0);
        // 0包邮 -1到付 1 有运费
        mainOrder.setPostageFlag(0);
        mainOrder.setPostage(0d);
        // 商品总额
        mainOrder.setProductTotalFee(rechargeDto.getPrice());
        mainOrder.setOrderTotalFee(rechargeDto.getPrice());
        mainOrder.setPayTotalFee(rechargeDto.getPrice());
        // 订单状态
        mainOrder.setStatus(0);
        // 余额支付金额
        mainOrder.setBalancePayFee(0d);
        // 实际支付金额(商品金额+运费-余额抵扣)
        mainOrder.setPayableAmount(rechargeDto.getPrice());
        // 售后条款
        mainOrder.setClauseId(1);
        mainOrder.setClauseName("无条款");
        // 是否返佣订单
        mainOrder.setRebateFlag(0);
        /*
         * 保存主订单数据，返回订单Id
         */
        submitMapper.insertMainOrder(mainOrder);
        log.info("【提交充值订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增主订单(insert[cm_order])orderId：" + mainOrder.getOrderId());
        // 获取数据库商品信息
        OrderProductPo rechargeProduct = submitMapper.getProductDetails(6060);
        // 供应商名称
        String shopName = baseMapper.getShopNameById(rechargeProduct.getShopId());
        /*
         * 初始化子订单
         */
        OrderShopPo rechargeShopOrder = new OrderShopPo();
        // 子订单编号
        rechargeShopOrder.setShopOrderNo(mainOrder.getOrderNo() + "01");
        rechargeShopOrder.setOrderNo(mainOrder.getOrderNo());
        rechargeShopOrder.setOrderId(mainOrder.getOrderId());
        rechargeShopOrder.setShopId(rechargeProduct.getShopId());
        rechargeShopOrder.setShopName(shopName);
        // 用户Id
        rechargeShopOrder.setClubId(mainOrder.getClubId());
        rechargeShopOrder.setUserId(mainOrder.getUserId());
        // 协销订单
        rechargeShopOrder.setSpId(rechargeDto.getServiceProviderId());
        rechargeShopOrder.setOrderSubmitType(mainOrder.getOrderSubmitType());
        rechargeShopOrder.setItemCount(1);
        rechargeShopOrder.setProductAmount(rechargeDto.getPrice());
        rechargeShopOrder.setTotalAmount(rechargeDto.getPrice());
        rechargeShopOrder.setNeedPayAmount(rechargeDto.getPrice());
        rechargeShopOrder.setOrderTime(mainOrder.getOrderTime());
        rechargeShopOrder.setCanRefundAmount(0d);
        rechargeShopOrder.setSplitFlag(0);
        /*
         * 保存子订单
         */
        submitMapper.insertShopOrder(rechargeShopOrder);
        log.info("【提交充值订单】>>>>>>>>>>>>>>>>>>>>>>>>>>新增子订单(insert[cm_shop_order])shopOrderId：" + rechargeShopOrder.getShopOrderId() + "，orderId：" + mainOrder.getOrderId());
        /*
         * 更新主订单的子订单Id信息,如:1000,1002
         */
        if (null != rechargeShopOrder.getShopOrderId()) {
            mainOrder.setShopOrderIds(rechargeShopOrder.getShopOrderId().toString());
            submitMapper.updateShopOrderIds(rechargeShopOrder.getShopOrderId().toString(), mainOrder.getOrderId());
        } else {
            // 设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("生成子订单异常！", null);
        }
        /*
         * 订单商品
         */
        rechargeProduct.setOrderNo(mainOrder.getOrderNo());
        rechargeProduct.setOrderId(mainOrder.getOrderId());
        rechargeProduct.setShopOrderId(rechargeShopOrder.getShopOrderId());
        rechargeProduct.setShopOrderNo(rechargeShopOrder.getShopOrderNo());
        rechargeProduct.setNum(1);
        rechargeProduct.setPresentNum(0);
        rechargeProduct.setPrice(rechargeDto.getPrice());
        rechargeProduct.setNormalPrice(rechargeDto.getPrice());
        rechargeProduct.setTotalAmount(rechargeDto.getPrice());
        rechargeProduct.setTotalFee(rechargeDto.getPrice());
        rechargeProduct.setShouldPayFee(rechargeDto.getPrice());
        rechargeProduct.setDiscount(100d);
        rechargeProduct.setDiscountPrice(rechargeDto.getPrice());
        rechargeProduct.setTaxRate(100d);
        rechargeProduct.setAddedValueTax(0d);
        rechargeProduct.setTotalAddedValueTax(0d);
        rechargeProduct.setShopFee(0d);
        rechargeProduct.setOtherFee(0d);
        rechargeProduct.setCmFee(rechargeDto.getPrice());
        rechargeProduct.setSingleShopFee(0d);
        rechargeProduct.setSingleOtherFee(0d);
        rechargeProduct.setSingleCmFee(rechargeDto.getPrice());
        rechargeProduct.setNotOutStore(0);
        rechargeProduct.setBuyAgainFlag(0);
        /*
         * 保存 订单商品
         */
        submitMapper.insertOrderProduct(rechargeProduct);
        log.info("【提交订单】>>>>>>>>>>>>>>>>>>>>>>>>>>保存订单商品(insert[cm_order_product])OrderProductId：" + rechargeProduct.getOrderProductId() + "，orderId：" + mainOrder.getOrderId());

        log.info("******************** 提交充值订单逻辑处理 end *******************");

        /*
         * 构造返回参数
         */
        Map<String, Object> info = new HashMap<>();
        info.put("orderId", String.valueOf(mainOrder.getOrderId()));
        info.put("orderNo", String.valueOf(mainOrder.getOrderNo()));
        info.put("orderMark", "#" + mainOrder.getOrderId() + "#");
        //应付订单金额
        info.put("payTotalFee", String.valueOf(mainOrder.getPayTotalFee()));
        //真实需要付款金额
        info.put("payableAmount", String.valueOf(mainOrder.getPayableAmount()));
        return ResponseJson.success(info);
    }

}
